; Mersenne Twister random number generator
;   Converted to NGPC by Jeff Siebold, AKA: NoonKnight (noonknight@mail.com)
;   Based on code by Makoto Matsumoto, Takuji Nishimura, Shawn Cokus and
;   Richard J. Wagner

; The Mersenne Twister is an algorithm for generating random numbers.  It
; was designed with consideration of the flaws in various other generators.
; The period, 2^19937-1, and the order of equidistribution, 623 dimensions,
; are far greater.  The generator is also fast; it avoids multiplication and
; division, and it benefits from caches and pipelines.  For more information
; see the inventors' web page at http://www.math.keio.ac.jp/~matumoto/emt.html

; Reference
; M. Matsumoto and T. Nishimura, "Mersenne Twister: A 623-Dimensionally
; Equidistributed Uniform Pseudo-Random Number Generator", ACM Transactions on
; Modeling and Computer Simulation, Vol. 8, No. 1, January 1998, pp 3-30.

; The original code included the following notices:
;
;     Copyright (C) 1997, 1999 Makoto Matsumoto and Takuji Nishimura.
;     Copyright (C) 2000  Richard J. Wagner
;     When you use this, send an email to: matumoto@math.keio.ac.jp
;     with an appropriate reference to your work.
;     It would be nice to CC: noonknight@mail.com, rjwagner@writeme.com and
;     Cokus@math.washington.edu when you write.

; FYI: tab spacing used: 4

	cpu 93C141 ; Select Toshiba TLCS-900/H cpu
	maxmode on

	include "hardware.inc"  ; Include hardware defines
	include "system.inc"    ; Include System Call defines

;
; Equates
;
RAND_N equ 624			; length of state vector
RAND_M equ 397			; a period parameter
RAND_K equ 9908B0DFh	; a magic constant

;
; Define variables
;
NEXT_RAM_OFS EVAL _MAINRAM

	RAMDD pRandNext ; pointer to the next number to take
	RAMDW nRandLeft ; numbers left before we need to reload
	RAMDD lRandHash ; guarantee time-based seeds will change
	RAMDDTBL lRandState, RAND_N
	RAMDBTBL cTimeArray, 7

;
; Standard cartridge header
;
    org 200000h
	DB " LICENSED BY SNK CORPORATION"
	DD Main				; Start location
	DW 0				; Cart ID Code (Catalog number?)
	DB 0				; Cart Version
	DB 10h				; System code (Monochrome = 0x00 / Color = 0x10)
;       123456789012
	DB "MTRand      "	; Cart title (12 characters)
	DD 0,0,0,0			; Reserved
;
; Start execution here.
;
Main:
	calr MainInit

	; Use the prime number 4357 for verification with Mersenne Twister.
	ld xwa, 4357
	calr SeedRandom

;	calr Randomize

; qwa = 16-bit count
; w = unused
; a = y scroll (rS1SOY)
	xor xwa, xwa
; iy = foreground index (pointer to current line)
	ld iy, SCRN_Y_B*SCRN_VX_B ; Line just beyond the users view.
;	ld iy, (SCRN_Y_B-1)*SCRN_VX_B ; Bottom line visable to the user. (debug)
MainLoop:
	ld xde, _SCR1RAM
	add de, iy
	ld xhl, strRandom
	ld xbc, strRandomLen/2
	ldirw
	ld xix, xde
	add iy, SCRN_VX_B ; Move down one line.
	and iy, _SCR2RAM-_SCR1RAM-1 ; Stay within the foreground.

	calr GetRandom ; xbc = 32-bit random number

	ld d, 8 ; Convert 8 nibbles (32 bits) to 8 bytes.
	calr BinToHex

	sub xix, 4

	ld d, 4 ; Convert 4 nibbles (16 bits) to 4 bytes.
	xor xbc, xbc
	ld bc, qwa
	inc qwa

	calr BinToHex

	calr ScrollLine

;
; User shutdown? (Power off pressed?)
;
	cp (rUSERS), 0
	jr z, MainLoop      ; no
;
; Power off NGP
;
	ld rw3, VECT_SHUTDOWN
	calr SYSTEM_CALL

	jr $ ; Loop forever.

ScrollLine:
	push c
	ld c, 8
slLoop:
	inc a
	ld (rS1SOY), a
	bit PADB_A, (rSL)
	jr nz, sl2
sl1:
	bit 6, (r2DSR)	; Within vertical blank?
	jr nz, sl1		; Yes, then wait for it to finish.
sl2:
	bit 6, (r2DSR)	; Within vertical blank?
	jr z, sl2		; No, then wait for it to start.
	djnz c, slLoop
	pop c
	ret

MainInit:
;
; Set the X & Y window size to zero.
;
	xor a, a
	ld (rWSIX), a
	ld (rWSIY), a
;
; Install the SNK BOIS patch.
;
	calr SYS_PATCH
;
; Shutdown on condition of no use.
;
	set ANSWERB_TIME, (rUSERA)
;
; Initialize the 18 interrupt vectors to reti.
;
	ld xwa, RetInt
	ld xde, rSWI3
	ld b, 18
mi1:
	ld (xde+), xwa
	djnz b, mi1
;
; Install the Vertical Blank interrupt vector.
;
	ld xwa, VbInt
	ld (rVBI), xwa
;
; Clear Sprite Attribute Memory
;
	xor wa, wa
	ld b, 128
	ld xix, _SPRRAM
mi2:
	ld (xix+), wa
	djnz b, mi2
;
; Load the system font tileset.
;
	ld ra3, 2 + (1<<4) ; fgclr + (bgclr<<4)
	ld rw3, VECT_SYSFONTSET
	calr SYSTEM_CALL
;
; Load the foreground color palettes.
;
	ld xhl, ColorPal
	ld xde, _SCR1PAL
	ld xbc, (ColorPalEnd - ColorPal)/2
	ldirw
;
; Load the background color palettes.
;
	ld xhl, ColorPal
	ld xde, _SCR2PAL
	ld xbc, (ColorPalEnd - ColorPal)/2
	ldirw
;
; Clear the foreground & background.
;
	ld xhl, _SCR1RAM
	ldw (xhl), ' ' ; 0020h
	ld xde, xhl
	inc xde
	inc xde
	ld bc, 800h-1
	ldirw
;
; Enable all interrupts.
;
	ei 0
;
; Restore the X & Y window size.
;
	ldb (rWSIX), SCRN_X
	ldb (rWSIY), SCRN_Y

	ret

VbInt:
    ldb (rWDCR), WD_CLR ; Clear the Watch Dog Timer to prevent CPU reset.
RetInt:
	reti

Randomize:
	ld xhl3, cTimeArray
	ldb  rw3, VECT_RTCGET
	calr SYSTEM_CALL

	xor xbc, xbc ; h1
	xor xde, xde
	ld xhl, cTimeArray

	ld c, (xhl+) ; cTimeArray[0]
	mul xbc, 257
	ld e, (xhl+) ; cTimeArray[1]
	add xbc, xde
	mul xbc, 257
	ld e, (xhl+) ; cTimeArray[2]
	add xbc, xde
	mul xbc, 257
	ld e, (xhl+) ; cTimeArray[3]
	add xbc, xde

	xor xwa, xwa ; h2
	ld a, (xhl+) ; cTimeArray[4]
	mul xwa, 257
	ld e, (xhl+) ; cTimeArray[5]
	add xwa, xde
	mul xwa, 257
	ld e, (xhl+) ; cTimeArray[6]
	add xwa, xde
	ld xde, (lRandHash)
	inc xde
	ld (lRandHash), xde
	add xwa, xde
	xor xwa, xbc
;	calr SeedRandom
;	ret

SeedRandom:
; Inputs: xwa = seed
	ld bc, RAND_N
	ld xhl, lRandState
sr1:
	; (xwa * 69069 + 1) = (xwa * 17267 * 4 + xwa + 1)
	ld xde, xwa
	xor xix, xix
	ld ix, qwa
	mul xix, 17267
	ld qix, ix
	xor ix, ix
	mul xwa, 17267
	add xwa, xix
	sll 2, xwa
	add xwa, xde
	inc xwa

	ld iy, qde
	ld qiy, iy
	ld iy, qwa
	ld (xhl+), xiy

	; (xwa * 69069 + 1) = (xwa * 17267 * 4 + xwa + 1)
	ld xde, xwa
	xor xix, xix
	ld ix, qwa
	mul xix, 17267
	ld qix, ix
	xor ix, ix
	mul xwa, 17267
	add xwa, xix
	sll 2, xwa
	add xwa, xde
	inc xwa

	djnz bc, sr1

ReloadRandom:
	push xwa
;	push xbc
	push xde
;	push xhl
	push xix
	push xiy
	push xiz

	; left = N - 1
	ldw (nRandLeft), RAND_N

	; p0 = &state[0];
	ld xix, lRandState

	; next = &state[0]
	ld (pRandNext), xix

	; pM = &state[M];
	ld xiz, xix
	add xiz, RAND_M*4

	; p2 = &state[0]; // soon to be &state[2]
	ld xiy, xix

	; s0 = *p2++; // state[0]
	ld xwa, (xiy+) ; s0 = state[0] & p2 = &state[1]

	; s1 = *p2++; // state[1]
	ld xbc, (xiy+) ; s1 = state[1] & p2 = &state[2]

	; for(j = N - M; j--; s0 = s1, s1 = *p2++)
	ld de, RAND_N-RAND_M
rr1:
	;	  *p0++ = *pM++ ^ (mixBits(s0, s1) >> 1) ^ (loBit(s1) ? K : 0U);
	and xwa, 80000000h
	ld xhl, xbc
	res 15, qhl ; and xhl, 7fffffffh | and qhl, 7fffh
	or xwa, xhl
	srl 1, xwa
	bit 0, c
	jr z, rr2
	xor xwa, RAND_K
rr2:
	xor xwa, (xiz+) ; xwa ^= *pM++
	ld (xix+), xwa ; *p0++ = xwa
	ld xwa, xbc ; s0 = s1
	ld xbc, (xiy+) ; s1 = *p2++
	djnz de, rr1

	; for(pM = &state[0], j = M - 1; j--; s0 = s1, s1 = *p2++)
	ld xiz, lRandState
	ld de, RAND_M-1
rr3:
	; *p0++ = *pM++ ^ (mixBits(s0, s1) >> 1) ^ (loBit(s1) ? K : 0U);
	and xwa, 80000000h
	ld xhl, xbc
	res 15, qhl ; and xhl, 7fffffffh | and qhl, 7fffh
	or xwa, xhl
	srl 1, xwa
	bit 0, c
	jr z, rr4
	xor xwa, RAND_K
rr4:
	xor xwa, (xiz+) ; xwa ^= *pM++
	ld (xix+), xwa ; *p0++ = xwa
	ld xwa, xbc ; s0 = s1
	ld xbc, (xiy+) ; s1 = *p2++
	djnz de, rr3

	ld xbc, (lRandState) ; s1 = state[0]
	; *p0 = *pM ^ (mixBits(s0, s1) >> 1) ^ (loBit(s1) ? K : 0U);
	and xwa, 80000000h
	ld xhl, xbc
	and xhl, 7fffffffh
	or xwa, xhl
	srl 1, xwa
	bit 0, c
	jr z, rr5
	xor xwa, RAND_K
rr5:
	xor xwa, (xiz) ; xwa ^= *pM++
	ld (xix), xwa ; *p0++ = xwa

	pop xiz
	pop xiy
	pop xix
;	pop xhl
	pop xde
;	pop xbc
	pop xwa
	ret

; returns xbc = a 32-bit random number.
GetRandom:
	push xhl

	ld bc, (nRandLeft)
	or bc, bc
	jr nz, gr0

	calr ReloadRandom
	ld bc, (nRandLeft) ; Load the new amount left.
gr0:
	dec bc
	ld (nRandLeft), bc

	ld xhl, (pRandNext)
	ld xbc, (xhl+)
	ld (pRandNext), xhl
gr1:
	; s1 ^= (s1 >> 11)
	ld xhl, xbc
	srl 11, xhl
	xor xbc, xhl

	; s1 ^= (s1 <<  7) & 0x9D2C5680U
	ld xhl, xbc
	sll 7, xhl
	and xhl, 9D2C5680h
	xor xbc, xhl

	; s1 ^= (s1 << 15) & 0xEFC60000U
	ld xhl, xbc
	sll 15, xhl
	and xhl, 0EFC60000h
	xor xbc, xhl

	; return(s1 ^ (s1 >> 18));
	ld xhl, xbc
	srl 15, xhl
	srl 3, xhl
	xor xbc, xhl

	pop xhl
	ret

BinToHex:
; Convert D nibbles of XBC from binary to ASCII hexadecimal & store at XIX.
	push wa
    ld xiz, BinToHexTable
bth1:
	ld a, c
	and a, 0fh
	srl 4, xbc
	ld a, (xiz+a)
	xor w, w ; title attribute, ie. color
	dec xix
	dec xix
	ld (xix), wa
	djnz d, bth1
	pop wa
	ret

BinToHexTable:
	DB "0123456789ABCDEF"

strRandom:
	DB "_", 0
	DB "_", 0
	DB "_", 0
	DB "_", 0
	DB ":", 0
	DB " ", 0
	DB "_", 0
	DB "_", 0
	DB "_", 0
	DB "_", 0
	DB "_", 0
	DB "_", 0
	DB "_", 0
	DB "_", 0
strRandomLen equ ($-strRandom)

ColorPal:
	db		0, 0 ; Always transparent?
	RGBSet	0f0h,0f0h,0f0h ; White
	RGBSet	000h,000h,000h ; Black
	RGBSet	000h,000h,0f0h ; Blue
ColorPalEnd:

SYSTEM_CALL:
        SystemCallCode
SYS_PATCH:
        SysPatchCode

; Make ROM 4mbits in length to satisfy NGPC flash cart software
        org     27ffffh
        DB      0ffh

        END
