;**************************************************************************************************
;*
;* Answer.inc: User Answer register manipulation macros for the NGPC.
;*
;* Author    : Steve Robb
;*
;**************************************************************************************************

	IfNDef ANSWER_INC
ANSWER_INC Equ TRUE


;***************************
;* Memory location defines *
;***************************

AUserAnswer_Register Equ 06F86h


;*****************
;* Public macros *
;*****************

;**************************************************************************************************
;*
;* Macro       - UserAnswer_IsBit5Enabled
;* Parameters  - void
;* Returns     - "true" if bit 5 is enabled, "false" otherwise.
;* Description - Tests if bit 5 is enabled.
;* Author      - Steve Robb
;*
;**************************************************************************************************

UserAnswer_IsBit5Enabled: Macro {NoExpand}

	;/* Test bit 5 of User Answer register */
	bit 5, (AUserAnswer_Register)

	EndM

;***********************************
;* End of UserAnswer_IsBit5Enabled *
;***********************************


;**************************************************************************************************
;*
;* Macro       - UserAnswer_EnableBit5
;* Parameters  - void
;* Returns     - void
;* Description - Enables bit 5 of User Answer.
;* Author      - Steve Robb
;*
;**************************************************************************************************

UserAnswer_EnableBit5: Macro {NoExpand}

	;/* Set bit 5 of User Answer register */
	set 5, (AUserAnswer_Register)

	EndM

;********************************
;* End of UserAnswer_EnableBit5 *
;********************************


;**************************************************************************************************
;*
;* Macro       - UserAnswer_DisableBit5
;* Parameters  - void
;* Returns     - void
;* Description - Disables bit 5 of User Answer.
;* Author      - Steve Robb
;*
;**************************************************************************************************

UserAnswer_DisableBit5: Macro {NoExpand}

	;/* Reset bit 5 of User Answer register */
	res 5, (AUserAnswer_Register)

	EndM

;*********************************
;* End of UserAnswer_DisableBit5 *
;*********************************


;**************************************************************************************************
;*
;* Macro       - UserAnswer_IsInactiveAutoShutdownEnabled
;* Parameters  - void
;* Returns     - "true" if automatic shutdown is enabled, "false" otherwise.
;* Description - Tests if the automatic shutdown due to prolonged activity is enabled.
;* Author      - Steve Robb
;*
;**************************************************************************************************

UserAnswer_IsInactiveAutoShutdownEnabled: Macro {NoExpand}

	;/* Test bit 6 of User Answer register */
	bit 6, (AUserAnswer_Register)

	EndM

;************************************************
;* End of UserAnswer_EnableInactiveAutoShutdown *
;************************************************


;**************************************************************************************************
;*
;* Macro       - UserAnswer_EnableInactiveAutoShutdown
;* Parameters  - void
;* Returns     - void
;* Description - Enables the automatic shutdown due to prolonged activity.
;* Author      - Steve Robb
;*
;**************************************************************************************************

UserAnswer_EnableInactiveAutoShutdown: Macro {NoExpand}

	;/* Set bit 6 of User Answer register */
	set 6, (AUserAnswer_Register)

	EndM

;************************************************
;* End of UserAnswer_EnableInactiveAutoShutdown *
;************************************************


;**************************************************************************************************
;*
;* Macro       - UserAnswer_DisableInactiveAutoShutdown
;* Parameters  - void
;* Returns     - void
;* Description - Disables the automatic shutdown due to prolonged activity.
;* Author      - Steve Robb
;*
;**************************************************************************************************

UserAnswer_DisableInactiveAutoShutdown: Macro {NoExpand}

	;/* Reset bit 6 of User Answer register */
	res 6, (AUserAnswer_Register)

	EndM

;*************************************************
;* End of UserAnswer_DisableInactiveAutoShutdown *
;*************************************************


;**************************************************************************************************
;*
;* Macro       - UserAnswer_IsResumeEnabled
;* Parameters  - void
;* Returns     - "true" if resuming is enabled, "false" otherwise.
;* Description - Tests if resuming is enabled.
;* Author      - Steve Robb
;*
;**************************************************************************************************

UserAnswer_IsResumeEnabled: Macro {NoExpand}

	;/* Test bit 7 of User Answer register */
	bit 7, (AUserAnswer_Register)

	EndM

;*************************************
;* End of UserAnswer_IsResumeEnabled *
;*************************************


;**************************************************************************************************
;*
;* Macro       - UserAnswer_EnableResume
;* Parameters  - void
;* Returns     - void
;* Description - Enables resuming on next start-up.
;* Author      - Steve Robb
;*
;**************************************************************************************************

UserAnswer_EnableResume: Macro {NoExpand}

	;/* Set bit 7 of User Answer register */
	set 7, (AUserAnswer_Register)

	EndM

;**********************************
;* End of UserAnswer_EnableResume *
;**********************************


;**************************************************************************************************
;*
;* Macro       - UserAnswer_DisableResume
;* Parameters  - void
;* Returns     - void
;* Description - Disables resuming on next start-up.
;* Author      - Steve Robb
;*
;**************************************************************************************************

UserAnswer_DisableResume: Macro {NoExpand}

	;/* Reset bit 7 of User Answer register */
	res 7, (AUserAnswer_Register)

	EndM

;***********************************
;* End of UserAnswer_DisableResume *
;***********************************


	EndIf ;// IfNDef ANSWER_INC

;*********************
;* End of Answer.inc *
;*********************
