;**************************************************************************************************
;*
;* Assert.inc: Assertion macros for the NGPC.
;*
;* Author    : Steve Robb
;*
;**************************************************************************************************

	IfNDef ASSERT_INC
ASSERT_INC Equ TRUE


;********************
;* Public functions *
;********************

;**************************************************************************************************
;*
;* Function    - FInRange
;* Parameters  - nValue: The value to test for being in the range.
;*               nMin  : The minimum value of the range.
;*               nMax  : The maximum value of the range.
;* Returns     - "true" if the value is in the range, "false" otherwise.
;* Description - Tests if a value is within a specified range.
;* Author      - Steve Robb
;*
;**************************************************************************************************

FInRange Function nValue, nMin, nMax, ((nValue) >= (nMin)) && ((nValue) <= (nMax))

;*******************
;* End of FInRange *
;*******************


;**************************************************************************************************
;*
;* Function    - FIsAColour
;* Parameters  - nValue: The value to test for being a colour.
;* Returns     - "true" if the value is a colour, "false" otherwise.
;* Description - Tests if a value is a valid colour.
;* Author      - Steve Robb
;*
;**************************************************************************************************

FIsAColour Function nValue, ((nValue) & 00FFFh == (nValue))

;*********************
;* End of FIsAColour *
;*********************


;**************************************************************************************************
;*
;* Function    - FIsNBitRegister
;* Parameters  - strParameter: The string containing the text to test.
;* Returns     - "true" if the string contains a register of the specified bit-depth, "false" otherwise.
;* Description - Tests a string for containing a N-bit register.
;* Author      - Steve Robb
;*
;**************************************************************************************************

FIs8BitRegister_1  Function strParameter, (UpString(strParameter) == "W"  ) || (UpString(strParameter) == "A"  ) || (UpString(strParameter) == "B"  ) || (UpString(strParameter) == "C"  )
FIs8BitRegister_2  Function strParameter, (UpString(strParameter) == "D"  ) || (UpString(strParameter) == "E"  ) || (UpString(strParameter) == "H"  ) || (UpString(strParameter) == "L"  )
FIs16BitRegister_1 Function strParameter, (UpString(strParameter) == "WA" ) || (UpString(strParameter) == "BC" ) || (UpString(strParameter) == "DE" ) || (UpString(strParameter) == "HL" )
FIs16BitRegister_2 Function strParameter, (UpString(strParameter) == "IX" ) || (UpString(strParameter) == "IY" ) || (UpString(strParameter) == "IZ" ) || (UpString(strParameter) == "SP" )
FIs32BitRegister_1 Function strParameter, (UpString(strParameter) == "XWA") || (UpString(strParameter) == "XBC") || (UpString(strParameter) == "XDE") || (UpString(strParameter) == "XHL")
FIs32BitRegister_2 Function strParameter, (UpString(strParameter) == "XIX") || (UpString(strParameter) == "XIY") || (UpString(strParameter) == "XIZ") || (UpString(strParameter) == "XSP")

FIs8BitRegister  Function strParameter, FIs8BitRegister_1(strParameter)  || FIs8BitRegister_2(strParameter)
FIs16BitRegister Function strParameter, FIs16BitRegister_1(strParameter) || FIs16BitRegister_2(strParameter)
FIs32BitRegister Function strParameter, FIs32BitRegister_1(strParameter) || FIs32BitRegister_2(strParameter)

;**************************
;* End of FIsNBitRegister *
;**************************


;*****************
;* Public macros *
;*****************

;**************************************************************************************************
;*
;* Macro       - StaticAssert
;* Parameters  - bExpression: The expression to test.
;*               strString  : The error message to display.
;* Returns     - void
;* Description - Causes a compiler error if the supplied expression is false.
;* Author      - Steve Robb
;*
;**************************************************************************************************

StaticAssert: Macro bExpression, strString, {NoExpand}

	;/* Check if the expression is blank */
	IfB bExpression

		;/* Display error message */
		Error "Unable to test assertion - expression is blank"

	EndIf

	;/* Check if the expression is false */
	If ~~(bExpression)

		;/* Check if an error message was supplied */
		IfNB strString

			;/* Display user error message */
			Error strString

		;/* An error message was not supplied */
		Else

			;/* Display default error message */
			Error "Static assert failed"

		EndIf

	EndIf

	EndM

;***********************
;* End of StaticAssert *
;***********************


;**************************************************************************************************
;*
;* Macro       - StaticAssert_NotBlank
;* Parameters  - parameter: The parameter to test for blankness.
;*               strString: The error message to display.
;* Returns     - void
;* Description - Causes a compiler error if the supplied parameter is blank.
;* Author      - Steve Robb
;*
;**************************************************************************************************

StaticAssert_NotBlank: Macro parameter, strString, {NoExpand}

	;/* Check if parameter is blank */
	IfB parameter

		;/* Check if an error message was supplied */
		IfNB strString

			;/* Display user error message */
			Error strString

		;/* An error message was not supplied */
		Else

			;/* Display default error message */
			Error "Static assert failed - parameter is blank"

		EndIf

	EndIf

	EndM

;********************************
;* End of StaticAssert_NotBlank *
;********************************


	EndIf ;// IfNDef ASSERT_INC

;*********************
;* End of Assert.inc *
;*********************
