;**************************************************************************************************
;*
;* Backgrnd.inc: Background manipulation macros for the NGPC.
;*
;* Author      : Steve Robb
;*
;**************************************************************************************************

	IfNDef BACKGRND_INC
BACKGRND_INC Equ TRUE


;***************************
;* Memory location defines *
;***************************

ABackground_Control  Equ 08118h ;// Background control register
ABackground_Palettes Equ 083E0h ;// Start of background colour palettes


;********************
;* Public functions *
;********************

;**************************************************************************************************
;*
;* Function    - FBackground_PaletteEntryAddress
;* Parameters  - nIndex: The index of the palette entry to return (0-7).
;* Returns     - The address of the specified background palette entry.
;* Description - Returns the address of a background palette entry.
;* Author      - Steve Robb
;*
;**************************************************************************************************

FBackground_PaletteEntryAddress Function nIndex, ABackground_Palettes + ((nIndex) * 2)

;******************************************
;* End of FBackground_PaletteEntryAddress *
;******************************************


;*****************
;* Public macros *
;*****************

;**************************************************************************************************
;*
;* Macro       - Background_Enable
;* Parameters  - void
;* Returns     - void
;* Description - Enables the background.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Background_Enable: Macro {NoExpand}

	;/* Enable background */
	andb (ABackground_Control), 00111111b
	orb  (ABackground_Control), 10000000b

	EndM

;****************************
;* End of Background_Enable *
;****************************


;**************************************************************************************************
;*
;* Macro       - Background_Disable
;* Parameters  - void
;* Returns     - void
;* Description - Disables the background.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Background_Disable: Macro {NoExpand}

	;/* Disable background */
	andb (ABackground_Control), 00111111b

	EndM

;*****************************
;* End of Background_Disable *
;*****************************


;**************************************************************************************************
;*
;* Macro       - Background_SetPaletteEntry
;* Parameters  - nIndex : The index of the palette entry to set.
;*               nColour: The colour to set the palette entry to.
;* Returns     - void
;* Description - Sets the colour of a background palette entry.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Background_SetPaletteEntry: Macro nIndex, nColour, {NoExpand}

	;/* Validate parameters */
	StaticAssert_NotBlank                   nIndex, "Syntax: Background_SetPaletteEntry <index:0-7>, <colour>"
	StaticAssert_NotBlank                  nColour, "Syntax: Background_SetPaletteEntry <index:0-7>, <colour>"
	StaticAssert          FInRange  (nIndex, 0, 7), "Background_SetPaletteEntry: index must be in the range 0-7"
	StaticAssert          FIsAColour(nColour     ), "Background_SetPaletteEntry: colour is invalid"

	;/* Set palette entry */
	ldw (FBackground_PaletteEntryAddress(nIndex)), nColour

	EndM

;*************************************
;* End of Background_SetPaletteEntry *
;*************************************


;**************************************************************************************************
;*
;* Macro       - Background_SetPaletteIndex
;* Parameters  - nIndex: The index of the palette entry to use.
;* Returns     - void
;* Description - Sets the index of the palette to be used for the background.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Background_SetPaletteIndex: Macro nIndex, {NoExpand}

	;/* Validate parameters */
	StaticAssert_NotBlank                 nIndex, "Syntax: Background_SetPaletteIndex <index:0-7>"
	StaticAssert          FInRange(nIndex, 0, 7), "Background_SetPaletteIndex: index must be in the range 0-7"

	;/* Check if any bits of index are not set */
	If nIndex <> 7

		;/* Clear bottom 3 bits of background control register */
		andb (ABackground_Control), 11111000b
		
	EndIf

	;/* Write index to bottom 3 bits of background control register */
	orb (ABackground_Control), nIndex

	EndM

;*************************************
;* End of Background_SetPaletteIndex *
;*************************************


	EndIf ;// IfNDef BACKGRND_INC

;***********************
;* End of Backgrnd.inc *
;***********************
