;**************************************************************************************************
;*
;* Interrupt   - Fade_PaletteEntry_ToBlack
;* Parameters  - pPaletteEntry: The address of the palette entry to fade.
;* Returns     - rFade        : The register to set if the colour has been changed.
;* Description - Fades a palette word one step closer to black.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Fade_PaletteEntry_ToBlack: Macro pPaletteEntry, rFade, {NoExpand}

	;/* Get palette value from address */
	ldw hl, (pPaletteEntry)

	;/* Separate colour components into registers */
	andb h,  00Fh
	ldb  a,     l
	andb l,  0F0h
	andb a,  00Fh

	;/* Decrement red value if not already at minimum */
	cpb   a,         0
	jr   eq, NoFadeRed
	decb  a

	;/* Set fade flag */
	ldb rFade, 1
NoFadeRed:

	;/* Decrement green value if not already at minimum */
	cpb   l,           0
	jr   eq, NoFadeGreen
	subb  l,        010h

	;/* Set fade flag */
	ldb rFade, 1
NoFadeGreen:

	;/* Decrement blue value if not already at minimum */
	cpb   h,          0
	jr   eq, NoFadeBlue
	decb  h

	;/* Set fade flag */
	ldb rFade, 1
NoFadeBlue:

	;/* Write colour back to address */
	orb                l,  a
	ldw  (pPaletteEntry), hl

	EndM

;************************************
;* End of Fade_PaletteEntry_ToBlack *
;************************************


;**************************************************************************************************
;*
;* Interrupt   - Fade_PaletteEntry_ToWhite
;* Parameters  - pPaletteEntry: The address of the palette entry to fade.
;* Returns     - rFade        : The register to set if the colour has been changed.
;* Description - Fades a palette word one step closer to white.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Fade_PaletteEntry_ToWhite: Macro pPaletteEntry, rFade, {NoExpand}

	;/* Get palette value from address */
	ldw hl, (pPaletteEntry)

	;/* Separate colour components into registers */
	andb h,  00Fh
	ldb  a,     l
	andb l,  0F0h
	andb a,  00Fh

	;/* Increment red value if not already at maximum */
	cpb   a,      00Fh
	jr   eq, NoFadeRed
	incb  a

	;/* Set fade flag */
	ldb rFade, 1
NoFadeRed:

	;/* Increment green value if not already at maximum */
	cpb   l,        0F0h
	jr   eq, NoFadeGreen
	addb  l,        010h

	;/* Set fade flag */
	ldb rFade, 1
NoFadeGreen:

	;/* Increment blue value if not already at maximum */
	cpb   h,       00Fh
	jr   eq, NoFadeBlue
	incb  h

	;/* Set fade flag */
	ldb rFade, 1
NoFadeBlue:

	;/* Write colour back to address */
	orb                l,  a
	ldw  (pPaletteEntry), hl

	EndM

;*****************************
;* Fade_PaletteEntry_ToWhite *
;*****************************


Fade_Scroll_ToBlack: MACRO pFinished, pScrollPalette, {NoExpand}

	ldw  bc,       (16 * 4) - 1
	ld  xde, pScrollPalette + 2
	ldb   w,                  0

Again:
	ldw   hl,         bc
	andw  hl,       003h
	jr     z, NotAColour

	Fade_PaletteEntry_ToBlack xde, w

NotAColour:
	inc   2,   xde
	djnz bc, Again

	ifnb pFinished

		cpb  w,         0
		jp  eq, pFinished

	endif

	ENDM

;******************************
;* End of Fade_Scroll_ToBlack *
;******************************

Fade_ToBlack: Macro pFinished, {NoExpand}

	;/* Reset fade flag */
	ldb w, 0

	;/**/
	ldw  bc,            (16 * 4) - 1
	ld  xix, AScroll1_PaletteRAM + 2
	ld  xiy, AScroll2_PaletteRAM + 2

Again:
	ldw   hl,         bc
	andw  hl,       003h
	jp     z, NotAColour

	Fade_PaletteEntry_ToBlack xix, w
	Fade_PaletteEntry_ToBlack xiy, w
NotAColour:
	inc   2,   xix
	inc   2,   xiy
	djnz bc, Again

	Fade_PaletteEntry_ToBlack FBackground_PaletteEntryAddress(0), w
	Fade_PaletteEntry_ToBlack FBackground_PaletteEntryAddress(1), w
	Fade_PaletteEntry_ToBlack FBackground_PaletteEntryAddress(2), w

	IfNB pFinished

		cpb  w,         0
		jp  eq, pFinished

	EndIf

	ENDM

Fade_ToWhite: Macro pFinished, {NoExpand}

	;/* Reset fade flag */
	ldb w, 0

	;/**/
	ldw  bc,            (16 * 4) - 1
	ld  xix, AScroll1_PaletteRAM + 2
	ld  xiy, AScroll2_PaletteRAM + 2

Again:
	ldw   hl,         bc
	andw  hl,       003h
	jp     z, NotAColour

	Fade_PaletteEntry_ToWhite xix, w
	Fade_PaletteEntry_ToWhite xiy, w
NotAColour:
	inc   2,   xix
	inc   2,   xiy
	djnz bc, Again

	Fade_PaletteEntry_ToWhite FBackground_PaletteEntryAddress(0), w
	Fade_PaletteEntry_ToWhite FBackground_PaletteEntryAddress(1), w
	Fade_PaletteEntry_ToWhite FBackground_PaletteEntryAddress(2), w

	IfNB pFinished

		cpb  w,         0
		jp  eq, pFinished

	EndIf

	EndM