;**************************************************************************************************
;*
;* FontCmn.inc: Common font handling routines for the NGPC.
;*
;* Author     : Steve Robb
;*
;**************************************************************************************************

	IfNDef FONTCMN_INC
FONTCMN_INC Equ TRUE


;*****************
;* Include files *
;*****************

	Include "Assert.inc"


;*****************
;* Public macros *
;*****************

;**************************************************************************************************
;*
;* Macro       - FontCmn_EnumerateString
;* Parameters  - strString : The string to be enumerated.
;*               strFont   : The font string.
;*               BYTEORWORD: "byte" if bytes are to be output, "word" if words are to be output.
;*               nOffset   : The offset to apply to each enumerated value (optional).
;*               nSpace    : The value to replace space with (optional).
;* Returns     - void
;* Description - Converts a string into an indexed buffer.
;* Author      - Steve Robb
;*
;**************************************************************************************************

FontCmn_EnumerateString: Macro strString, strFont, BYTEORWORD, nOffset, nSpace, {NoExpand}

FontCmn_EnumerateString_bByte Eval UpString("BYTEORWORD") == "BYTE"

	StaticAssert (FontCmn_EnumerateString_bByte) || (UpString("BYTEORWORD") == "WORD"), "FontCmn_EnumerateString: format must be \Ibyte\I or \Iword\I"

	IfNB nOffset

FontCmn_String_nOffset Eval nOffset

	Else

FontCmn_String_nOffset Eval 0

	EndIf

FontCmn_String_nChar Eval 0

	Rept StrLen(strString)

FontCmn_String_bNormalChar Eval TRUE

FontCmn_String_strChar Eval SubStr(strString, FontCmn_String_nChar, 1)

		IfNB nSpace

			If (FontCmn_String_strChar == " ")

				If FontCmn_EnumerateString_bByte

					DB nSpace

				Else

					DW nSpace

				EndIf

FontCmn_String_bNormalChar Eval FALSE

			EndIf

		EndIf

		If FontCmn_String_bNormalChar

FontCmn_String_nIndex Eval StrStr(strFont, FontCmn_String_strChar)

			If FontCmn_String_nIndex < 0

				Error "Character '" + FontCmn_String_strChar + "' not supported by font"

			EndIf

			If FontCmn_EnumerateString_bByte

				DB FontCmn_String_nIndex + FontCmn_String_nOffset

			Else

				DW FontCmn_String_nIndex + FontCmn_String_nOffset

			EndIf

		EndIf

FontCmn_String_nChar Eval FontCmn_String_nChar + 1

	EndM

	EndM

;**********************************
;* End of FontCmn_EnumerateString *
;**********************************


	EndIf ;// IfNDef FONTCMN_INC

;**********************
;* End of FontCmn.inc *
;**********************
