;**************************************************************************************************
;*
;* Input.inc: Input testing macros for the NGPC.
;*
;* Author   : Steve Robb
;*
;**************************************************************************************************

	IfNDef INPUT_INC
INPUT_INC Equ TRUE


;*****************
;* Include files *
;*****************

	Include "Assert.inc"


;***************************
;* Memory location defines *
;***************************

AInput_Register Equ 06F82h ;// Joystick and button control register


;******************
;* Private macros *
;******************

;**************************************************************************************************
;*
;* Macro       - Input_IsBitSet
;* Parameters  - nBit    : The bit to test.
;*               pAddress: The address containing the controls to test (optional).
;* Returns     - "true" if the bit is set, "false" otherwise.
;* Description - Tests a bit on the control register or an optional address holding control data.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Input_IsBitSet: Macro nBit, pAddress, {NoExpand}

	;/* Validate parameters */
	StaticAssert_NotBlank                 nBit, "Syntax: Input_IsBitSet <bit:0-7>[, <address>]"
	StaticAssert          FInRange(nBit, 0, 7), "Input_IsBitSet: bit must be in the range 0-7"

	;/* Check if an address was specified */
	IfNB pAddress

		;/* Test bit of address */
		bit nBit, pAddress

	;/* No address was specified */
	Else

		;/* Test bit of control register */
		bit nBit, (AInput_Register)

	EndIf

	EndM

;*************************
;* End of Input_IsBitSet *
;*************************


;*****************
;* Public macros *
;*****************

;**************************************************************************************************
;*
;* Macro       - Input_IsUpPressed
;* Parameters  - pAddress: The address containing the controls to test (optional).
;* Returns     - "true" if Up is pressed, "false" otherwise.
;* Description - Tests to see if Up is pressed on the control register or an optional address
;*               holding control data.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Input_IsUpPressed: Macro pAddress, {NoExpand}

	;/* Test bit 0 */
	Input_IsBitSet 0, pAddress

	EndM

;****************************
;* End of Input_IsUpPressed *
;****************************


;**************************************************************************************************
;*
;* Macro       - Input_IsDownPressed
;* Parameters  - pAddress: The address containing the controls to test (optional).
;* Returns     - "true" if Down is pressed, "false" otherwise.
;* Description - Tests to see if Down is pressed on the control register or an optional address
;*               holding control data.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Input_IsDownPressed: Macro pAddress, {NoExpand}

	;/* Test bit 1 */
	Input_IsBitSet 1, pAddress

	EndM

;******************************
;* End of Input_IsDownPressed *
;******************************


;**************************************************************************************************
;*
;* Macro       - Input_IsLeftPressed
;* Parameters  - pAddress: The address containing the controls to test (optional).
;* Returns     - "true" if Left is pressed, "false" otherwise.
;* Description - Tests to see if Left is pressed on the control register or an optional address
;*               holding control data.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Input_IsLeftPressed: Macro pAddress, {NoExpand}

	;/* Test bit 2 */
	Input_IsBitSet 2, pAddress

	EndM

;******************************
;* End of Input_IsLeftPressed *
;******************************


;**************************************************************************************************
;*
;* Macro       - Input_IsRightPressed
;* Parameters  - pAddress: The address containing the controls to test (optional).
;* Returns     - "true" if Right is pressed, "false" otherwise.
;* Description - Tests to see if Right is pressed on the control register or an optional address
;*               holding control data.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Input_IsRightPressed: Macro pAddress, {NoExpand}

	;/* Test bit 3 */
	Input_IsBitSet 3, pAddress

	EndM

;*******************************
;* End of Input_IsRightPressed *
;*******************************


;**************************************************************************************************
;*
;* Macro       - Input_IsAPressed
;* Parameters  - pAddress: The address containing the controls to test (optional).
;* Returns     - "true" if A is pressed, "false" otherwise.
;* Description - Tests to see if A is pressed on the control register or an optional address
;*               holding control data.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Input_IsAPressed: Macro pAddress, {NoExpand}

	;/* Test bit 4 */
	Input_IsBitSet 4, pAddress

	EndM

;***************************
;* End of Input_IsAPressed *
;***************************


;**************************************************************************************************
;*
;* Macro       - Input_IsBPressed
;* Parameters  - pAddress: The address containing the controls to test (optional).
;* Returns     - "true" if B is pressed, "false" otherwise.
;* Description - Tests to see if B is pressed on the control register or an optional address
;*               holding control data.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Input_IsBPressed: Macro pAddress, {NoExpand}

	;/* Test bit 5 */
	Input_IsBitSet 5, pAddress

	EndM

;***************************
;* End of Input_IsBPressed *
;***************************


;**************************************************************************************************
;*
;* Macro       - Input_IsOptionPressed
;* Parameters  - pAddress: The address containing the controls to test (optional).
;* Returns     - "true" if Option is pressed, "false" otherwise.
;* Description - Tests to see if Option is pressed on the control register or an optional address
;*               holding control data.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Input_IsOptionPressed: Macro pAddress, {NoExpand}

	;/* Test bit 6 */
	Input_IsBitSet 6, pAddress

	EndM

;********************************
;* End of Input_IsOptionPressed *
;********************************


	EndIf ;// IfNDef INPUT_INC

;********************
;* End of Input.inc *
;********************
