;**************************************************************************************************
;*
;* StHandlr.inc: State handler system for NGPC framework.
;*
;* Author      : Steve Robb
;*
;**************************************************************************************************

	IfNDef STHANDLR_INC
STHANDLR_INC Equ TRUE


;*****************
;* Include files *
;*****************

	Include "Framewrk.inc"
	Include "LinkUp.inc"
	Include "UserRAM.inc"
	Include "Watchdog.inc"


;**********
;* Macros *
;**********

State_U__: Macro name, {NoExpand}

name:
	DD name_Update
	DD Function_Unused
	DD Interrupt_Unused

	EndM


State__D_: Macro name, {NoExpand}

name:
	DD Function_Unused
	DD name_Display
	DD Interrupt_Unused

	EndM


State_UD_: Macro name, {NoExpand}

name:
	DD name_Update
	DD name_Display
	DD Interrupt_Unused

	EndM


State_U_H: Macro name, {NoExpand}

name:
	DD name_Update
	DD Function_Unused
	DD name_HInt

	EndM


State__DH: Macro name, {NoExpand}

name:
	DD Function_Unused
	DD name_Display
	DD name_HInt

	EndM


State_UDH: Macro name, {NoExpand}

name:
	DD name_Update
	DD name_Display
	DD name_HInt

	EndM


State_Advance: Macro {NoExpand}

	add (dNextState), 12

	EndM

State_Set: Macro next, {NoExpand}

	ld  (dNextState), next

	EndM


;******************
;* RAM allocation *
;******************

	UserRAM_DB bVSyncDone
	UserRAM_DD dState
	UserRAM_DD dNextState


;**************************************************************************************************
;*
;* Function    - main
;* Parameters  - none
;* Returns     - none
;* Description - The main framework entry point.
;* Author      - Steve Robb
;*
;**************************************************************************************************

main:

	;/* Reset to first state */
	ld          xde, States
	ld (dState    ),    xde
	ld (dNextState),    xde

	;/* Set vsync-done flag */
	ldb (bVSyncDone), 1

main_Loop:
	;/* Ensure user has not turned off machine */
	cp (rUSERS),             0
	jr       ne, main_ShutDown

	;/* Call state handler */
	ld    xde , (dState)
	ld    xde ,    (xde)
	call           (xde)

	;/* Clear vsync-done flag */
	ldb (bVSyncDone), 0

main_VSyncLoop:
	;/* Loop until vsync occurs */
	cpb (bVSyncDone),              0
	jr            eq, main_VSyncLoop

	;/* Set new state and continue looping */
	ld      xde, (dNextState)
	ld (dState),          xde
	ld      xde,      (xde+8)
	ld   (rTI0),          xde
	jr              main_Loop

main_ShutDown:
	;/* Turn off NGP */
	ld   rw3,VECT_SHUTDOWN
	call       SYSTEM_CALL

main_Done:
	;/* Loop indefinitely */
	jr main_Done

;***************
;* End of main *
;***************


;**************************************************************************************************
;*
;* Interrupt   - Interrupt_VSync
;* Parameters  - none
;* Returns     - none
;* Description - Called when a vsync interrupt occurs.
;* Author      - Steve Robb
;*
;**************************************************************************************************

		Warning "Must remove link-up dependency from state handler"

Interrupt_VSync:

	;/* Store rw3 register */
	push rw3

	;/* Disable serial transmissions */
	ldb  rw3, VECT_COMOFFRTS
	call         SYSTEM_CALL

	;/* Reset watchdog timer */
	Watchdog_Reset

	;/* Ensure loop has run */
	cpb (bVSyncDone),                   0
	jr            ne, Interrupt_VSync_End

	;/* Call state display handler */
	push           xde
	ld   xde, (dState)
	ld   xde,  (xde+4)
	call         (xde)
	pop            xde

	;/* Flag end of vsync */
	ldb (bVSyncDone), 1

	;/* Reenable serial transmissions */
	ldb  rw3, VECT_COMONRTS
	call        SYSTEM_CALL

Interrupt_VSync_End:
	;/* Restore rw3 register */
	pop rw3

	;/* Return from interrupt */
	reti

;**************************
;* End of Interrupt_VSync *
;**************************


;**************************************************************************************************
;*
;* Interrupt   - Interrupt_Timer0
;* Parameters  - none
;* Returns     - none
;* Description - Called when a timer 0 interrupt occurs.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Interrupt_Timer0:

	;/* Return from interrupt */
	reti

;***************************
;* End of Interrupt_Timer0 *
;***************************


	EndIf ;// IfNDef STHANDLR_INC

;***********************
;* End of StHandlr.inc *
;***********************
