;**************************************************************************************************
;*
;* UserRAM.inc: User RAM allocation macros for the NGPC.
;*
;* Author     : Steve Robb
;*
;**************************************************************************************************

	IfNDef USERRAM_INC
USERRAM_INC Equ TRUE


;*****************
;* Include files *
;*****************

	Include "Assert.inc"


;***************************
;* Memory location defines *
;***************************

AUserRAM_Start Equ 04000h ;// Start of user RAM
AUserRAM_End   Equ 05FFFh ;// End of user RAM


;*******************
;* Private defines *
;*******************

AIUserRAMAllocation Eval AUserRAM_Start ;// Start of unassigned RAM


;*****************
;* Public macros *
;*****************

;**************************************************************************************************
;*
;* Macro       - UserRAM_DS
;* Parameters  - label: The label to give to the block.
;*               nSize: The size of the block to reserve.
;* Returns     - void
;* Description - Reserves a block of RAM and assigns a label to its address.
;* Author      - Steve Robb
;*
;**************************************************************************************************

UserRAM_DS: Macro label, nSize, {NoExpand}

	;/* Validate parameters */
	StaticAssert_NotBlank       label, "Syntax: UserRAM_DS <label>, <size>"
	StaticAssert_NotBlank       nSize, "Syntax: UserRAM_DS <label>, <size>"
	StaticAssert          (nSize) > 0, "UserRAM_DS: size must be at least 1 byte"

;/* Assign label to current RAM allocation pointer */
label Equ AIUserRAMAllocation

;/* Increment RAM allocation pointer to next block */
AIUserRAMAllocation Eval AIUserRAMAllocation + nSize

	;/* Validate that RAM has not been completely allocated */
	StaticAssert AIUserRAMAllocation <= AUserRAM_End, "UserRAM: entire of RAM has been allocated"

	EndM

;*********************
;* End of UserRAM_DS *
;*********************


;**************************************************************************************************
;*
;* Macro       - UserRAM_DB
;* Parameters  - label: The label to give to the byte.
;* Returns     - void
;* Description - Reserves a byte of RAM and assigns a label to its address.
;* Author      - Steve Robb
;*
;**************************************************************************************************

UserRAM_DB: Macro label, {NoExpand}

	;/* Validate parameters */
	StaticAssert_NotBlank label, "Syntax: UserRAM_DB <label>"

	;/* Allocate byte of data */
	UserRAM_DS label, 1

	EndM

;*********************
;* End of UserRAM_DB *
;*********************


;**************************************************************************************************
;*
;* Macro       - UserRAM_DW
;* Parameters  - label: The label to give to the word.
;* Returns     - void
;* Description - Reserves a word of RAM and assigns a label to its address.
;* Author      - Steve Robb
;*
;**************************************************************************************************

UserRAM_DW: Macro label, {NoExpand}

	;/* Validate parameters */
	StaticAssert_NotBlank label, "Syntax: UserRAM_DW <label>"

	;/* Allocate word of data */
	UserRAM_DS label, 2

	EndM

;*********************
;* End of UserRAM_DW *
;*********************


;**************************************************************************************************
;*
;* Macro       - UserRAM_DD
;* Parameters  - label: The label to give to the double word.
;* Returns     - void
;* Description - Reserves a double word of RAM and assigns a label to its address.
;* Author      - Steve Robb
;*
;**************************************************************************************************

UserRAM_DD: Macro label, {NoExpand}

	;/* Validate parameters */
	StaticAssert_NotBlank label, "Syntax: UserRAM_DD <label>"

	;/* Allocate double word of data */
	UserRAM_DS label, 4

	EndM

;*********************
;* End of UserRAM_DD *
;*********************


	EndIf ;// IfNDef USERRAM_INC

;**********************
;* End of UserRAM.inc *
;**********************
