;**************************************************************************************************
;*
;* VFont.inc: Variable-sized font support for the NGPC.
;*
;* Author   : Steve Robb
;*
;**************************************************************************************************

	IfNDef VFONT_INC
VFONT_INC Equ TRUE


;*****************
;* Include files *
;*****************

	Include "Assert.inc"

	Include "FontCmn.inc"
	Include "Memory.inc"
	Include "Scroll.inc"


;*****************
;* Public macros *
;*****************

VFont_Initialise Macro

	;/* Clear tile RAM */
	Memory_Zero ATile_RAM, 16*20*24

	;/* Clear scroll plane 2 */
	Scroll_Clear 2

	;/* Set up scroll plane */
	Scroll_SetRectangle 1, 0, 0, 20, 24, FScroll_RAMWord(1, 0, FALSE, FALSE), Vert

	EndM

;**************************************************************************************************

VFont_Begin Macro fontname, pFontData, nSpaceWidth

vfontstring_fontname     Eval          ""
vfontcuroff_fontname     Eval           0
vfontspacewidth_fontname Eval nSpaceWidth

vfontoffsets_fontname Equ         $
vfontdata_fontname    Equ pFontData

	EndM

;**************************************************************************************************

VFont_Char Macro fontname, nChar, nWidth

	DW vfontcuroff_fontname

vfontcuroff_fontname Eval vfontcuroff_fontname + nWidth
vfontstring_fontname Eval vfontstring_fontname +  nChar

	EndM

;**************************************************************************************************

VFont_End Macro fontname

	DW vfontcuroff_fontname

	EndM

;**************************************************************************************************

VFont_EnumerateString Macro fontname, strString

	FontCmn_EnumerateString strString, vfontstring_fontname, Byte, 1, 0

	EndM

;**************************************************************************************************

VFont_DW Macro nNum

	IfNDef VFont_DW_nNum

VFont_DW_nNum Eval 0

		IRP nBit, 0, 1, 2, 3, 4, 5, 6, 7

			If (((nNum) & (1 << (nBit * 2))) <> 0) || (((nNum) & (1 << ((nBit * 2) + 1))) <> 0)

VFont_DW_nNum Eval VFont_DW_nNum | (1 << nBit)

			EndIf

		EndM

	EndIf

	DB VFont_DW_nNum

	EndM

;**************************************************************************************************

VFont_PrintString: Macro fontname, strString, nX, nY, nColour

	;/* Jump past string */
	jp PastString

String:
	;/* Enumerate string */
	VFont_EnumerateString fontname, strString
	DB 0FFh

PastString:
	;/* Store X position in RAM */
	ldb (bFontX), nX

	;/* Store pointer to current character in xiz */
	ld xiz, String

NextChar:
	;/* Get address of character offset in xbc and test if the last character's been reached */
	ld  xbc,      0
	ldb   c, (xiz+)
	cpb   c,   0FFh
	jp   eq,    End

	;/* Test if character is a space */
	cpb c, 0
	jp  ne, NotASpace

	;/* Increment X position by space width */
	incb vfontspacewidth_fontname, (bFontX)

	;/* Process next character */
	jr NextChar

NotASpace:
	;/* Get address of character offset, store in xbc */
	decb   c
	sll    1,                   xbc
	add  xbc, vfontoffsets_fontname

	;/* Get width of character, store in iy */
	ldw  iy, (xbc+2)
	subw iy,   (xbc)

	;/* Get address of character byte, store in xhl */
	ld  xhl,                  0
	ldw  hl,              (xbc)
	add xhl, vfontdata_fontname

NextByte:
	;/* Get character byte, store in b */
	ldb b, (xhl+)

	push xhl
	push xbc

	;/* Get address of line to set pixel on */
	ld  xde,                       0
	ldb   e,                      nY
	sll   1,                     xde
	ld  xhl,                       0
	ldb   w,                (bFontX)
	ldb   l,                       w
	srl   3,                     xhl
	mul xhl,                 16 * 24
	add xhl,                     xde
	add xhl, FTile_GetTileAddress(1)
	ld  xix,                     xhl

	;/* Determine bit to set */
	andb   w,       7
	ldb    a,       7
	subb   a,       w
	sllb   1,       a
	incb   a
	ldw   de,       1
	sllw   a,      de
	srlw   1,      de
	ldw   wa,      de
	mul  xde,       3
	xorw  de,  0FFFFh
	mul  xwa, nColour

	pop xbc
	pop xhl

	;/* Check byte for remaining bits */
	jr CheckByte

NextBit:
	;/* Check if bit is to be set */
	bit 0,         b
	jr  z, IgnoreBit

	push bc

	;/* Set correct bits and move to next line */
	ldw    bc, (xix)
	andw   bc,    de
	orw    bc,    wa
	ldw (xix),    bc

	pop bc

IgnoreBit:
	;/* Move to next line */
	inc 2, xix

	;/* Prepare byte for next bit */
	srl 1, b

CheckByte:
	;/* Check if any bits remain */
	cpb  b,       0
	jr  ne, NextBit

	;/* Repeat for each line of character */
	incb (bFontX)
	djnz       iy, NextByte

BitsDone:
	;/* Repeat for each character */
	incb (bFontX)
	jp   NextChar

End:
	EndM

;****************************
;* End of VFont_PrintString *
;****************************


	EndIf ;// IfNDef VFONT_INC

;********************
;* End of VFont.inc *
;********************
