;;;; Flavor's NGPC Multi-ROM Menu
;;;;
;;;;
;;;;
;;;; to do:
;;;;      - progress bar while loading
;;;;             check to see if a register/memLoc is being updated when flashing
;;;;
;;;;
;;;; Credits:  Flavor        (everything)
;;;;           Kevlar        (disassembly help)
;;;;           Ivan          (ideas/answers)
;;;;           Judge         (answers)
;;;;           Steve [Fuz]   (ideas/support)
;;;;           Roger Bacon   (sample code)
;;;;           Thor          (sample code)
;;;;           Jeff          (sample code)
;;;;
; Tell the AS assembler we want to code for NGPC
;
   cpu 93C141
   maxmode on

   include "hardware.inc"  ; Include hardware defines
   include "system.inc"    ; Include System Call defines

;
; Define variables
;
START_OF_RAM    EVAL    _MAINRAM + 200h
   RAMDB ROM_NUMBER
   RAMBUF B_U_F_F_E_R_1, 10h         ;not intended for use
   RAMDB ROM_SIZE
   RAMBUF B_U_F_F_E_R_2, 10h         ;not intended for use
   RAMBUF ROMBITS, 8               ;need 8 bits to tell if ROMs are there
   RAMBUF B_U_F_F_E_R_3, 10h         ;not intended for use
   RAMDB LAST_ROM_NUMBER
   RAMBUF B_U_F_F_E_R_7, 10h         ;not intended for use
   RAMDB MENU_ROW
   ;RAMBUF B_U_F_F_E_R_8, 10h         ;not intended for use
   ;RAMDB MENU_NUM

   RAMBUF B_U_F_F_E_R_4, 10h         ;not intended for use
   RAMBUF TEXT_TEMP, 80h           ;a place to put text before you send it to the screen
   RAMBUF B_U_F_F_E_R_5, 10h         ;not intended for use
   RAMBUF cartHeader, 100h         ;temp storage of cart header

   RAMBUF B_U_F_F_E_R_6, 10h         ;not intended for use
   RAMBUF flashROM, (endCopy8M - copy8M) + 20h      ;holds the functions that flash the ROM

HEADER_OFS      EQU     0300000H      ;install this at 0x7F000 in the second ROM (2FF000 of the cart)

ROM_SIZE4Mb     EQU     0080000H      ;ROM_SIZE is 0x80000 for 4Mb carts / 0x100000 for 8Mb carts
ROM_SIZE8Mb     EQU     0100000H      ;ROM_SIZE is 0x80000 for 4Mb carts / 0x100000 for 8Mb carts
CART_HEAD_OFS   EQU     0200000H      ;header offset of the start of the cart ROM
LICENSED_OFS    EQU     0000001H      ;offset of the LICENSED... header
LICENSED_LEN    EQU     000001BH      ;length of the LICENSED... header

SIZE_4Mb        EQU     04
SIZE_8Mb        EQU     08
SIZE_16Mb       EQU     16
SIZE_32Mb       EQU     32

SAVEGAME_4Mb    EQU     0070000H
SAVEGAME_8Mb    EQU     00F0000H

NEWSTART_OFS    EQU     34h
ROMSIZE_OFS     EQU     38h
ROMNUM_OFS      EQU     39h

;
; Standard cartridge header
;
    org    HEADER_OFS
    db     " LICENSED BY SNK CORPORATION"    ; 28 bytes license string
    dd     _start                            ; Program Counter
    dw     0                                 ; Catalog number
    db     0                                 ; Sub catalog number
    db     10h                               ; colour or b+w (10h = colour)
;           123456789012
    db     "Multi-Rom v1"                    ; Game name (12 bytes)
    dd     0                                 ; padding - reserve for Ivan's Menu
startOffset:
    dd     _start                            ; padding - Start address of this Multi-ROM at 0x34
    db     SIZE_4Mb                          ; this is a 4Mb ROM   0x38
	db     2                                 ; 0-based ROM Number  0x39
	db     0, 0                              ; padding - reserve for future use
    dd     0                                 ; padding - reserve for future use

; User Interrupt Vectors

UserIntVect:
        dd      nada            ; Software Interrupt (SWI 3)
        dd      nada            ; Software Interrupt (SWI 4)
        dd      nada            ; Software Interrupt (SWI 5)
        dd      nada            ; Software Interrupt (SWI 6)
        dd      nada            ; RTC Alarm Interrupt
        dd      nada            ; Vertical Blanking Interrupt
        dd      nada            ; Interrupt from Z80
        dd      nada            ; Timer Interrupt (8 bit timer 0)
        dd      nada            ; Timer Interrupt (8 bit timer 1)
        dd      nada            ; Timer Interrupt (8 bit timer 2)
        dd      nada            ; Timer Interrupt (8 bit timer 3)
        dd      nada            ; Serial Transmission Interrupt
        dd      nada            ; Serial Reception Interrupt
        dd      nada            ; (Reserved)
        dd      nada            ; End Micro DMA Int (MicroDMA 0)
        dd      nada            ; End Micro DMA Int (MicroDMA 1)
        dd      nada            ; End Micro DMA Int (MicroDMA 2)
        dd      nada            ; End Micro DMA Int (MicroDMA 3)

nada:   reti

; *** Start of User Code ***
        db 0EDh            ;Ed is my name
        db 0EDh
        db 0EDh
        db 0EDh
_start:
        calr    OS_VERSION      ; Initialize NGP or NGPC mode
        set     6,(rUSERA)      ; User Answer


; Install User Interrupt Vectors

   ; set vector address on RAM
   ld	 xhl,UserIntVect
   ld	 xde,rSWI3
   ld	 bc,18*2
   ldirw (xde+),(xhl+)

   calr InitialiseMemory

; set up screen size 160x152
   ld   (08002h),0
   ld   (08003h),0
   ld   (08004h),0a0h
   ld   (08005h),98h


; Load system font tileset

   ld      ra3,3
   ld      rw3,VECT_SYSFONTSET
   calr    SYSTEM_CALL

; Set Window X & Y Size

   ldb     (rWSIX),SCRN_X
   ldb     (rWSIY),SCRN_Y

; Set scroll 1 palette colors

   ld      xhl,ColorSet
   ld      xde,_SCR1PAL
   ld      xbc,(ColorSetEnd - ColorSet)/2
   ldirw

; enable interrupts
   ei

;-----------------------------------
;the 'program'

   call findROMs
   calr textMenu


   cp (ROM_NUMBER), 0
   jr nz, selected      ;selected new ROM to load

   ld xhl, 200034h       ;otherwise, run current ROM
   ld xde, (xhl)
   jp (xde)

  selected:

   calr clearscreen

   ld (rWDCR),WD_CLR

;;start of Flavor's code



_main:

   ld (rWDCR),WD_CLR

   ;;copy copy8M to RAM and execute there

   ld bc,(endCopy8M - copy8M) + 20h
   ld xde,flashROM
   ld xhl,copy8M
   ldir	(xde+),(xhl+)

   ld (rWDCR),WD_CLR     ;watchdog clear

   call flashROM       ;copy8M

; Power off NGP

   ld      rw3,VECT_SHUTDOWN
   calr    SYSTEM_CALL

_done:   jr      _done




; ---------------------------------------------------------------
; Initialise Memory before game starts
;
; ---------------------------------------------------------------
InitialiseMemory:
; Clear Sprite Attribute Memory

        ld      wa,0
        ld      b,64
        ld      xix,_SPRRAM
SAMloop:
        ld      (xix+),wa
        ld      (xix+),wa
        djnz    b,SAMloop

; Clear Scroll 1 & 2 VRAM with 0
ClearScreen:
        ld      xhl,_SCR1RAM
        ld      wa, 0
        ld      (xhl),wa
        ld      xde,xhl
        inc     xde
        inc     xde
        ld      bc,800h-1
        ldirw
        ret



copy8M:
   ;; copy 4Mbit of data from 1 of the 4M roms to the base
   ;;

   cp (ROM_NUMBER), 0   ;can't load ROM #0
   jr nz, contCopy8M
   ret

 contCopy8M:
   call loading_warning


   call setROMSize   ;set the ROM_SIZE variable


   di

   calr BACKUP_SAVEGAME   ;back-up the save-game area


   LD     (00006Eh),014h  ;14h makes you hold down pow / 04 lets your turn off quick, but causes probs / 10h and 1Ch act like 14
   LD     (00006Fh),0B1h  ;turn off the watchdogging


   ; Erase blocks first (mandatory)
   LD     XIX, 00200000h ;set up XIX to point to the base of the cart
   ld xhl, 16            ; 8 for 4Mb games and 16 for 8Mb games
 eraseBlock:
  dec xhl

   ;set up the first address of the bank to erase
   LD     XDE, XHL       ;XHL is the block number
   SLL    16, XDE        ;each bank is 0x10000 byes large, so shift left 16 is like mul by 0x10000
   ADD    XDE, XIX ;set up XDE to point to the 1st address of the bank to erase (with cart offset)
   calr   BLOCK_ERASE

  cp xhl, 0h
 jr gt, eraseBlock

  LD     (00006Fh),04Eh  ;reset watchdog
  LD     (00006Eh),0F0h


   ; Then write from source to dest
   ld xbc, (ROM_NUMBER)
   ld xhl3, 0
romAddrSet:
   add xhl3, ROM_SIZE4Mb ;add ROM_SIZE to xhl3 ROM_NUMBER of times (xhl3 = ROM_SIZE * ROM_NUMBER)
   dec xbc
   cp xbc, 0
   jr gt, romAddrSet
   add xhl3, CART_HEAD_OFS     ;add the cart header offset
   cp xhl3, 3FFFFFh
   jr le, romInLower16Mb
   add xhl3, 400000h
romInLower16Mb:



 ;  ;edit the cart header so it points to this cart
   ld xhl, xhl3                     ;source
   ld bc, 100h / 2                  ;num words
   ld	xde,cartHeader                ;dest
   ldirw	(xde+),(xhl+)

   ;save the old jump address at 0x34
   ld xhl, xhl3                     ;source
   add xhl, 1Ch
   ld	xde,cartHeader + 34h          ;dest
   ld bc, 2     ;2 words
   ldirw	(xde+), (xhl+)

   ;set up the new jump address at 0x1C (to jump to this ROM)
   ld	xde,cartHeader + 1Ch          ;dest
   ld	xhl,startOffset               ;source
   ld bc, 2     ;2 words
   ldirw	(xde+), (xhl+)

   ;set up the new jump address at 0x1C (to jump to this ROM)
   ld	(cartHeader + ROMSIZE_OFS), (ROM_SIZE)         ;dest
   ld   (cartHeader + ROMNUM_OFS), (ROM_NUMBER)       ;the ROM's bank number

   push xhl3     ;save

   ld  xhl3, cartHeader
   ld  ra3,  0             ; 0 is to select the cart (not the dev kit thingie)
   ld  rbc3, 1; (rbc3 * 0x100) bytes
   ld  xde3, 00000000h     ;to the dest
   ld  rw3,VECT_FLASHWRITE
   ld      (rWDCR),WD_CLR
   swi	1
   ld      (rWDCR),WD_CLR

   pop xhl3       ;restore
   push xhl3      ;save again
   add xhl3, 100h


   ld  ra3,  0             ; 0 is to select the cart (not the dev kit thingie)
   ld  rbc3, (ROM_SIZE4Mb / 0100h) - 1; (rbc3 * 0x100) bytes    [0x1000 = 8Mbit] [0800h = 4Mbit]
   ld  xde3, 00000100h     ;to the dest
   ld  rw3, VECT_FLASHWRITE
   ld      (rWDCR),WD_CLR
   swi	1
   ld      (rWDCR),WD_CLR
   pop xhl3 ;restore again


   ;; check to see if we need to load the second 4Mb
   ; first, check to see if we just loaded the last location

   cp (ROM_SIZE), SIZE_4Mb
   jr z, exitCopy8M   ;next location is a new ROM (we already copied 4Mb)

   ; we're half done

   add xhl3, ROM_SIZE4Mb
   cp  xhl3, 400000h
   jr  nz, keepGoing
   add xhl3, 400000h       ;just crossed the 16Mb border, so jump to 0x800000
 keepGoing:
   ld  ra3,  0             ; 0 is to select the cart (not the dev kit thingie)
   ld  rbc3, ROM_SIZE4Mb / 0100h; (rbc3 * 0x100) bytes    [0x1000 = 8Mbit] [0800h = 4Mbit]
   ld  xde3, ROM_SIZE4Mb     ;to the dest
   ld  rw3, VECT_FLASHWRITE
   ld      (rWDCR),WD_CLR
   swi	1
   ld      (rWDCR),WD_CLR

exitCopy8M:
   ei
   ld      (rWDCR),WD_CLR     ;watchdog clear
   ld xhl, 200034h
   ld xde, (xhl)
   jp (xde)

   ret


;; Do a Block Erase
BLOCK_ERASE:
 push xde
 push xix
 push xiy
 ld     xiy, 0 ;set up the counter
 ; don't ask!
 db 0F3h, 0F1h, 055h, 055h, 000h, 0AAh
 db 0F3h, 0F1h, 0AAh, 02Ah, 000h, 055h
 db 0F3h, 0F1h, 055h, 055h, 000h, 0F0h
 db 000h, 000h
 db 0F3h, 0F1h, 055h, 055h, 000h, 0AAh
 db 0F3h, 0F1h, 0AAh, 02Ah, 000h, 055h
 db 0F3h, 0F1h, 055h, 055h, 000h, 080h
 db 0F3h, 0F1h, 055h, 055h, 000h, 0AAh
 db 0F3h, 0F1h, 0AAh, 02Ah, 000h, 055h
; set (XDE) = 50h
;	30h is the command for SST's Sector Erase
;       a sector is 4KBytes
;       50h is the command for SST's Block Erase
;       a block is 64KBytes
;       we want to erase a block
        LD     (XDE),50h
; repeat until (XDE) == FFh || 200000h iterations
startLP:
        CP     (XDE),0FFh ;  do {               ;82 3F FF
        JR     Z,done                           ;66 15
        INC    1,XIY                            ;ED 61
        CP     XIY,001FFFFFh                    ;ED CF FF FF 1F 00
        JR     Z,failed                         ;66 09
        BIT    5,(XDE) ;*                       ;B2 CD
        JR     Z,startLP ; } while (b5 == 0)    ;66 ED
        CP     (XDE),0FFh ;  if (xde) == 0xff   ;82 3F FF
        JR     Z,done                           ;66 02
failed: LD     A,0FFh                           ;21 FF
done:
 db 0F3h, 0F1h, 055h, 055h, 000h, 0AAh
 db 0F3h, 0F1h, 0AAh, 02Ah, 000h, 055h
 db 0F3h, 0F1h, 055h, 055h, 000h, 0F0h
 pop xiy
 pop xix
 pop xde
 RET

BACKUP_SAVEGAME:
   ;it seems that some ROMs don't save savegame info in the last 64Kb like they should
   ; until I figure out what to do, I'll just return

   ret

   ;copy 0x70000 for 4Mb
   ; and 0xF0000 for 8Mb

   ld c, (CART_HEAD_OFS + ROMNUM_OFS) 
   cp c, 2
   jr gt, contBackup
   ret
contBackup:


   ld xhl, 070000h

   ld c, (CART_HEAD_OFS + ROMSIZE_OFS) 
   cp c, SIZE_4Mb
   jr z, is4Mb
   ld xhl, 0F0000h
is4Mb:

   LD  XIX, 00200000h ;set up XIX to point to the base of the cart

   ld xbc, 0
   ld c, (CART_HEAD_OFS + ROMNUM_OFS)
    
   ld xde, xhl

backupAddrSet:
   LD     (00006Fh),04Eh  ;reset watchdog  ;08 6F 4E
   add xde, ROM_SIZE4Mb ;add ROM_SIZE to xde3 ROM_NUMBER of times (xhl = ROM_SIZE * ROM_NUMBER)
   dec xbc
   cp xbc, 0
   jr gt, backupAddrSet

   add xde, CART_HEAD_OFS     ;add the cart header offset
   cp xde, 3FFFFFh
   jr le, backupLower16Mb

   add xde, 00400000h
   LD  XIX, 00800000h ;set up XIX to point to the base of the cart

 backupLower16Mb:


   push xhl

   LD     (00006Fh),04Eh  ;reset watchdog  ;08 6F 4E
   calr   BLOCK_ERASE
   LD     (00006Fh),04Eh  ;reset watchdog  ;08 6F 4E

   pop xhl

   ; xhl is src
   add xhl, CART_HEAD_OFS
   ; xde is dest

   ;ld xix, 800000h ;set up XIX to point to the base of the cart
   ;ld xhl, 2F0000h
   ;ld xde, 8F0000h
   
   ld xbc, 010000h; SaveGame area is 0x10000

   calr FLASH_RAM

   ret


FLASH_RAM:
        ;before calling this, set up
		;  xbc = number of bytes to flash
		;  xde = destination address
		;  xhl = source address
		;  xix = 200000h for the lower 16Mb, 800000h for upper 16Mb
		push xbc
		push xde
		push xhl
		push xix
        LD     (00006Eh),014h  ;these           ;08 6E 14
        LD     (00006Fh),0B1h  ;disable watchdog;08 6F B1

        LD     (XIX+5555h),0AAh ;              ;F3 F1 55 55 00 AA
        LD     (XIX+2AAAh),055h ;              ;F3 F1 AA 2A 00 55
        LD     (XIX+5555h),0F0h ;              ;F3 F1 55 55 00 F0
        ;LD     QBC,0                            ;D7 E6 A8
        DB     0E9h, 0ECh
        LD     (000021h),00h                    ;08 21 00
continueFlashing:
        LD     W,(XHL+)                         ;C5 EC 20
        LD     XIY,0                            ;ED A8
        CALR   flashAByte                       ;1E 24 00
        INC    1,XDE                            ;EA 61
        CP     A,0FFh                           ;C9 CF FF
        JR     Z,doneFlashing                   ;66 0A
        DEC    1,XBC                            ;E9 69
        CP     XBC,00000000h                    ;E9 CF 00 00 00 00
        JR     NZ,continueFlashing              ;6E E7
doneFlashing:
        LD     (XIX+5555h),0AAh ;              ;F3 F1 55 55 00 AA
        LD     (XIX+2AAAh),055h ;              ;F3 F1 AA 2A 00 55
        LD     (XIX+5555h),0F0h ;              ;F3 F1 55 55 00 F0

        LD     (00006Fh),04Eh  ;reset watchdog  ;08 6F 4E
        LD     (00006Eh),0F0h  ;                ;08 6E F0
		pop xix 
		pop xhl
		pop xde 
		pop xbc
        RET                                     ;0E



flashAByte:
        LD     (XIX+5555h),0AAh ;               ;F3 F1 55 55 00 AA
        LD     (XIX+2AAAh),055h ;               ;F3 F1 AA 2A 00 55
        LD     (XIX+5555h),0A0h ;               ;F3 F1 55 55 00 A0
        LD     (XDE),W ;                        ;B2 40
continueFlashAByte:
        CP     W,(XDE) ;                        ;82 F0
        JR     Z,doneFlashAByte                 ;66 14
        INC    1,XIY                            ;ED 61
        CP     XIY,0003FFFFh                    ;ED CF FF FF 03 00
        JR     Z,failedFlashAByte               ;66 08
        BIT    5,(XDE) ;                        ;B2 CD
        JR     Z,continueFlashAByte             ;66 EE
        CP     W,(XDE) ;                        ;82 F0
        JR     Z,doneFlashAByte                 ;66 02
failedFlashAByte:
        LD     A,0FFh                            ;21 FF
doneFlashAByte:
        RET                                     ;0E

endCopy8M:



textMenu:
    ;display the main-mene background

    ; copy charset to character RAM
    ld bc,(ENDMMCHARSET-MMCHARSET)/2
    ld xde, 0a810h
    ld xhl, MMCHARSET
    ldirw (xde+),(xhl+)

    ; copy palette to character palette RAM
    ld bc, (ENDMMPALETTE-MMPALETTE)/2
    ld xde, 08300h
    ld xhl, MMPALETTE
    ldirw (xde+),(xhl+)

    ; copy TEXT to VRAM
    ld bc, (ENDMMTEXT-MMTEXT)/2
    ld xde, 09800h
    ld xhl, MMTEXT
    ldirw (xde+),(xhl+)

    ; background color
    ld de, (MMBACKGROUND)
    ld xhl, 083e0h
    ld (xhl), de

   ld (ROM_NUMBER), 3  ;set default rom number



   ;ld xhl, A_TO_RUN            ;source
   ;ld xde, TEXT_TEMP           ;dest
   ;ld xbc, A_TO_RUN_END - A_TO_RUN
   ;ldir

   ;ld xhl, 200024h             ;source
   ;ld xde, TEXT_TEMP           ;dest
   ;add xde, A_TO_RUN_END - A_TO_RUN
   ;ld xbc, 12
   ;ldir

   ;set the text to the screen
   ;ld xix, TEXT_TEMP
   ;ld xhl, 2
   ;ld xde, 0
   ;ld b, 12 + (A_TO_RUN_END - A_TO_RUN)
   ;calr text_set


   ;ld xhl, UP_DN_SELECT        ;source
   ;ld xde, TEXT_TEMP           ;dest
   ;ld xbc, UP_DN_SELECT_END - UP_DN_SELECT
   ;ldir

   ;set the text to the screen
   ;ld xix, TEXT_TEMP
   ;ld xhl, 6
   ;ld xde, 0
   ;ld b, UP_DN_SELECT_END - UP_DN_SELECT
   ;calr text_set

   ;arrow is 0x10, I think
   ld (MENU_ROW), 1
 showROMs:

   ld xbc, ROMBITS
   ld xwa, 0
   ld a, (MENU_ROW)
   add a, 2
   add xbc, xwa
   cp (xbc), 1
   jr nz, contShowROMs

   ld c, (MENU_ROW)
   add c, 30h              ; 30h is '0'
   ld (TEXT_TEMP), c
   ld (TEXT_TEMP+1), 2dH   ; 2Dh is the '-'

   ld xbc, (MENU_ROW)
   add xbc, 2              ; 1-based vs 3-based
   ld xhl, 0
menuRomNameAddrSet:
   cp xbc, 0
   jr z, doneAdding
   add xhl, ROM_SIZE4Mb ;add ROM_SIZE to xhl3 ROM_NUMBER of times (xhl3 = ROM_SIZE * ROM_NUMBER)
   dec xbc
   jr menuRomNameAddrSet
doneAdding:
   add xhl, 200024h            ;source
   cp xhl, 3FFFFFh
   jr le, inLower16MB
   add xhl, 400000h
inLower16Mb:
   ld xde, TEXT_TEMP           ;dest
   add xde, 2
   ld xbc, 12
   ldir


   ld xix, TEXT_TEMP
   ld xde, 1
   ld xhl, 0
   ld l, (MENU_ROW)
   add l, 5
   ld b, 14
   calr text_set

  contShowROMs:
   inc (MENU_ROW)
   cp (MENU_ROW), 5
   jrl le, showROMs

   ld xix, ARROW
   ld xde, 0
   ld xhl, 0
   ld l, (ROM_NUMBER)
   add l, 3
   ld b, 1
   calr text_set


update_menu:
;   ld      (rWDCR),WD_CLR     ;watchdog clear;

;   ld xhl, B_TO_LOAD       ;source
;   ld xde, TEXT_TEMP           ;dest
;   ld xbc, B_TO_LOAD_END - B_TO_LOAD
;   ldir

;   ld xbc, (ROM_NUMBER)
;   ld xhl, 0
;menuRomNameAddrSet:
;   cp xbc, 0
;   jr z, doneAdding
;   add xhl, ROM_SIZE4Mb ;add ROM_SIZE to xhl3 ROM_NUMBER of times (xhl3 = ROM_SIZE * ROM_NUMBER)
;   dec xbc
;   jr menuRomNameAddrSet
;doneAdding:
;   add xhl, 200024h            ;source
;   cp xhl, 3FFFFFh
;   jr le, inLower16MB
;   add xhl, 400000h
;inLower16Mb:
;   ld xde, TEXT_TEMP           ;dest
;   add xde, B_TO_LOAD_END - B_TO_LOAD
;   ld xbc, 12
;   ldir

   ;set the text to the screen
;   ld xix, TEXT_TEMP
;   ld xhl, 8    ;x
;   ld xde, 0    ;y
;   ld b, 12 + (B_TO_LOAD_END - B_TO_LOAD)
;   calr text_set

waitInput:
   ld      (rWDCR),WD_CLR     ;watchdog clear
	ld a, (rSL)		;read joystick
   cp a, 0
   jr z, waitInput
waitInputUp:
   ld      (rWDCR),WD_CLR     ;watchdog clear
   cp (rSL), 0
   jr nz, waitInputUp

   bit PADB_A, a		;compare bit setting for button A
   jr nz, menuA

   bit PADB_B, a		;compare bit setting for button B
   jr nz, menuB

   ;bit PADB_OPTION, a		;compare bit setting for OPTION
   ;jr nz, menuO

   ;bit PADB_RIGHT, a		;compare bit setting for RIGHT
   ;jr nz, menuR
   ;bit PADB_LEFT, a		;compare bit setting for LEFT
   ;jr nz, menuL

   bit PADB_DOWN, a		;compare bit setting for DOWN
   jr nz, menuD
   bit PADB_UP, a		;compare bit setting for UP
   jr nz, menuU

   jr waitInput		;if its 0 (not pressed) loop again

menuA:
   ;ld (ROM_NUMBER), 0
   ;jr exitMenu
   ;same as B

menuB:
   ld c, (CART_HEAD_OFS + ROMNUM_OFS) 
   cp (ROM_NUMBER), c
   jrl nz, exitMenu
   ld (ROM_NUMBER), 0    ;load ROM 0 if user selected same ROM as what's loaded
   jrl exitMenu

menuU:
   ld d, (ROM_NUMBER) ;temp ROM_NUM
 menuUpLoop:

   cp d, 3
   jr le, exitMenuUp

   dec d

   ld xbc, ROMBITS
   ld xwa, 0
   ld a, d
   add xbc, xwa
   cp (xbc), 1
   jr nz, menuUpLoop

   push d
   ld xix, SPACE
   ld xde, 0
   ld xhl, 0
   ld l, (ROM_NUMBER)
   add l, 3
   ld b, 1
   calr text_set
   pop d

   ld (ROM_NUMBER), d

   ld xix, ARROW
   ld xde, 0
   ld xhl, 0
   ld l, (ROM_NUMBER)
   add l, 3
   ld b, 1
   calr text_set

 exitMenuUp:
   jp update_menu

menuD:
   ld d, (ROM_NUMBER) ;temp ROM_NUM
 menuDnLoop:
   cp d, (LAST_ROM_NUMBER)
   jr ge, exitMenuDn

   inc d

   ld xbc, ROMBITS
   ld xwa, 0
   ld a, d
   add xbc, xwa
   cp (xbc), 1
   jr nz, menuDnLoop


   push d
   ld xix, SPACE
   ld xde, 0
   ld xhl, 0
   ld l, (ROM_NUMBER)
   add l, 3
   ld b, 1
   calr text_set
   pop d

   ld (ROM_NUMBER), d

   ld xix, ARROW
   ld xde, 0
   ld xhl, 0
   ld l, (ROM_NUMBER)
   add l, 3
   ld b, 1
   calr text_set

 exitMenuDn:

   jp update_menu

exitMenu:
   ret

loading_warning:

    ; copy charset to character RAM
    ld bc,(ENDCHARSET-CHARSET)/2
    ld xde, 0a810h
    ld xhl, CHARSET
    ldirw (xde+),(xhl+)

    ; copy palette to character palette RAM
    ld bc, (ENDPALETTE-PALETTE)/2
    ld xde, 08300h
    ld xhl, PALETTE
    ldirw (xde+),(xhl+)

    ; copy TEXT to VRAM
    ld bc, (ENDTEXT-TEXT)/2
    ld xde, 09800h
    ld xhl, TEXT
    ldirw (xde+),(xhl+)

    ; background color
    ld de, (MMBACKGROUND) ;MMBackground is nicer looking?
    ld xhl, 083e0h
    ld (xhl), de


   ld xhl, LOADING       ;source
   ld xde, TEXT_TEMP     ;dest
   ld xbc, LOADING_END - LOADING
   ldir

   ld      (rWDCR),WD_CLR     ;watchdog clear

   ld xbc, (ROM_NUMBER)
   ld xhl, 0
romNameAddrSet:
   cp xbc, 0
   jr z, doneAddingLW
   add xhl, ROM_SIZE4Mb ;add ROM_SIZE to xhl3 ROM_NUMBER of times (xhl3 = ROM_SIZE * ROM_NUMBER)
   dec xbc
   jr romNameAddrSet
doneAddingLW:
   add xhl, 200024h            ;source
   cp xhl, 3FFFFFh
   jr le, loadWarnInLower16Mb
   add xhl, 400000h
loadWarnInLower16Mb:
   ld xde, TEXT_TEMP           ;dest
   add xde, LOADING_END - LOADING
   ld xbc, 12
   ldir

   ;set the text to the screen
   ld xix, TEXT_TEMP
   ld xhl, 18    ;x
   ld xde, 0    ;y
   ld b, 12 + (LOADING_END - LOADING)
   calr text_set

   ;ld xhl, DO_NOT_TURN_OFF       ;source
   ;ld xde, TEXT_TEMP     ;dest
   ;ld xbc, DO_NOT_TURN_OFF_END - DO_NOT_TURN_OFF
   ;ldir
   ;set the text to the screen
   ;ld xix, TEXT_TEMP
   ;ld xhl, 8    ;x
   ;ld xde, 2    ;y
   ;ld b, DO_NOT_TURN_OFF_END - DO_NOT_TURN_OFF
   ;calr text_set
   ret

text_set:
   ;;XIX must be a pointer to a text string
   ;; B is the length of the string
   ;; XHL is the Y position
   ;; XDE is the X position

   mul XHL, SCRN_VX_B
   mul XDE, 2
   ld	xiy,_SCR1RAM
   add xiy, XHL
   add xiy, XDE
   ld	wa,0

   ld      (rWDCR),WD_CLR     ;watchdog clear
text_set_100:
   ld	a,(xix+)
   ld	(xiy+),wa
   djnz	b,text_set_100

   ret

strncmp:
   ; compare 2 strings (xhl), (xde)
   ; return a = 0 on same, a = 1 on different
   ; xbc = num of bytes to compare
   ld a, 1
   push xhl
   push xde
   push xbc
   push w
   cp xbc, 0
   jr z, exitstrncmp
 contCompare:
   ld      (rWDCR),WD_CLR     ;watchdog clear
   ld w, (xde+)
   cp (xhl+), w
   jr nz, exitstrncmp
   dec xbc
   cp xbc, 0
   jr gt, contCompare
   ld a, 0
 exitstrncmp:
   pop w
   pop xbc
   pop xde
   pop xhl
   ret

findROMs:
   ; break the cart up into 8 4Mb sections
   ;  look for the
   ;   LICENSED BY SNK CORPORATION
   ;  header to find out if a ROM is at that location
   ;  if it's there, set a bit in (ROMBITS) to let us know where ROMs are


   ;set up starting values
   ld (LAST_ROM_NUMBER), 0  ;  it needs to keep track of the last found ROM set to 0 for now
   ld xbc2, ROMBITS   ;make sure we're zeroed
   ld w, 0            ;counter
   ld xhl, HEADER_OFS + LICENSED_OFS      ;this ROM's license
   ld xde, CART_HEAD_OFS + LICENSED_OFS   ;0-based ROM's license
   ld xbc, LICENSED_LEN

 findROMLoop:
   ld      (rWDCR),WD_CLR     ;watchdog clear

   ld (xbc2), 0
   call strncmp     ;a will return 0 for equal
   cp a, 0
   jr nz, noROMfound

   ;found one
   ld (xbc2), 1
   ld (LAST_ROM_NUMBER), w

 noROMfound:
   inc xbc2
   inc w
   cp w, 8
   jr z, exitFindROMs     ; if w is 8, we're done
   add xde, ROM_SIZE4Mb   ; move to the next ROM

   cp  w, 4               ; we're on to the second 16Mb
   jr nz, notAdd16Mb
   add xde, 400000h       ; so add to jump up to 0x800000 area
  notAdd16Mb:

   jr findROMLoop

 exitFindROMs:
   ; also should make sure the bit for the currently loaded ROM is off
   ret


setROMSize
   ; uses LAST_ROM_NUMBER, ROM_NUMBER, and ROMBITS to set ROM_SIZE
   push xwa
   push xbc

   ldb (ROM_SIZE), SIZE_4Mb

   ld xwa, (LAST_ROM_NUMBER)
   cp (ROM_NUMBER), xwa
   jr z, exitSetROMSize   ;there is no next location

   ld xbc, ROMBITS
   ld xwa, 0
   ld a, (ROM_NUMBER)
   add a, 1            ;the next ROM
   add xbc, xwa
   cp (xbc), 1
   jr z, exitSetROMSize   ;next location is a new ROM

   ldb (ROM_SIZE), SIZE_8Mb

exitSetROMSize:
   pop xbc
   pop xwa
   ret


ColorSet:
        DW      0         ; Don't Care (Transparent)
        RGBSet  255,0,0   ; Red
        RGBSet  0,255,0   ; Green
        RGBSet  0,0,0     ; Black
ColorSetEnd:

   align	2
   db "Strings  "
A_TO_RUN:
   db	"A runs "
A_TO_RUN_END:

A_TO_LOAD:
   db	"A loads "
A_TO_LOAD_END:

B_TO_LOAD:
   db	"B loads "
B_TO_LOAD_END:

LOADING:
   db	"Loading "
LOADING_END:

DO_NOT_TURN_OFF:
   db	"Do not turn off!"
DO_NOT_TURN_OFF_END:

UP_DN_SELECT:
   db	"Use UP/DN to select."
UP_DN_SELECT_END:

ARROW:
   db 10h
ARROW_END:

SPACE:
   db	" "
SPACE_END:

   align	2

SYSTEM_CALL:
   SystemCallCode
OS_VERSION:
   OsVersionCode

   include "MainMenu.inc"  ; Main Menu graphic
   include "lik-sang.inc"  ; Lik-Sang graphic

   org HEADER_OFS + 007FFFFh   ;set to 4Mb
   db 0FFh