; Tell the AS assembler we want to code for NGPC
;
        cpu 93C141
        maxmode on

        include "stddef96.inc"


        include "hardware.inc"  ; Include hardware defines
        include "system.inc"    ; Include System Call defines

;
; Define variables
;
Lightarray      equ _MAINRAM ; 5x5 map
START_OF_RAM    EVAL    _MAINRAM + 200h
      RAMDB CURRENTTILE
		RAMDB CURSORTILE
      RAMDB VBCounter
    	RAMDB YPOS
      RAMDB XPOS
      RAMDB JoystickMoved
		RAMDB CURRLEVEL
		RAMDB SPEED
		RAMDB LEVELCOMPLETE
      RAMDD FLASHDEST
      RAMDD FLASHSRC
      RAMDB BLOCK
      RAMDB MEM_006C58h
      RAMDB MEM_006C59h 

		RAMBUF FLASHBUFFER, 1   ;1000h

      RAMDB LASTVAR

HEADER_OFS      EQU     0200000H

MAGIC_NB	EQU	0ED00ED00h
;SAVEOFFSET EQU 000010000h ; offset of flash block
;BLOCK_NB   EQU 01       ; number of this block
; refer to the doc for address/block nb


;
; Standard cartridge header
;
    org    HEADER_OFS
    db     " LICENSED BY SNK CORPORATION"    ; 28 bytes license string
    dd     _start                            ; Program Counter
    dw     0                                 ; Catalog number
    db     0                                 ; Sub catalog number
    db     10h                               ; colour or b+w (10h = colour)
;           123456789012
    db     "Multi-Rom v1"                    ; Game name (12 bytes)
    dd     0,0,0,0                           ; padding - reserve for future use

; User Interrupt Vectors

UserIntVect:
        dd      nada            ; Software Interrupt (SWI 3)
        dd      nada            ; Software Interrupt (SWI 4)
        dd      nada            ; Software Interrupt (SWI 5)
        dd      nada            ; Software Interrupt (SWI 6)
        dd      nada            ; RTC Alarm Interrupt
        dd      6800h           ; Vertical Blanking Interrupt
        dd      nada            ; Interrupt from Z80
        dd      nada            ; Timer Interrupt (8 bit timer 0)
        dd      nada            ; Timer Interrupt (8 bit timer 1)
        dd      nada            ; Timer Interrupt (8 bit timer 2)
        dd      nada            ; Timer Interrupt (8 bit timer 3)
        dd      nada            ; Serial Transmission Interrupt
        dd      nada            ; Serial Reception Interrupt
        dd      nada            ; (Reserved)
        dd      nada            ; End Micro DMA Int (MicroDMA 0)
        dd      nada            ; End Micro DMA Int (MicroDMA 1)
        dd      nada            ; End Micro DMA Int (MicroDMA 2)
        dd      nada            ; End Micro DMA Int (MicroDMA 3)

nada:   reti

; *** Start of User Code ***

_start:
        calr    OS_VERSION      ; Initialize NGP or NGPC mode
        set     6,(rUSERA)      ; User Answer

; Install User Interrupt Vectors

        lda     xix,(UserIntVect)
        lda     xiy,(rSWI3)
        ld      b,18
UIVloop:
        ld      xwa,(xix+)
        ld      (xiy+),xwa
        djnz    b,UIVloop

; set up screen size 160x152
    ld   (08002h),0
    ld   (08003h),0
    ld   (08004h),0a0h
    ld   (08005h),98h

    ; _BGCPAL = GRxB
    ldw (_BGCPAL), 0 ;background colour
    ld (8118h), 80h

    calr InitialiseMemory

; VBlank int to RAM
; characters found in included file
	ld	bc,(endVBlankInt - VBlankInt) * 2 ;
	ld	xde,6800h
	ld	xhl,VBlankInt
	ldirw	(xde+),(xhl+)

; enable interrupts
        ei

;-----------------------------------
;the 'program'

menu:
    call clearscreen

    call dotitlescreen
    call clearscreen
    ld a, 0
    ld w, 20
    call vbwait

;;start of Flavor's code

;wait for button input
Restart:


_main:

   ;;copy copy8M to RAM and execute there

; copy charset to character RAM
; characters found in included file
	ld	bc,(endCopy8M - copy8M) *2 ;    /2	;in this case 'first.inc'
	ld	xde,LASTVAR     ;6000h
	ld	xhl,copy8M
	ldirw	(xde+),(xhl+)

	call LASTVAR       ;6000h     ;copy8M

   ; _BGCPAL = GRxB  (xBGR ???)
   ldw (_BGCPAL), 0000Fh ;background colour
   ld (8118h), 80h

stoopLoop:
    jr stoopLoop

notcomplete:

    ; User shutdown? (Power off pressed?)

        cp      (rUSERS),0
        jp      z,_main      ; no

; Power off NGP

        ld      rw3,VECT_SHUTDOWN
        calr    SYSTEM_CALL

_done:   jr      _done




; ---------------------------------------------------------------
; Initialise Memory before game starts
;
; ---------------------------------------------------------------
InitialiseMemory:
; Clear Sprite Attribute Memory

        ld      wa,0
        ld      b,64
        ld      xix,_SPRRAM
SAMloop:
        ld      (xix+),wa
        ld      (xix+),wa
        djnz    b,SAMloop

; Clear Scroll 1 & 2 VRAM with 0
ClearScreen:
        ld      xhl,_SCR1RAM
        ld      wa, 0
        ld      (xhl),wa
        ld      xde,xhl
        inc     xde
        inc     xde
        ld      bc,800h-1
        ldirw
        ret

;------------------------------
;initgamevars	- game stuff
;----------------------------

initgamevars:

;load game tiles

; copy internal charset to character RAM
        ld      ra3,3
        ld      rw3,VECT_SYSFONTSET
        calr    SYSTEM_CALL

; copy charset to character RAM
    ld bc, (ENDCHARSET - CHARSET)/2
    ld xde, _TILERAM
    ld xhl, CHARSET
    ldirw (xde+),(xhl+)

; copy palette to character palette RAM
    ld bc, (ENDPALETTE-PALETTE)/2
    ld xde, _SCR2PAL
    ld xhl, PALETTE
    ldirw (xde+),(xhl+)

; copy palette to character palette RAM (grey palette for text)
    ld bc, (ENDTITLEPALETTE-TITLEPALETTE)/2
    ld xde, _SCR1PAL
    ld xhl, TITLEPALETTE
    ldirw (xde+),(xhl+)



	ld (CURSORTILE), 13	;pick center square (13)
	ld (YPOS), 3
	ld (XPOS), 3
	ld (CURRLEVEL), 1	;start at level 1
	ld (LEVELCOMPLETE), 0	;level is not complete

	ret

; ---------------------------------------------------------------
; Handle Movement
;
; ---------------------------------------------------------------
MoveUp:
	cp (YPOS), 1
	jr z, moveupend	
	sub (YPOS), 1
	sub (CURSORTILE), 5
MoveUpEnd:
        ret

MoveDown:
	cp (YPOS), 5
	jr z, movedownend	
	add (YPOS), 1
	add (CURSORTILE), 5
MoveDownEnd:
        ret

MoveLeft:
	cp (XPOS), 1
	jr z, moveleftend
	sub (XPOS), 1
	sub (CURSORTILE), 1
MoveLeftEnd:
        ret

MoveRight:
	cp (XPOS), 5
	jr z, moverightend	
	add (XPOS), 1
	add (CURSORTILE), 1
MoveRightEnd:
        ret

ToggleLight:
	;middle
	call litetog

	;left
	cp (XPOS), 1	
	jr z, liteleftend	
	dec (CURSORTILE)
	call litetog
	inc (CURSORTILE)	;back again	
LiteLeftEnd:

	;right
	cp (XPOS), 5	
	jr z, literightend	
	inc (CURSORTILE)
	call litetog
	dec (CURSORTILE)	;back again	
LiteRightEnd:

	;up
	cp (YPOS), 1	
	jr z, liteupend
	sub (CURSORTILE), 5
	call litetog
	add (CURSORTILE), 5	;back again	
LiteupEnd:
	
	;down
	cp (YPOS), 5	
	jr z, litedownend	
	add (CURSORTILE), 5
	call litetog
	sub (CURSORTILE), 5	;back again	
LitedownEnd:
	ret


litetog:
	; get cursor tile into a 32bit register
	push xwa

	ld xwa, 0
	ld a, (CURSORTILE)
	dec a	;for some reason, it does the next one


	; add the start address of the array
	add xwa, lightarray

	cp (xwa), 0	;see if its off
	jr z, makelit	;if its not, make it lit
	ld (xwa), 0	;if it is, make it not.
	jp tlitend
	; store light value
makelit	ld (xwa), 1	;light
	jp tlitend
tlitend
	pop xwa

	ret

;-------------------
TestArray:
;checks the lightarray to see if they are all off.
;-------------------	

	push xwa
	push xhl
	push xbc

	ld xbc, lightarray
	ld l, 1			;init counter
checkagain:
	ld a, (xbc)		;load array into a
	cp a, 0			; if a light is on, level is not done.
	jr nz, checkend
	inc l
	inc xbc
	cp l, 26		;if l has counted all 25 and not exited
	jr nz, checkagain
	ld (LEVELCOMPLETE), 1	;signal the end of the level

checkend:
	pop xbc
	pop xhl
	pop xwa
	ret

;-------------------
returntotitle:
;-------------------
	jp menu
	ret

;-------------------
lcomplete:    
; level done!
;-------------------	
	
	ld (LEVELCOMPLETE), 0 	;reset level complete counter

	;pause
	ld w, 20
	call vbWait

	ld a, 1 	;counter
	
pulsestrt:
	;copy end level arrays to make a pulse
	ld bc, 25		; how many bytes to copy
	ld xde, lightarray	; destination
	ld xhl, leveldone1	; source
	ldir (xde+), (xhl+)	; do copy
	call drawscreen
	ld w, 5
	call vbWait

	;copy end level arrays to make a pulse
	ld bc, 25		; how many bytes to copy
	ld xde, lightarray	; destination
	ld xhl, leveldone2	; source
	ldir (xde+), (xhl+)	; do copy
	call drawscreen
	ld w, 5
	call vbWait

	;copy end level arrays to make a pulse
	ld bc, 25		; how many bytes to copy
	ld xde, lightarray	; destination
	ld xhl, leveldone3	; source
	ldir (xde+), (xhl+)	; do copy
	call drawscreen
	ld w, 5
	call vbWait
	
	inc a
	cp a, 6
	jp nz, pulsestrt

	ld w, 20
	call vbWait


	inc (CURRLEVEL)

	call resetlightarray

	;check to see if game has been won
	cp (CURRLEVEL), 27
	jp nz, notwon
	call gamewon
	jp menu
notwon
	ret

	;shitty win screen but what they heck!
wontext1	db "! CONGRATULATIONS  !", 0
wontext2	db "YOU HAVE BEATEN ALL", 0
wontext3	db "     26 LEVELS!", 0

gamewon:
	ld (CURRLEVEL), 0
	call clearscreen
        ldw (_BGCPAL), 008h ;background colour
        ld (8118h), 80h

	ld h, 0
	ld l, 6
	ld xbc, wontext1
	call renderstring
	ld h, 0
	ld l, 11
	ld xbc, wontext2
	call renderstring
	ld h, 0
	ld l, 12
	ld xbc, wontext3
	call renderstring

winwait:
	ld a, (rSL)		;read joystick
	bit PADB_A, a		;compare bit setting for button A
	jr z, winwait		;if its 0 (not pressed) loop again

        ldw (_BGCPAL), 000h ;background colour
        ld (8118h), 80h
	ld w, 20
	call vbwait
	jp menu

	ret

;-------------------
DrawScreen:    
;refreshes the tiles based on light array settings
;-------------------	
	ld xbc, lightarray	;load array into xbc

	push xwa
	push xhl	

	ld (CURRENTTILE), 1

	ld wa, 0			;tile=0
        ld l, 1			;row = 1
tiles_NextY
	ld h, 0			;column 0
tiles_NextX

	push a
	;check array to see which light to draw.  0-off 1-on
	ld a, (xbc)
	calr DrawLight		;draw char at these co-ords
	pop a

	inc (CURRENTTILE)
        inc wa			;goto next tile
	inc xbc			;next light in array
        inc h			;next column
        cp h, 5			;compare column to 5
        jr nz, tiles_NextX	;if its returns false, repeat above
        inc l			;if it has reached 20, add 1 to row.
        cp l, 6		        ;compare row to 5
        jr nz, tiles_NextY	;if its not 5, add 1 to row
tiledrawend

	;make window for text
	ld a, 17 		;(blank blue)
        ld l, 2			;row = 3
	ld h, 5			;column 0
border_NextX
        call drawchar
        inc h			;next column
        cp h, 15			;compare column to 5
        jr nz, border_NextX	;if its returns false, repeat above


	ld h, 5
	ld l, 2
	ld xbc, strlevel
	call renderstring


	ld h, 1
	ld l, 16
	ld xbc, strexit1
	call renderstring

	ld h, 1
	ld l, 17
	ld xbc, strexit2
	call renderstring

	ld h, 1
	ld l, 18
	ld xbc, strexit3
	call renderstring

	call displaylevel

	pop xhl	
	pop xwa
	ret

			;   --------------------
strlevel		db "LEVEL :", 0
strexit1		db " A  TOGGLES LIGHT", 0
strexit2		db " B  RESETS GRID  ", 0
strexit3		db " OPTION EXITS", 0

displaylevel: 		;ooh pinched from mines..beyond my feeble brain capacity.. ;)
   push xwa
   push xbc
   push xde
   push xhl
   ld xbc, 0
   ld c, (CURRLEVEL)
   div xbc, 10
   ld de, qbc
   add de, '0'
   ld a, e
	ld h, 14
	ld l, 2
   calr DrawChar2
   ld de, bc
   add de, '0'
   ld a, e
	ld h, 13
	ld l, 2
   calr DrawChar2
   pop xhl
   pop xde
   pop xbc
   pop xwa
   ret


;-------------------
dotitleScreen:
;-------------------

	calr PIC_DISPLAY ; thanks to Ivan for this one :)


titlewait:
	ld a, (rSL)		;read joystick
	bit PADB_A, a		;compare bit setting for button A
	jr z, titlewait		;if its 0 (not pressed) loop again

	ret


;------------------------------
; level light arrays
;-------------------------------

;level arrays, to be copied over the blank one
level1array	db 0,0,0,0,0
		db 0,0,0,0,0
		db 1,0,1,0,1
		db 0,0,0,0,0
		db 0,0,0,0,0

level2array     db 1,0,1,0,1
                db 1,0,1,0,1
		db 0,0,0,0,0
                db 1,0,1,0,1
                db 1,0,1,0,1

level3Array     db 0,1,0,1,0
                db 1,1,0,1,1
                db 1,1,0,1,1
                db 1,1,0,1,1
                db 0,1,0,1,0

level4array     db 0,0,0,0,0
                db 1,1,0,1,1
                db 0,0,0,0,0
                db 1,0,0,0,1
                db 1,1,0,1,1

level5array     db 1,1,1,1,0
                db 1,1,1,0,1
                db 1,1,1,0,1
                db 0,0,0,1,1
                db 1,1,0,1,1

level6array     db 0,0,0,0,0
		db 0,0,0,0,0
		db 1,0,1,0,1
                db 1,0,1,0,1
                db 0,1,1,1,0

level7array     db 1,1,1,1,0
                db 1,0,0,0,1
                db 1,0,0,0,1
                db 1,0,0,0,1
                db 1,1,1,1,0

level8array     db 0,0,0,0,0
                db 0,0,1,0,0
                db 0,1,0,1,0
                db 1,0,1,0,1
                db 0,1,0,1,0

level9array     db 0,1,0,1,0
                db 1,1,1,1,1
                db 0,1,1,1,0
                db 0,1,0,1,1
                db 1,1,1,0,0

level10array    db 0,1,1,1,0
                db 0,1,1,1,0
                db 0,1,1,1,0
		db 0,0,0,0,0
		db 0,0,0,0,0

level11array    db 1,0,1,0,1
                db 1,0,1,0,1
                db 1,0,1,0,1
		db 1,0,1,0,1
		db 0,1,1,1,0


level12array    db 1,1,1,1,1
                db 0,1,0,1,0
                db 1,1,0,1,1
		db 0,1,1,1,0
		db 0,1,0,1,0

level13array    db 0,0,0,1,0
                db 0,0,1,0,1
                db 0,1,0,1,0
		db 1,0,1,0,0
		db 0,1,0,0,0

level14array    db 0,0,0,0,0
                db 0,0,0,0,0
                db 0,1,0,0,0
		db 0,1,0,0,0
		db 0,1,0,0,0

level15array    db 0,0,0,0,0
                db 0,1,0,0,0
                db 0,0,0,0,0
		db 0,1,0,0,0
		db 0,0,0,0,0

level16array    db 1,0,0,0,0
                db 1,0,0,0,0
                db 1,0,0,0,0
		db 1,0,0,0,0
		db 1,1,1,1,1

level17array    db 0,0,0,0,0
                db 0,0,0,0,0
                db 0,0,1,0,0
		db 0,1,1,1,0
		db 1,1,1,1,1

level18array    db 0,0,1,0,0
                db 0,1,0,1,0
                db 1,0,1,0,1
		db 0,1,0,1,0
		db 0,0,1,0,0

level19array    db 1,0,1,0,1
                db 0,0,0,0,0
                db 1,0,1,0,1
		db 0,0,0,0,0
		db 1,0,1,0,1

level20array    db 0,0,0,0,0
                db 0,0,0,0,0
                db 1,0,0,0,1
		db 0,0,0,0,0
		db 0,0,0,0,0

level21array    db 0,1,1,1,1
                db 0,1,0,0,0
                db 0,1,1,1,0
		db 0,1,0,0,0
		db 0,1,0,0,0

level22array    db 0,1,1,1,0
                db 1,0,0,0,1
                db 1,0,0,0,1
		db 1,0,0,0,1
		db 0,1,1,1,0

level23array    db 0,0,0,0,0
                db 0,0,0,0,0
                db 0,0,1,1,1
		db 0,0,1,1,0
		db 0,0,1,0,0

level24array    db 0,0,0,0,0
                db 0,0,0,0,0
                db 1,0,0,0,1
		db 1,1,1,1,1
		db 0,1,0,0,1

level25array    db 1,0,0,0,0
                db 1,1,0,0,0
                db 1,1,1,0,0
		db 1,1,1,1,0
		db 0,1,1,1,1

level26array    db 1,0,0,0,1
                db 1,0,0,0,1
                db 1,1,1,1,1
		db 1,0,0,0,1
		db 1,0,0,0,1

leveldone1      db 0,0,0,0,0
                db 0,0,0,0,0
                db 0,0,1,0,0
		db 0,0,0,0,0
		db 0,0,0,0,0

leveldone2      db 0,0,0,0,0
                db 0,1,1,1,0
                db 0,1,0,1,0
		db 0,1,1,1,0
		db 0,0,0,0,0

leveldone3      db 1,1,1,1,1
                db 1,0,0,0,1
                db 1,0,0,0,1
		db 1,0,0,0,1
		db 1,1,1,1,1



ResetLightArray:

	push xbc	
	push xwa
	push xhl
	push xde	

	;copy Array
	ld bc, 25		; how many bytes to copy
	ld xde, lightarray	; destination
	

	cp (Currlevel), 1
	jp nz, l2
	ld xhl, level1array	; source
	jp docopy
l2:
	cp (Currlevel), 2
	jp nz, l3
	ld xhl, level2array	; source
	jp docopy
l3:
	cp (Currlevel), 3
	jp nz, l4
	ld xhl, level3array	; source
	jp docopy
l4:
	cp (Currlevel), 4
	jp nz, l5
	ld xhl, level4array	; source
	jp docopy
l5:
	cp (Currlevel), 5
	jp nz, l6
	ld xhl, level5array	; source
	jp docopy
l6:
	cp (Currlevel), 6
	jp nz, l7
	ld xhl, level6array	; source
	jp docopy
l7:
	cp (Currlevel), 7
	jp nz, l8
	ld xhl, level7array	; source
	jp docopy
l8:
	cp (Currlevel), 8
	jp nz, l9
	ld xhl, level8array	; source
	jp docopy
l9:
	cp (Currlevel), 9
	jp nz, l10
	ld xhl, level9array	; source
	jp docopy
l10:
	cp (Currlevel), 10
	jp nz, l11
	ld xhl, level10array	; source
	jp docopy
l11:
	cp (Currlevel), 11
	jp nz, l12
	ld xhl, level11array	; source
	jp docopy
l12:
	cp (Currlevel), 12
	jp nz, l13
	ld xhl, level12array	; source
	jp docopy
l13:
	cp (Currlevel), 13
	jp nz, l14
	ld xhl, level13array	; source
	jp docopy
l14:
	cp (Currlevel), 14
	jp nz, l15
	ld xhl, level14array	; source
	jp docopy
l15:
	cp (Currlevel), 15
	jp nz, l16
	ld xhl, level15array	; source
	jp docopy
l16:
	cp (Currlevel), 16
	jp nz, l17
	ld xhl, level16array	; source
	jp docopy
l17:
	cp (Currlevel), 17
	jp nz, l18
	ld xhl, level17array	; source
	jp docopy
l18:
	cp (Currlevel), 18
	jp nz, l19
	ld xhl, level18array	; source
	jp docopy
l19:
	cp (Currlevel), 19
	jp nz, l20
	ld xhl, level19array	; source
	jp docopy
l20:
	cp (Currlevel), 20
	jp nz, l21
	ld xhl, level20array	; source
	jp docopy
l21:
	cp (Currlevel), 21
	jp nz, l22
	ld xhl, level21array	; source
	jp docopy
l22:
	cp (Currlevel), 22
	jp nz, l23
	ld xhl, level22array	; source
	jp docopy
l23:
	cp (Currlevel), 23
	jp nz, l24
	ld xhl, level23array	; source
	jp docopy
l24:
	cp (Currlevel), 24
	jp nz, l25
	ld xhl, level24array	; source
	jp docopy
l25:
	cp (Currlevel), 25
	jp nz, l26
	ld xhl, level25array	; source
	jp docopy
l26:
	cp (Currlevel), 26
	jp nz, lvlend
	ld xhl, level26array	; source
	jp docopy

lvlend:


docopy:	ldir (xde+), (xhl+)	; do copy

	pop xde
	pop xhl
	pop xwa
	pop xbc


	ld (CURSORTILE), 13	;pick center square (13)
	ld (YPOS), 3
	ld (XPOS), 3

	call drawscreen

	ret



;--------------------------------------------
; vbWait
; Waits for n vertical blank interrupts
; inputs: w = number of VBIs to wait for
;--------------------------------------------

vbWait:
   push xwa
   push xbc
   ld (VBCOUNTER), 0
   ld c, w
vbw1:
   ld w, c
   ld a, (VBCOUNTER)
   sub w, a
   jr nc, vbw1
   pop xbc
   pop xwa
   ret

;----------------------------
; RenderString
; hl = xy, xbc = pointer to null terminated string
;----------------------------
RenderString:
    push xwa
    push xhl
rsAgain:
    ld a, (xbc)
    cp a, 0
    jr z, rsEnd
    calr DrawChar2
    inc xbc
    inc h
    jr rsAgain
rsEnd:
    pop xhl
    pop xwa
    ret

; ---------------------------------------------------------------
; 
; DrawChar2
;
; Draws a character on the background plane
; inputs: wa = char, hl = xy
;
; ---------------------------------------------------------------
DrawChar2
    push xhl
    push xde
    push xbc
    ld xde, _SCR1RAM
    ld c, h
    ld b, 0
    mul xbc, 2
    add de, bc
    ld h, 0
    mul xhl, 40h
    add de, hl
    ld (xde), a
    add xde, 1
    ld (xde), w
    pop xbc
    pop xde
    pop xhl
    ret

; ---------------------------------------------------------------
; 
; DrawChar
;
; Draws a character on the background plane
; inputs: wa = char, hl = xy
;
; ---------------------------------------------------------------
DrawChar
    push xhl
    push xde
    push xbc
    ld xde, _SCR2RAM
    ld c, h
    ld b, 0
    mul xbc, 2
    add de, bc
    ld h, 0
    mul xhl, 40h
    add de, hl
    ld (xde), a
    add xde, 1
    ld (xde), w
    pop xbc
    pop xde
    pop xhl
    ret


; ---------------------------------------------------------------
; 
; DrawLight = special 16*16 tile drawing routine
;
; Draws a character on the background plane
; inputs: wa = char, hl = xy
;
; ---------------------------------------------------------------
DrawLight
	push wa
	push hl
	
	push bc
		
	cp a, 0			;check if light is off
	jr nz, lightison 	;if its not off, go to lightison
	; light is off
	
	ld c, (CURRENTTILE)
	cp c, (CURSORTILE)	;check to see if cursor
	;cp (CURRENTTILE), 13	;check to see if cursor

	jp nz, nocursoff
	;is cursor
	ld a, 13		;load the 'off' character (with curs)
	jp drawme

nocursoff	ld a, 5	;load the 'off' character
		jp drawme

lightison
	ld c, (CURRENTTILE)
	cp c, (CURSORTILE)	;check to see if cursor

	jp nz, nocurson
	;is cursor
	ld a, 9			;load the 'on' character (with curs)
	
	jp drawme

nocurson	ld a, 1		;load the 'on' character
		jp drawme

drawme
	pop bc
	add h, h		;extra spacing for bigger characters
	add l, l		;extra spacing for bigger characters
	add h, 5			;just nudge it off the edge
	add l, 2


	;loop to make 4 in a square	
	call drawchar		;top left
	inc a	;next 8*8		
	inc h			;top right
	call drawchar
	inc a	;next 8*8		
	dec h			;bottom left
	inc l
	call drawchar
	inc a	;next 8*8		
	inc h			;bottom right
	call drawchar

	pop hl
	pop wa

	ret

flashBufFill  ;make a backup of what FLASHDEST points at to the FLASHBUFFER
   ld xbc, 0
 fbfLoop:
   ld xhl, FLASHBUFFER
   add xhl, xbc

   ld xde, (FLASHDEST)
   add xde, xbc

   ld de, (xde)
   ld (xhl), de

   inc xbc
   cp xbc, 1000h
   jr lt, fbfLoop
   ret

swap16Ms
   ;; Swap the lower 16Mbit with the upper 16Mbit of flash mem
   ;;

	; Erase block first (mandatory) : 64kb for only 256 bytes (for my game)
	;ld	ra3,0
	;ld	rb3,BLOCK_NB
	;ld	rw3,VECT_FLASHERS
	;ld	(rWDCR),WD_CLR
	;swi	1


   ld xbc,        001FFFFFh
   ld (FLASHSRC), xbc
   ld xbc,        000FFFFFh
   ld (FLASHDEST),xbc
   ld xbc, 0
 swapAChunk:      ;;do the flashing in 0x1000 byte chunks

   ; set up src and dest
   ld xhl, xbc
   sll 12, xhl
   sub (FLASHDEST), xhl
   sub (FLASHSRC), xhl

   call flashBufFill  ;;make a backup of dest

	; Then write from source to dest
	ld	ra3,0
	ld	rbc3,010h	; (rbc3 * 0x100) bytes
   ;;;;;;;;
   ld xhl, (FLASHSRC)
   ld xhl3, xhl     ;from the location FLASHSRC points to
   ;;;;;;;;
   ld xhl, (FLASHDEST)
	ld xde3, xhl      ;to the location FLASHDEST points to
   ;;;;;;;;
	ld	rw3,VECT_FLASHWRITE
	ld	(rWDCR),WD_CLR
	swi	1

   ; Then write from buffer to source
	ld	ra3,0
	ld	rbc3,010h	; (rbc3 * 0x100) bytes
   ;;;;;;;;
   ld xhl3, FLASHBUFFER     ;from FLASHBUFFER
   ;;;;;;;;
   ld xhl, (FLASHSRC)
	ld xde3, xhl      ;to the location FLASHSRC points to
   ;;;;;;;;
	ld	rw3,VECT_FLASHWRITE
	ld	(rWDCR),WD_CLR
	swi	1

	ld	(rWDCR),WD_CLR

	ld	w,50
	calr	VBWait

   cp (FLASHSRC), 0
   jr gt, swapAChunk
	ret

copy8M:
   ;; copy 8Mbit of data from 1 of the 8M roms to the base
   ;;

   ;ld h, 0     ;15
 ;eraseBlock:
  ;doOver:
	; Erase block first (mandatory)
	;ld	a, 0
	;ld	rb3, 0   ;BLOCK_NB
	;ld	rw3, VECT_FLASHERS
	;swi	1
   ;calr CLR_FLASH_RAM


   ld xde, 00200000h  ;the start address of the bank to erase
   LD     XIX, 00200000h     ;set up XIX to point to the base of the cart
   LD     XDE, 00000000h     ;first address of the bank to erase
   ADD    XDE, XIX ;set up XDE to point to the 1st address of the bank to erase (with cart offset)
   ld xiy, 0
   calr BLOCK_ERASE

	;ld	(rWDCR),WD_CLR
   ;cp ra3, SYS_SUCCESS
  ;jr ne, doOver
  ;inc h
  ;cp h, 0Fh
 ;jr lt, eraseBlock


   ; _BGCPAL =    xBGR
   ldw (_BGCPAL), 0FFFh ;background colour
   ld (8118h), 80h

   jr infLoop ;;;;;;;;;;;;;;;

	; Then write from source to dest
	ld	ra3,0
	ld	rbc3,0002h	; (rbc3 * 0x100) bytes    [0x1000 = 8Mbit]
   ld xhl3, 00100000h + HEADER_OFS    ;from the source  [souce SHOULD be (00100000h*ROM number)+dest]
	ld xde3, 00000000h                 ;to the dest
	ld	rw3,VECT_FLASHWRITE
	ld	(rWDCR),WD_CLR
	swi	1
	ld	(rWDCR),WD_CLR

   ;;VBWait
	ld	w,50
   calr vbw
   ;;end VBWait

   ; _BGCPAL =    xBGR
   ldw (_BGCPAL), 0F00h ;background colour
   ld (8118h), 80h

   infLoop:  jr infLoop

   ;;jp (20001Ch)  ;restart???

   ;; *** SHUTDOWN ***
   ld      rw3,VECT_SHUTDOWN
   calr    SYSTEM_CALL

	ret

vbw:
   ld (VBCOUNTER), 0
   ld c, w
vbwLoop:
   ld w, c
   ld a, (VBCOUNTER)
   sub w, a
   jr nc, vbwLoop
   ret

CLR_FLASH_RAM:
 PUSH   SR                               ;02
 LDF    3                                ;17 03
 EI     7                                ;06 07
 PUSH   XIY                              ;3D
 PUSH   XIX                              ;3C
 PUSH   XHL                              ;3B
 PUSH   XDE                              ;3A
 PUSH   XBC                              ;39
 LD     (00006Eh),14h                    ;08 6E 14
 LD     (00006Fh),0B1h                   ;08 6F B1
 LD     E,A                              ;C9 8D
 LD     XIX, 00200000h            ;set up XIX to point to the base of the cart
 LD     XDE, XIX  ;set up XDE to point to the 1st address of the bank to erase
 LD     A,00h                            ;21 00
 LD     XIY,0                            ;ED A8
 CALR   BLOCK_ERASE
 LD     (00006Fh),4Eh                    ;08 6F 4E
 LD     (00006Eh),0F0h                   ;08 6E F0
 POP    XBC                              ;59
 POP    XDE                              ;5A
 POP    XHL                              ;5B
 POP    XIX                              ;5C
 POP    XIY                              ;5D
 POP    SR                               ;03
 RET                                     ;0E


;; Do a Block Erase
BLOCK_ERASE:
 db 0F3h
 db 0F1h
 db 55h
 db 55h
 db 00h
 db 0AAh
 db 0F3h
 db 0F1h
 db 0AAh
 db 02Ah
 db 00h
 db 55h
 db 0F3h
 db 0F1h
 db 55h
 db 55h
 db 00h
 db 0F0h
 db 00h
 db 00h
 db 0F3h
 db 0F1h
 db 55h
 db 55h
 db 00h
 db 0AAh
 db 0F3h
 db 0F1h
 db 0AAh
 db 2Ah
 db 00h
 db 55h
 db 0F3h
 db 0F1h
 db 55h
 db 55h
 db 00h
 db 80h
 db 0F3h
 db 0F1h
 db 55h
 db 55h
 db 00h
 db 0AAh
 db 0F3h
 db 0F1h
 db 0AAh
 db 2Ah
 db 00h
 db 55h
; set (XDE) = 30h
;	30h is the command for SST's Sector Erase
;       a sector is 4KBytes
;       50h is the command for SST's Block Erase
;       a block is 64KBytes
;       we want to erase a block
;201991:         LD     (XDE),30h                        ;B2 00 30
 LD     (XDE),50h
; repeat until (XDE) == FFh || 200000h iterations
startLP:
        CP     (XDE),0FFh ;  do {               ;82 3F FF
        JR     Z,done                           ;66 15
        INC    1,XIY                            ;ED 61
        CP     XIY,001FFFFFh                    ;ED CF FF FF 1F 00
        JR     Z,failed                         ;66 09
        BIT    5,(XDE) ;*                       ;B2 CD
        JR     Z,startLP ; } while (b5 == 0)    ;66 ED
        CP     (XDE),0FFh ;  if (xde) == 0xff   ;82 3F FF
        JR     Z,done                           ;66 02
failed: LD     A,0FFh                           ;21 FF
done:
 db 0F3h
 db 0F1h
 db 055h
 db 055h
 db 000h
 db 0AAh
 db 0F3h
 db 0F1h
 db 0AAh
 db 02Ah
 db 000h
 db 055h
 db 0F3h
 db 0F1h
 db 055h
 db 055h
 db 000h
 db 0F0h
 RET                                     ;0E

endCopy8M:


VBlankInt:

    push xwa
    ; Update Watch Dog Timer to prevent CPU reset
    ld      (rWDCR),WD_CLR

    ; increment a counter
    inc (VBCOUNTER)
    pop xwa
    reti
endVBlankInt:


SYSTEM_CALL:
        SystemCallCode
OS_VERSION:
        OsVersionCode

         include "menupic.inc"
         include "tiles.inc"

   org HEADER_OFS + 00FFFFFh   ;set to 8Mbit file size
   ;db 0FFh
   db 00h




