;;;; Flavor's NGPC Multi-ROM Menu
;;;;
;;;;
;;;;
;;;; to do:
;;;;      - backup save-game info
;;;;
;;;;
;;;; Credits:  Flavor        (everything)
;;;;           Kevlar        (disassembly help)
;;;;           Ivan          (ideas/answers)
;;;;           Judge         (answers)
;;;;           Steve [Fuz]   (ideas/support)
;;;;           Roger Bacon   (sample code)
;;;;           Thor          (sample code)
;;;;           Jeff          (sample code)
;;;;
; Tell the AS assembler we want to code for NGPC
;
   cpu 93C141
   maxmode on

   include "hardware.inc"  ; Include hardware defines
   include "system.inc"    ; Include System Call defines

;
; Define variables
;
START_OF_RAM    EVAL    _MAINRAM + 200h
   RAMDB ROM_NUMBER
   RAMBUF B_U_F_F_E_R_1, 4h         ;not intended for use
   RAMDD ROM_SIZE
   RAMBUF B_U_F_F_E_R_2, 4h         ;not intended for use
   RAMDW PROGRESS_MAX
   RAMBUF B_U_F_F_E_R_7, 4h         ;not intended for use
   RAMDD CURR_ROM_TABLE_PTR
   RAMBUF B_U_F_F_E_R_8, 4h         ;not intended for use
   RAMDB NUM_ROMS
   RAMBUF B_U_F_F_E_R_10, 4h         ;not intended for use
   RAMDB MENU_START
   RAMBUF B_U_F_F_E_R_4, 4h         ;not intended for use
   RAMDB CURR_MENU_PAGE             ;starts at 0
   RAMBUF B_U_F_F_E_R_9, 4h         ;not intended for use
   RAMDB NUM_MENU_PAGES             ;0-based
   RAMBUF B_U_F_F_E_R_11, 4h         ;not intended for use
   RAMDB ROMS_ON_CURR_MENU_PAGE
   RAMBUF B_U_F_F_E_R_12, 4h         ;not intended for use
   RAMDW MENU_PAGE_ARROW_COUNTER
   RAMBUF B_U_F_F_E_R_13, 4h         ;not intended for use

   RAMBUF TEXT_TEMP, 20h           ;a place to put text before you send it to the screen
   RAMBUF B_U_F_F_E_R_5, 4h         ;not intended for use
   RAMBUF cartHeader, 100h         ;temp storage of cart header

   RAMBUF B_U_F_F_E_R_6, 4h         ;not intended for use
   RAMBUF flashROM, (endCopy8M - copy8M) + 20h      ;holds the functions that flash the ROM

HEADER_OFS      EQU     00280000H

ROM_SIZE4Mb     EQU     00080000H      ;ROM_SIZE is 0x80000 for 4Mb carts / 0x100000 for 8Mb carts
ROM_SIZE8Mb     EQU     00100000H      ;ROM_SIZE is 0x80000 for 4Mb carts / 0x100000 for 8Mb carts
CART_HEAD_OFS   EQU     00200000H      ;header offset of the start of the cart ROM
LICENSED_OFS    EQU     00000009H      ;offset of the LICENSED... header
LICENSED_LEN    EQU     00000013H      ;length of the LICENSED... header

SIZE_4Mb        EQU     00080000h
SIZE_8Mb        EQU     00100000h
SIZE_16Mb       EQU     00200000h

SAVEGAME_4Mb    EQU     00070000H
SAVEGAME_8Mb    EQU     000F0000H

NEWSTART_OFS    EQU     34h
ROMSIZE_OFS     EQU     38h
ROMNUM_OFS      EQU     39h

ROMSPERMENUPAGE  EQU     10

;
; Standard cartridge header
;
    org    HEADER_OFS
    db     " LICENSED BY SNK CORPORATION"    ; 28 bytes license string
    dd     _start                            ; Program Counter
    dw     0                                 ; Catalog number
    db     0                                 ; Sub catalog number
    db     10h                               ; colour or b+w (10h = colour)
;           123456789012
    db     "MultiROM16v2"                    ; Game name (12 bytes)
    dd     0                                 ; padding - reserve for Ivan's Menu
startOffset:
    dd     _start                            ; padding - Start address of this Multi-ROM at 0x34
    db     0                                 ; this is a 4Mb ROM   0x38
    db     0FFh                              ; 0-based ROM Number  0x39
    db     0, 0                              ; padding - reserve for future use
    dd     0                                 ; padding - reserve for future use

; User Interrupt Vectors

UserIntVect:
        dd      nada            ; Software Interrupt (SWI 3)
        dd      nada            ; Software Interrupt (SWI 4)
        dd      nada            ; Software Interrupt (SWI 5)
        dd      nada            ; Software Interrupt (SWI 6)
        dd      nada            ; RTC Alarm Interrupt
        dd      nada            ; Vertical Blanking Interrupt
        dd      nada            ; Interrupt from Z80
        dd      nada            ; Timer Interrupt (8 bit timer 0)
        dd      nada            ; Timer Interrupt (8 bit timer 1)
        dd      nada            ; Timer Interrupt (8 bit timer 2)
        dd      nada            ; Timer Interrupt (8 bit timer 3)
        dd      nada            ; Serial Transmission Interrupt
        dd      nada            ; Serial Reception Interrupt
        dd      nada            ; (Reserved)
        dd      nada            ; End Micro DMA Int (MicroDMA 0)
        dd      nada            ; End Micro DMA Int (MicroDMA 1)
        dd      nada            ; End Micro DMA Int (MicroDMA 2)
        dd      nada            ; End Micro DMA Int (MicroDMA 3)

nada:   reti

        DB 01h, 02h, 03h, 04h

; *** Start of User Code ***
_start:
        calr    OS_VERSION      ; Initialize NGP or NGPC mode
        set     6,(rUSERA)      ; User Answer


; Install User Interrupt Vectors

   ; set vector address on RAM
   ld	 xhl,UserIntVect
   ld	 xde,rSWI3
   ld	 bc,18*2
   ldirw (xde+),(xhl+)

   calr InitialiseMemory

; set up screen size 160x152
   ld   (08002h),0
   ld   (08003h),0
   ld   (08004h),0a0h
   ld   (08005h),98h


; Load system font tileset

   ld      ra3,3
   ld      rw3,VECT_SYSFONTSET
   calr    SYSTEM_CALL
   calr    load_progress_bar

; Set Window X & Y Size

   ldb     (rWSIX),SCRN_X
   ldb     (rWSIY),SCRN_Y

; Set scroll 1 palette colors

   ld      xhl,ColorSet
   ld      xde,_SCR1PAL
   ld      xbc,(ColorSetEnd - ColorSet)/2
   ldirw

; enable interrupts
   ei

;-----------------------------------
;the 'program'

   ldb (NUM_ROMS), (ROM_TABLE)   ;set num_roms
        


   calr textMenu


   cpb (ROM_NUMBER), 0FFh
   jr nz, selected      ;selected new ROM to load

   ld xhl, 200034h       ;otherwise, run current ROM
   ld xde, (xhl)
   jp (xde)

  selected:

   calr clearscreen

   ld (rWDCR),WD_CLR

;;start of Flavor's code



;_main:

   ld (rWDCR),WD_CLR

   ;;copy copy8M to RAM and execute there

   ld bc,(endCopy8M - copy8M) + 20h
   ld xde,flashROM
   ld xhl,copy8M
   ldir	(xde+),(xhl+)

   ld (rWDCR),WD_CLR     ;watchdog clear

   call flashROM       ;copy8M

; Power off NGP

   ld      rw3,VECT_SHUTDOWN
   calr    SYSTEM_CALL

_done:   jr      _done




; ---------------------------------------------------------------
; Initialise Memory before game starts
;
; ---------------------------------------------------------------
InitialiseMemory:
; Clear Sprite Attribute Memory

        ld      wa,0
        ld      b,64
        ld      xix,_SPRRAM
SAMloop:
        ld      (xix+),wa
        ld      (xix+),wa
        djnz    b,SAMloop

; Clear Scroll 1 & 2 VRAM with 0
ClearScreen:
        ld      xhl,_SCR1RAM
        ld      wa, 0
        ld      (xhl),wa
        ld      xde,xhl
        inc     xde
        inc     xde
        ld      bc,800h-1
        ldirw
        ret



copy8M:
   ;; copy 4Mbit of data from 1 of the 4M roms to the base
   ;;


   cpb (ROM_NUMBER), 0FFh   ;can't load ROM #0
   jr nz, contCopy8M
   ret

 contCopy8M:
   call loading_warning


   call setROMSize   ;set the ROM_SIZE variable


   di

   calr BACKUP_SAVEGAME   ;back-up the save-game area

   LD     (00006Eh),014h  ;14h makes you hold down pow / 04 lets your turn off quick, but causes probs / 10h and 1Ch act like 14
   LD     (00006Fh),0B1h  ;turn off the watchdogging

   ; Erase blocks first (mandatory)
   LD     XIX, 00200000h ;set up XIX to point to the base of the cart
   ld xhl, 8            ; 8 for 4Mb games and 16 for 8Mb games
 eraseBlock:
  dec xhl

   ;set up the first address of the bank to erase
   LD     XDE, XHL       ;XHL is the block number
   SLL    16, XDE        ;each bank is 0x10000 byes large, so shift left 16 is like mul by 0x10000
   ADD    XDE, XIX ;set up XDE to point to the 1st address of the bank to erase (with cart offset)
   calr   BLOCK_ERASE

  cp xhl, 0h
 jr gt, eraseBlock


   ; Then write from source to dest
;   ld xbc, (ROM_NUMBER)
;   ld xhl3, 0
;romAddrSet:
;   add xhl3, ROM_SIZE4Mb ;add ROM_SIZE to xhl3 ROM_NUMBER of times (xhl3 = ROM_SIZE * ROM_NUMBER)
;   dec xbc
;   cp xbc, 0
;   jr gt, romAddrSet
;   add xhl3, CART_HEAD_OFS     ;add the cart header offset
;   cp xhl3, 3FFFFFh
;   jr le, romInLower16Mb
;   add xhl3, 400000h
;romInLower16Mb:


   ld xbc, (CURR_ROM_TABLE_PTR)
   ld xbc, (xbc)
   ld xhl3, xbc


   ;edit the cart header so it points to this cart
   ld xhl, xhl3                     ;source
   ld bc, 100h / 2                  ;num words
   ld	xde,cartHeader                ;dest
   ldirw	(xde+),(xhl+)

   ;save the old jump address at 0x34
   ld xhl, xhl3                     ;source
   add xhl, 1Ch
   ld	xde,cartHeader + 34h          ;dest
   ld bc, 2     ;2 words
   ldirw	(xde+), (xhl+)

   ;set up the new jump address at 0x1C (to jump to this ROM)
   ld	xde,cartHeader + 1Ch          ;dest
   ld	xhl,startOffset               ;source
   ld bc, 2     ;2 words
   ldirw	(xde+), (xhl+)


   ;set up the new jump address at 0x1C (to jump to this ROM)
   ld	(cartHeader + ROMSIZE_OFS), (ROM_SIZE)         ;dest
   ld   (cartHeader + ROMNUM_OFS), (ROM_NUMBER)       ;the ROM's bank number

   push xhl3     ;save


   ld  xhl, cartHeader
   ld  a,  0             ; 0 is to select the cart (not the dev kit thingie)
   ld  w,  0             ; 1 is to do progress bar, 0 to not
   ld  xbc, 1h; (rbc * 0x100) bytes
   ld  xde, 00000000h     ;to the dest
   ld  xix, 00200000h
   ld      (rWDCR),WD_CLR
   calr FLASH_RAM2
   ld      (rWDCR),WD_CLR


   pop xhl       ;restore
   push xhl      ;save again
   add xhl, 100h

   ld  a,  0             ; 0 is to select the cart (not the dev kit thingie)
   ld  w,  1             ; 1 is to do progress bar, 0 to not
   ld  xbc, (ROM_SIZE)
   add xbc, 0FFh          ;round up to the next 100h bytes
   srl 8, xbc             ;devide by 100h
   ld (PROGRESS_MAX), BC            ;set max progress
   dec xbc

   ld  xde, 00000100h     ;to the dest
   ld      (rWDCR),WD_CLR
   calr FLASH_RAM2
   ld      (rWDCR),WD_CLR

   pop xhl ;restore again


   ;; check to see if we need to load the second 4Mb
   ; first, check to see if we just loaded the last location

;skip this for now
   jr exitCopy8M

   ld xwa, (ROM_SIZE)
   cp xwa, SIZE_4Mb
   jr z, exitCopy8M   ;next location is a new ROM (we already copied 4Mb)

   ; we're half done

   add xhl3, ROM_SIZE4Mb
   cp  xhl3, 400000h
   jr  nz, keepGoing
   add xhl3, 400000h       ;just crossed the 16Mb border, so jump to 0x800000
 keepGoing:
   ld  ra3,  0             ; 0 is to select the cart (not the dev kit thingie)
   ld  rbc3, ROM_SIZE4Mb / 0100h; (rbc3 * 0x100) bytes    [0x1000 = 8Mbit] [0800h = 4Mbit]
   ld  xde3, ROM_SIZE4Mb     ;to the dest
   ld  rw3, VECT_FLASHWRITE
   ld      (rWDCR),WD_CLR
   swi	1
   ld      (rWDCR),WD_CLR

exitCopy8M:
   LD     (00006Fh),04Eh  ;reset watchdog
   LD     (00006Eh),0F0h

   ei

   call clearscreen

   ld      (rWDCR),WD_CLR     ;watchdog clear
   ld xhl, 200034h
   ld xde, (xhl)
   jp (xde)

   ret


;; Do a Block Erase
BLOCK_ERASE:
 push xde
 push xix
 push xiy
 ld     xiy, 0 ;set up the counter
 ; don't ask!
 db 0F3h, 0F1h, 055h, 055h, 000h, 0AAh
 db 0F3h, 0F1h, 0AAh, 02Ah, 000h, 055h
 db 0F3h, 0F1h, 055h, 055h, 000h, 0F0h
 db 000h, 000h
 db 0F3h, 0F1h, 055h, 055h, 000h, 0AAh
 db 0F3h, 0F1h, 0AAh, 02Ah, 000h, 055h
 db 0F3h, 0F1h, 055h, 055h, 000h, 080h
 db 0F3h, 0F1h, 055h, 055h, 000h, 0AAh
 db 0F3h, 0F1h, 0AAh, 02Ah, 000h, 055h
; set (XDE) = 50h
;	30h is the command for SST's Sector Erase
;       a sector is 4KBytes
;       50h is the command for SST's Block Erase
;       a block is 64KBytes
;       we want to erase a block
        LD     (XDE),50h
; repeat until (XDE) == FFh || 200000h iterations
startLP:
        CP     (XDE),0FFh ;  do {               ;82 3F FF
        JR     Z,done                           ;66 15
        INC    1,XIY                            ;ED 61
        CP     XIY,001FFFFFh                    ;ED CF FF FF 1F 00
        JR     Z,failed                         ;66 09
        BIT    5,(XDE) ;*                       ;B2 CD
        JR     Z,startLP ; } while (b5 == 0)    ;66 ED
        CP     (XDE),0FFh ;  if (xde) == 0xff   ;82 3F FF
        JR     Z,done                           ;66 02
failed: LD     A,0FFh                           ;21 FF
done:
 db 0F3h, 0F1h, 055h, 055h, 000h, 0AAh
 db 0F3h, 0F1h, 0AAh, 02Ah, 000h, 055h
 db 0F3h, 0F1h, 055h, 055h, 000h, 0F0h
 pop xiy
 pop xix
 pop xde
 RET

FLASH_RAM2:
        ;Same entry conditions as the VECT_FLASHWRITE BIOS call
        ; added W = 1 for progress bar, 0 otherwise
        PUSH   SR                               ;02
        EI     7h                               ;06 07
        PUSH   XIX                              ;3C
        PUSH   XBC  ;*                          ;D7 E6 04
        PUSH   XIY                              ;3D

        ;LD     (00006Eh),014h                     ;08 6E 14
        ;LD     (00006Fh),0B1h                     ;08 6F B1


        LD     XIX,00200000h                    ;44 00 00 20 00
        CP     A,0                              ;C9 D8
        JR     Z,onLow16MChip                         ;66 09
        LD     XIX,00800000h                    ;44 00 00 80 00
onLow16MChip:
        ADD    XDE,XIX                          ;EC 82

        LD     (XIX+5555h),0AAh ;*               ;F3 F1 55 55 00 AA
        LD     (XIX+2AAAh),055h ;*               ;F3 F1 AA 2A 00 55
        LD     (XIX+5555h),0F0h ;*               ;F3 F1 55 55 00 F0

        LD     QBC,0                            ;D7 E6 A8
        ;DB     0E9H, 0ECh, 007h, 021h, 000h     ;??????????
        SLA    07H, XBC                         ;E9 EC 07
        LD     A, 00H                           ;21 00

        CP     w, 1
        JR     nz, noProgressBar0
         ;;SHOW empty prog bar
         push xhl
         push xde
         push xix
         push xbc
          ld xix, EMPTY_PROG_BAR
          ld xde, 0
          ld xhl, 0
          ld b, 20
          calr text_set
         pop xbc
         pop xix
         pop xde
         pop xhl
        noProgressBar0:

nextFlashRam2:
        CP     XHL,00400000h
        JR     lt, contNextFlashRam2
        CP     XHL,00800000h
        JR     ge, contNextFlashRam2
        ADD    XHL,00400000h                     ;jump to the upper 16MB
contNextFlashRam2:
        CP     w, 1
        JR     nz, noProgressBar
        CALR   updateProgressBar
noProgressBar:
	   push w
        CALR   flashAByte2                           ;1E 24 00
        CP     A,0FFh                            ;C9 CF FF
        JR     Z,exit_FLASH_RAM2                         ;66 12
        CALR   flashAByte2                           ;1E 1C 00
	   pop w
        CP     A,0FFh                            ;C9 CF FF
        JR     Z,exit_FLASH_RAM2                         ;66 0A
        DEC    1,XBC                            ;E9 69
        CP     XBC,00000000h                    ;E9 CF 00 00 00 00
        JR     NZ,nextFlashRam2                 ;6E E6

exit_FLASH_RAM2:
        LD     (XIX+5555h),0AAh ;*               ;F3 F1 55 55 00 AA
        LD     (XIX+2AAAh),055h ;*               ;F3 F1 AA 2A 00 55
        LD     (XIX+5555h),0F0h ;*               ;F3 F1 55 55 00 F0

        ;LD     (00006Fh),04Eh                     ;08 6F 4E
        ;LD     (00006Eh),0F0h                     ;08 6E F0

        POP    XIY                              ;5D
        POP    XBC  ;*                          ;D7 E6 05
        POP    XIX                              ;5C
        POP    SR                               ;03
        RET                                     ;0E


flashAByte2:
        LD     (XIX+5555h),0AAh ;*               ;F3 F1 55 55 00 AA
        LD     (XIX+2AAAh),055h ;*               ;F3 F1 AA 2A 00 55
        LD     (XIX+5555h),0A0h ;*               ;F3 F1 55 55 00 A0
        LD     W,(XHL+)                       ;C5 EC 20
        LD     XIY,0                            ;ED A8
continueFlashAByte2:
        LD     (XDE),W ;*                       ;B2 40
        CP     W,(XDE) ;*                       ;82 F0
        JR     Z,exitFlashAByte2                ;66 14
        INC    1,XIY                            ;ED 61
        CP     XIY,0001FFFFh                    ;ED CF FF FF 01 00
        JR     NC,failFlashAByte2               ;6F 08
        BIT    5,(XDE) ;*                       ;B2 CD
        JR     Z,continueFlashAByte2            ;66 EE
        CP     W,(XDE) ;*                       ;82 F0
        JR     Z,exitFlashAByte2                ;66 02
failFlashAByte2:
        LD     A,0FFh                            ;21 FF
exitFlashAByte2:
        INC    1,XDE                            ;EA 61
        RET                                     ;0E

updateProgressBar:
     ;;This is screwing up.  Find out why.  Also, change the progress bar
     ;; to a string of dashes that only TEXT_SETS n length
   PUSH XWA
   PUSH XIY
   PUSH XBC
   PUSH XDE
   PUSH XHL
   PUSH XIX

   srl    07H, XBC                         
   ld qbc, 0

   ld xhl, xbc ; load current progress into xhl
   ld de, (PROGRESS_MAX)

  cp hl, de
  jr gt, exitUpdateProgressBar    ;curr can't be > max

   mul xhl, 20 ; multiply it by 20
   div xhl, de ; divide it by max progress

  cp l, 20
  jr gt, exitUpdateProgressBar

   ld b, 20  ; invert the result so it is in the range 20 down to 0
   sub b, l  ; b now equals result

  cp b, 20
  jr gt, exitUpdateProgressBar

   ;;XIX must be a pointer to a text string
   ;; B is the length of the string
   ;; XHL is the Y position
   ;; XDE is the X position
   ld xix, FILLED_PROG_BAR
   ld xde, 0
   ld xhl, 0

   calr text_set

 exitUpdateProgressBar:
   POP XIX
   POP XHL
   POP XDE
   POP XBC
   POP XIY
   POP XWA

   RET

BACKUP_SAVEGAME:
   ;it seems that some ROMs don't save savegame info in the last 64Kb like they should
   ; until I figure out what to do, I'll just return

   ret

   ;copy 0x70000 for 4Mb
   ; and 0xF0000 for 8Mb

   ld c, (CART_HEAD_OFS + ROMNUM_OFS) 
   cp c, 2
   jr gt, contBackup
   ret
contBackup:


   ld xhl, 070000h

   ld c, (CART_HEAD_OFS + ROMSIZE_OFS)

;;THIS NEEDS FIXING, FOR SURE

;cp c, SIZE_4Mb


   jr z, is4Mb
   ld xhl, 0F0000h
is4Mb:

   LD  XIX, 00200000h ;set up XIX to point to the base of the cart

   ld xbc, 0
   ld c, (CART_HEAD_OFS + ROMNUM_OFS)
    
   ld xde, xhl

backupAddrSet:
   LD     (00006Fh),04Eh  ;reset watchdog  ;08 6F 4E
   add xde, ROM_SIZE4Mb ;add ROM_SIZE to xde3 ROM_NUMBER of times (xhl = ROM_SIZE * ROM_NUMBER)
   dec xbc
   cp xbc, 0
   jr gt, backupAddrSet

   add xde, CART_HEAD_OFS     ;add the cart header offset
   cp xde, 3FFFFFh
   jr le, backupLower16Mb

   add xde, 00400000h
   LD  XIX, 00800000h ;set up XIX to point to the base of the cart

 backupLower16Mb:


   push xhl

   LD     (00006Fh),04Eh  ;reset watchdog  ;08 6F 4E
   calr   BLOCK_ERASE
   LD     (00006Fh),04Eh  ;reset watchdog  ;08 6F 4E

   pop xhl

   ; xhl is src
   add xhl, CART_HEAD_OFS
   ; xde is dest

   ;ld xix, 800000h ;set up XIX to point to the base of the cart
   ;ld xhl, 2F0000h
   ;ld xde, 8F0000h
   
   ld xbc, 010000h; SaveGame area is 0x10000

   calr FLASH_RAM2

   ret

text_set:
   ;;XIX must be a pointer to a text string
   ;; B is the length of the string
   ;; XHL is the Y position
   ;; XDE is the X position
  cp b, 0
  jr z, endTextSetNoPop
   push xhl
   push xde
   push xix
   push xiy
   push xwa
   push xbc

   mul XHL, SCRN_VX_B
   mul XDE, 2
   ld	xiy,_SCR1RAM
   add xiy, XHL
   add xiy, XDE
   ld	wa,0

   ;;;;ld      (rWDCR),WD_CLR     ;watchdog clear
text_set_100:
   ld	a,(xix+)
   ld	(xiy+),wa
   djnz	b,text_set_100

   pop xbc
   pop xwa
   pop xiy
   pop xix
   pop xde
   pop xhl
endTextSetNoPop:
   ret


endCopy8M:


showMenuPage:
   push (ROM_NUMBER)

   ;set NUM_MENU_PAGES
   ld c, (NUM_ROMS)
   dec c                    ;make it 0 based
   ld b, 0
   div bc, ROMSPERMENUPAGE
   ld (NUM_MENU_PAGES), c

   ld b, (CURR_MENU_PAGE)
   cp c, b
   jr z, onLastMenuPage
   ld h, ROMSPERMENUPAGE  ; set h to be number of ROMS on this page
   jr notOnLastMenuPage
  onLastMenuPage:
   ld l, (CURR_MENU_PAGE)
   ld h, 0
   mul hl, ROMSPERMENUPAGE
   ld h, (NUM_ROMS)       ; set h to be number of ROMS on this page
   sub h, l
  notOnLastMenuPage:
   ld (ROMS_ON_CURR_MENU_PAGE), h

   ld l, 8               ; 8 is the middle of the menu area
   ld b, h
   srl b
   sll b
   cp b, h
   jr z, contSetMenuStart ;if they're equal, it's even
   ld b, h
   inc b                  ;if it's odd, add 1 to make it even
  contSetMenuStart:
   srl b
   sub l, b
   ldb (MENU_START), l


    ;display the main-menu background

    ; copy charset to character RAM
    ld bc,(ENDMMCHARSET-MMCHARSET)/2
    ld xde, 0a810h
    ld xhl, MMCHARSET
    ldirw (xde+),(xhl+)

    ; copy palette to character palette RAM
    ld bc, (ENDMMPALETTE-MMPALETTE)/2
    ld xde, 08300h
    ld xhl, MMPALETTE
    ldirw (xde+),(xhl+)

    ; copy TEXT to VRAM
    ld bc, (ENDMMTEXT-MMTEXT)/2
    ld xde, 09800h
    ld xhl, MMTEXT
    ldirw (xde+),(xhl+)

    ; background color
    ld de, (MMBACKGROUND)
    ld xhl, 083e0h
    ld (xhl), de

   ld w, 0
  clearMenuRows: 
   ld xix, CLEAR_ROW
   ld xde, 0   ;col    
   ld xhl, 0   ;row
   ld l, w
   ld b, CLEAR_ROW_END-CLEAR_ROW
   calr text_set
   inc w
   cp w, 20
   jr lt, clearMenuRows


   ld (ROM_NUMBER), 0  ;set default rom number

   ld xwa, ROM_TABLE
   inc xwa
   ld xhl, 0
   ld l, (CURR_MENU_PAGE)
   mul xhl, ROMSPERMENUPAGE
   mul xhl, 10h
   add xwa, xhl
   ld (CURR_ROM_TABLE_PTR), xwa   ;;CURR_ROM_TABLE_PTR starts at ROM_TABLE+1


 showROMs:

   ld c, (ROM_NUMBER)

   ld hl, 0
   ld l, (CURR_MENU_PAGE)
   mul hl, ROMSPERMENUPAGE
   add c, l

   add c, 1                ; make it 1 based instead of 0-based
   ld b, 0
   div bc, 10
   push c                  ; save the 10-based number
   add c, '0'              ; '0', because the 10's column is 0-based
   ;cp c, '0'
   ;jr nz, notMenuZero10s
   ;ld c, ' '               ; instead of 0, use a space
  ;notMenuZero10s:
   ld (TEXT_TEMP), c

   pop c
   ld b, 0
   mul bc, 10

   ld b, (ROM_NUMBER)
   ld hl, 0
   ld l, (CURR_MENU_PAGE)
   mul hl, ROMSPERMENUPAGE
   add b, l

   add b, 1                ; make it 1 based instead of 0-based
   sub b, c
   add b, '0'              ; 
   ld (TEXT_TEMP+1), b
   ld (TEXT_TEMP+2), '-'   ; add the '-'


   ld xhl, (CURR_ROM_TABLE_PTR)  ;curr ptr should be at rom pointer
   ld xhl, (xhl)
   add xhl, 24h                  ;the text start
   ld xde, TEXT_TEMP+3           ;dest +3 to allow for ##-
   ld xbc, 12
   ldir


   ld xix, TEXT_TEMP
   ld xde, 3
   ld xhl, 0
   ld l, (ROM_NUMBER)
   inc l
   add l, (MENU_START)
   ld b, 15
   calr text_set

   inc (ROM_NUMBER)
   ld xbc, (CURR_ROM_TABLE_PTR)
   add xbc, 10h
   ld (CURR_ROM_TABLE_PTR), xbc

   ld l, (ROMS_ON_CURR_MENU_PAGE)
   dec l
   cp (ROM_NUMBER), l
   jrl le, showROMs

   pop (ROM_NUMBER)

   ret

updateMenuPageArrows:
   push xde

   decw (MENU_PAGE_ARROW_COUNTER)

   ld hl, (MENU_PAGE_ARROW_COUNTER)
   sll 2, hl
   srl 15, hl
   cp hl, 0
   jr nz, dontDoSpaces

   ld xix, SPACE
   ld xde, 19
   ld xhl, 8
   ld b, 1
   calr text_set
   ld xde, 0
   calr text_set
   jr endUpdateArrows

  dontDoSpaces:


   ld w, (CURR_MENU_PAGE)
   cp w, (NUM_MENU_PAGES)
   jr z, noNextPage

   ld xix, ARROW
   ld xde, 19
   ld xhl, 8
   ld b, 1
   calr text_set

  noNextPage:

   cp w, 0
   jr z, noPrevPage

   ld xix, L_ARROW
   ld xde, 0
   ld xhl, 8
   ld b, 1
   calr text_set

  noPrevPage:

  endUpdateArrows:

   pop xde
   ret


textMenu:
   ldb (CURR_MENU_PAGE), 0      ;start with the first menu page
   calr showMenuPage

   ld (ROM_NUMBER), 0


   ld xhl, 0
   ld l, (ROM_NUMBER)
   add l, (MENU_START)
   inc l
   ld xix, ARROW
   ld xde, 2
   ld b, 1
   calr text_set

   ldw (MENU_PAGE_ARROW_COUNTER), 0ffffh

update_menu:

   ld xde, 00fffh
waitInputUp:
   ld      (rWDCR),WD_CLR     ;watchdog clear
   dec xde
   cp xde, 0
   jr z, doneWaitInputUp
   calr updateMenuPageArrows
   cp (rSL), 0
   jr nz, waitInputUp
 doneWaitInputUp:

waitInput:
   calr updateMenuPageArrows
   ld      (rWDCR),WD_CLR     ;watchdog clear
   ld a, (rSL)		;read joystick
   cp a, 0
   jr z, waitInput

   bit PADB_A, a		;compare bit setting for button A
   jr nz, menuA

   bit PADB_B, a		;compare bit setting for button B
   jr nz, menuB

   ;bit PADB_OPTION, a		;compare bit setting for OPTION
   ;jr nz, menuO

   bit PADB_RIGHT, a      ;compare bit setting for RIGHT
   jrl nz, menuR
   bit PADB_LEFT, a    ;compare bit setting for LEFT
   jrl nz, menuL

   bit PADB_DOWN, a		;compare bit setting for DOWN
   jrl nz, menuD
   bit PADB_UP, a		;compare bit setting for UP
   jrl nz, menuU

   jr waitInput		;if its 0 (not pressed) loop again

menuA: ;same as B
menuB:
   ld l, (CURR_MENU_PAGE)
   ld h, 0
   mul hl, ROMSPERMENUPAGE
   ld c, (ROM_NUMBER)
   add c, l
   ld (ROM_NUMBER), c        ;add in the offset for the current page

   ld c, (CART_HEAD_OFS + ROMNUM_OFS) 
   cp (ROM_NUMBER), c
   jrl nz, exitMenu
   ldb (ROM_NUMBER), 0FFh    ;load ROM 0FF if user selected same ROM as what's loaded
   jrl exitMenu

menuL:
   ld w, (CURR_MENU_PAGE)
   cp w, 0
   jr z, endPrevMenuPage
   dec (CURR_MENU_PAGE)
   pushb (ROMS_ON_CURR_MENU_PAGE)
   pushb (MENU_START)   ;save old menu start
   calr showMenuPage

   pop l                ;old menu start
   pop a 
   cp a, ROMSPERMENUPAGE
   jr ge, noRNUpdatePrevMenuPage

   ld h, (MENU_START)   ;new menu start
   sub l, h
   ld a, (ROM_NUMBER)
   add a, l
   ld (ROM_NUMBER), a    ;change it to be the same physical location

  noRNUpdatePrevMenuPage:
   
   ld xhl, 0
   ld l, (ROM_NUMBER)
   add l, (MENU_START)
   inc l

   ld xix, ARROW
   ld xde, 2
   ld b, 1
   calr text_set

  endPrevMenuPage:
   jrl update_menu

menuR:
   ld w, (CURR_MENU_PAGE)
   cp w, (NUM_MENU_PAGES)
   jr ge, endNextMenuPage

   inc (CURR_MENU_PAGE)

   pushb (MENU_START)   ;save old menu start
   calr showMenuPage
   pop h                ; old menu start

   ld l, (MENU_START)   ;new menu start
   sub l, h
   ld a, (ROM_NUMBER)
   cp a, l
   jr lt, updateNextTo0
   sub a, l
   jr contUpdateNext
  updateNextTo0:
   ld a, 0
  contUpdateNext:
   ld (ROM_NUMBER), a    ;change it to be the same physical location

   ld w, (ROM_NUMBER)
   ld a, (ROMS_ON_CURR_MENU_PAGE)
   dec a
   cp w, a
   jr le, noRNUpdateNextMenuPage
   ld (ROM_NUMBER), a
  noRNUpdateNextMenuPage:
   
   ld xhl, 0
   ld l, (ROM_NUMBER)
   add l, (MENU_START)
   inc l

   ld xix, ARROW
   ld xde, 2
   ld b, 1
   calr text_set

  endNextMenuPage:
   jrl update_menu


menuU:
   ld d, (ROM_NUMBER) ;temp ROM_NUM
 menuUpLoop:

   cp d, 0
   jr gt, decMenuUp
   
   ld d, (ROMS_ON_CURR_MENU_PAGE)
   ;inc d

decMenuUp:
   dec d

   push d
   ld xhl, 0
   ld l, (ROM_NUMBER)
   add l, (MENU_START)
   inc l

   ld xix, SPACE
   ld xde, 2
   ld b, 1
   calr text_set
   pop d

   ld (ROM_NUMBER), d

   ld xhl, 0
   ld l, (ROM_NUMBER)
   add l, (MENU_START)
   inc l

   ld xix, ARROW
   ld xde, 2
   ld b, 1
   calr text_set

 exitMenuUp:
   jrl update_menu

menuD:
   ld d, (ROM_NUMBER) ;temp ROM_NUM
 menuDnLoop:
   ld l, (ROMS_ON_CURR_MENU_PAGE)
   dec l
   cp d, l
   jr lt, incMenuDn
   
   ld d, 0
   dec d

incMenuDn:

   inc d

   push d
   ld xhl, 0
   ld l, (ROM_NUMBER)
   add l, (MENU_START)
   inc l

   ld xix, SPACE
   ld xde, 2
   ld b, 1
   calr text_set
   pop d

   ld (ROM_NUMBER), d

   ld xhl, 0
   ld l, (ROM_NUMBER)
   add l, (MENU_START)
   inc l

   ld xix, ARROW
   ld xde, 2
   ld b, 1
   calr text_set

 exitMenuDn:

   jrl update_menu

exitMenu:
   ret

load_progress_bar:
    ; copy charset to character RAM
    ld bc,(ENDPROGBARTILES-PROGBARTILES)/2
    ld xde, 0a010h     ;;to the first few of the charset
    ld xhl, PROGBARTILES
    ldirw (xde+),(xhl+)
    ret

loading_warning:

    ; copy charset to character RAM
    ld bc,(ENDCHARSET-CHARSET)/2
    ld xde, 0a810h
    ld xhl, CHARSET
    ldirw (xde+),(xhl+)

    ; copy palette to character palette RAM
    ld bc, (ENDPALETTE-PALETTE)/2
    ld xde, 08300h
    ld xhl, PALETTE
    ldirw (xde+),(xhl+)

    ; copy TEXT to VRAM
    ld bc, (ENDTEXT-TEXT)/2
    ld xde, 09800h
    ld xhl, TEXT
    ldirw (xde+),(xhl+)

    ; background color
    ld de, (BACKGROUND) 
    ld xhl, 083e0h
    ld (xhl), de


   ld xhl, LOADING       ;source
   ld xde, TEXT_TEMP     ;dest
   ld xbc, LOADING_END - LOADING
   ldir

   ld      (rWDCR),WD_CLR     ;watchdog clear

   ld xhl, (ROM_NUMBER)
   sll 4, xhl                 ;multiply by 10h
   ld xwa, ROM_TABLE
   inc xwa
   add xhl, xwa
   ld (CURR_ROM_TABLE_PTR), xhl
   ld xhl, (xhl)
   add xhl, 24h

   ld xbc, (ROM_NUMBER)

   ;ld xhl, 0
;romNameAddrSet:
   ;cp xbc, 0
   ;jr z, doneAddingLW
   ;add xhl, ROM_SIZE4Mb ;add ROM_SIZE to xhl3 ROM_NUMBER of times (xhl3 = ROM_SIZE * ROM_NUMBER)
   ;dec xbc
   ;jr romNameAddrSet
;doneAddingLW:
   ;add xhl, 200024h            ;source
   ;cp xhl, 3FFFFFh
   ;jr le, loadWarnInLower16Mb
   ;add xhl, 400000h
;loadWarnInLower16Mb:
   ld xde, TEXT_TEMP           ;dest
   add xde, LOADING_END - LOADING
   ld xbc, 12
   ldir

   ;set the text to the screen
   ld xix, TEXT_TEMP
   ld xhl, 18    ;x
   ld xde, 0    ;y
   ld xbc, 0
   ld c, 12 + (LOADING_END - LOADING)

   push xix
   
   add xix, xbc
   ;inc xix
   inc c
  stripEndString:
   dec xix
   dec c
   cp (xix), 20h
   jr z, stripEndString
   cp (xix), 00h
   jr z, stripEndString

   pop xix

   ld e, 20
   sub e, c
   srl e
   ld b, c

   calr text_set

   ret

strncmp:
   ; compare 2 strings (xhl), (xde)
   ; return a = 0 on same, a = 1 on different
   ; xbc = num of bytes to compare
   ld a, 1
   push xhl
   push xde
   push xbc
   push w
   cp xbc, 0
   jr z, exitstrncmp
 contCompare:
   ld      (rWDCR),WD_CLR     ;watchdog clear
   ld w, (xde+)
   cp (xhl+), w
   jr nz, exitstrncmp
   dec xbc
   cp xbc, 0
   jr gt, contCompare
   ld a, 0
 exitstrncmp:
   pop w
   pop xbc
   pop xde
   pop xhl
   ret


setROMSize
   ; uses CURR_ROM_TABLE_PTR to set ROM_SIZE
   push xwa
   push xbc

   ld xwa, (CURR_ROM_TABLE_PTR)
   add xwa, 4    ;skip the ROM pointer, the next 4 are the size
   ld xbc, (xwa)
   ld (ROM_SIZE), xbc

   pop xbc
   pop xwa
   ret


ColorSet:
        DW      0         ; Don't Care (Transparent)
        RGBSet  255,0,0   ; Red
        RGBSet  0,255,0   ; Green
        RGBSet  0,0,0     ; Black
ColorSetEnd:

   align	2
   db "Strings  "
A_TO_RUN:
   db	"A runs "
A_TO_RUN_END:

A_TO_LOAD:
   db	"A loads "
A_TO_LOAD_END:

B_TO_LOAD:
   db	"B loads "
B_TO_LOAD_END:

LOADING:
   db	"Loading "
LOADING_END:

DO_NOT_TURN_OFF:
   db	"Do not turn off!"
DO_NOT_TURN_OFF_END:

UP_DN_SELECT:
   db	"Use UP/DN to select."
UP_DN_SELECT_END:

EMPTY_PROG_BAR:
   db 01h, 02h, 02h, 02h, 02h, 02h, 02h, 02h, 02h, 02h
   db 02h, 02h, 02h, 02h, 02h, 02h, 02h, 02h, 02h, 03h
EMPTY_PROG_BAR_END:

FILLED_PROG_BAR:
   db 04h, 05h, 05h, 05h, 05h, 05h, 05h, 05h, 05h, 05h
   db 05h, 05h, 05h, 05h, 05h, 05h, 05h, 05h, 05h, 06h
FILLED_PROG_BAR_END:

CLEAR_ROW:
   db 0h, 0h, 0h, 0h, 0h, 0h, 0h, 0h, 0h, 0h
   db 0h, 0h, 0h, 0h, 0h, 0h, 0h, 0h, 0h, 0h
CLEAR_ROW_END:


ARROW:
   db 10h
ARROW_END:

L_ARROW:
   db 11h
L_ARROW_END:

SPACE:
   db	" "
SPACE_END:

   align	2

SYSTEM_CALL:
   SystemCallCode
OS_VERSION:
   OsVersionCode

   include "MainMenu.inc"  ; Main Menu graphic
   include "lik-sang.inc"  ; Lik-Sang graphic
   include "pbartile.inc"  ; Lik-Sang graphic

   db 0EDh, 0EDh, 0EDh, 0EDh ;signature
ROM_TABLE:
   ;this ROM_TABLE will take this form

   ;first byte = number of ROMs
   db 02h  ;fake number of roms

   ; ROM 0
   ;  each ROM gets 0x10 bytes in the table
   ;    bytes 0-3 = ROM start address
   db 00h, 40h, 28h, 00h
   ;    bytes 4-7 = ROM size in bytes
   db 00h, 80h, 00h, 00h
   ;    bytes 8-F = reserved for future use
   db 00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h

   ; ROM 1
   ;    bytes 0-3 = ROM start address
   db 00h, 0C0h, 28h, 00h
   ;    bytes 4-7 = ROM size in bytes
   db 00h, 80h, 00h, 00h
   ;    bytes 8-F = reserved for future use
   db 00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h

   ; ROM 2
   ;    bytes 0-3 = ROM start address
   db 00h, 00h, 00h, 00h
   ;    bytes 4-7 = ROM size in bytes
   db 00h, 00h, 00h, 00h
   ;    bytes 8-F = reserved for future use
   db 00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h

   ; ROM 3
   ;    bytes 0-3 = ROM start address
   db 00h, 00h, 00h, 00h
   ;    bytes 4-7 = ROM size in bytes
   db 00h, 00h, 00h, 00h
   ;    bytes 8-F = reserved for future use
   db 00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h

   ; ROM 4
   ;    bytes 0-3 = ROM start address
   db 00h, 00h, 00h, 00h
   ;    bytes 4-7 = ROM size in bytes
   db 00h, 00h, 00h, 00h
   ;    bytes 8-F = reserved for future use
   db 00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h

   ; ROM 5
   ;    bytes 0-3 = ROM start address
   db 00h, 00h, 00h, 00h
   ;    bytes 4-7 = ROM size in bytes
   db 00h, 00h, 00h, 00h
   ;    bytes 8-F = reserved for future use
   db 00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h

   ; ROM 6
   ;    bytes 0-3 = ROM start address
   db 00h, 00h, 00h, 00h
   ;    bytes 4-7 = ROM size in bytes
   db 00h, 00h, 00h, 00h
   ;    bytes 8-F = reserved for future use
   db 00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h

   ; ROM 7
   ;    bytes 0-3 = ROM start address
   db 00h, 00h, 00h, 00h
   ;    bytes 4-7 = ROM size in bytes
   db 00h, 00h, 00h, 00h
   ;    bytes 8-F = reserved for future use
   db 00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h

   ; ROM 8
   ;    bytes 0-3 = ROM start address
   db 00h, 00h, 00h, 00h
   ;    bytes 4-7 = ROM size in bytes
   db 00h, 00h, 00h, 00h
   ;    bytes 8-F = reserved for future use
   db 00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h

   ;org HEADER_OFS + 007FFFFh   ;set to 4Mb
   org HEADER_OFS + 0003FFFh
   db 0FFh
