;**************************************************************************************************
;*
;* Plink.asm: Pong link-up for the NGPC.
;*
;* Author   : Steve Robb
;*
;**************************************************************************************************

;_DEBUG      Equ 1
_FAKELINKUP Equ 3

;*****************
;* Include files *
;*****************

	Include "StHandlr.inc"

	Include "Backgrnd.inc"
	Include "Fade.inc"
	Include "Font.inc"
	Include "Input.inc"
	Include "LinkUp.inc"
	Include "Memory.inc"
	Include "Raster.inc"
	Include "Scroll.inc"
	Include "Sprite.inc"
	Include "Tile.inc"
	Include "Window.inc"


;***********
;* Defines *
;***********

CART_TITLE_12_CHARS Equ "PLINK       "

BALL_SPEED    Equ                        3.0
BALL_DIAMETER Equ                          4
BAT_LENGTH    Equ                         24
BAT_WIDTH     Equ                          4
BORDER_SIZE   Equ                          2
HIT_POINTS    Equ BAT_LENGTH + BALL_DIAMETER
EXTREME_ANGLE Equ                       60.0


;**********
;* States *
;**********

States:
	State_U__ GameState_FrontEndInitialise
	State_U_H GameState_FrontEnd
	State_U__ GameState_FrontEndFadeOutInitialise
	State_U__ GameState_FrontEndFadeOut
	State_U__ GameState_NegotiateInitialise
	State_UD_ GameState_Negotiate
	State_UD_ GameState_NegotiateSuccessfulInitialise
	State_UD_ GameState_NegotiateSuccessful
	State_U__ GameState_GameInitialise
	State_U__ GameState_PlayerArrows
	State_UD_ GameState_Game


;*************
;* Functions *
;*************

BallVelocityX_Degrees Function nAngle, Trunc( Sin((nAngle) * CONSTPI / 180.0) * BALL_SPEED * 256.0)
BallVelocityY_Degrees Function nAngle, Trunc(-Cos((nAngle) * CONSTPI / 180.0) * BALL_SPEED * 256.0)
BallVelocityX_Radians Function nAngle, Trunc( Sin( nAngle                   ) * BALL_SPEED * 256.0)
BallVelocityY_Radians Function nAngle, Trunc(-Cos( nAngle                   ) * BALL_SPEED * 256.0)


;*********
;* Tiles *
;*********

Tiles_Start:

	Tile_Blank

	Include "Tiles.ntt"
	Include "URL.ntt"
	Include "Plink.ntt"

Tiles_Font_Start:

	Include "Font.ntt"

Tiles_Link_Start:

	Include "LinkGame.ntt"

Tiles_LinkCable_Start:

	Include "LinkCabl.ntt"
	Include "LinkCabH.ntt"

Tiles_Numbers_Start:

	Include "Numbers.ntt"

Tiles_Arrow_Start:

	Include "Arrow.ntt"

Tiles_End:

Tiles_Font_Index      Equ FTile_GetTileIndex(Tiles_Font_Start     , Tiles_Start)
Tiles_Link_Index      Equ FTile_GetTileIndex(Tiles_Link_Start     , Tiles_Start)
Tiles_LinkCable_Index Equ FTile_GetTileIndex(Tiles_LinkCable_Start, Tiles_Start)
Tiles_Numbers_Index   Equ FTile_GetTileIndex(Tiles_Numbers_Start  , Tiles_Start)
Tiles_Arrow_Index     Equ FTile_GetTileIndex(Tiles_Arrow_Start    , Tiles_Start)


;************
;* Palettes *
;************

Palettes_Start:

	DW FRGB(  0,   0,   0)
	DW FRGB(255, 255, 255)
	DW FRGB(255, 255, 255)
	DW FRGB(255, 255, 255)

	DW FRGB(  0,   0,   0)
	DW FRGB(  0,   0,   0)
	DW FRGB(  0,   0,   0)
	DW FRGB(  0,   0,   0)

	DW FRGB(  0,   0,   0)
	DW FRGB( 86,  86,   0)
	DW FRGB(171, 171,   0)
	DW FRGB(255, 255,   0)

	DW FRGB(  0,   0,   0)
	DW FRGB(  0,   0,   0)
	DW FRGB(255, 255,   0)
	DW FRGB(  0, 255, 255)

	DW FRGB(  0,   0,   0)
	DW FRGB( 32,   0, 112)
	DW FRGB(128, 144, 224)
	DW FRGB(208, 208, 240)

Palettes_End:

Palettes_LinkUp_Start:

	DW FRGB(  0,   0,   0)
	DW FRGB(  0,   0,   0)
	DW FRGB(144, 160, 208)
	DW FRGB(192, 192, 224)

	DW FRGB(  0,   0,   0)
	DW FRGB(  0,   0,   0)
	DW FRGB(144, 160, 208)
	DW FRGB(240,   0,   0)

	DW FRGB(  0,   0,   0)
	DW FRGB(  0,   0,   0)
	DW FRGB(144, 160, 208)
	DW FRGB( 80,  80, 160)

	DW FRGB(  0,   0,   0)
	DW FRGB( 48,  48,  48)
	DW FRGB(128, 128, 128)
	DW FRGB(240, 240, 240)

	DW FRGB(  0,   0,   0)
	DW FRGB(  0,   0,   0)
	DW FRGB(144, 160, 208)
	DW FRGB(144, 144, 208)

	DW FRGB(  0,   0,   0)
	DW FRGB(  0,   0,   0)
	DW FRGB(144, 160, 208)
	DW FRGB(176, 176, 240)

	DW FRGB(  0,   0,   0)
	DW FRGB(  0,   0,   0)
	DW FRGB( 96,  96, 160)
	DW FRGB(192, 192, 224)

	DW FRGB(  0,   0,   0)
	DW FRGB(  0,   0,   0)
	DW FRGB( 96,  96, 160)
	DW FRGB(208, 208, 224)

	DW FRGB(  0,   0,   0)
	DW FRGB( 32,  32,  32)
	DW FRGB( 80,  80,  80)
	DW FRGB(112, 112, 112)

	DW FRGB(  0,   0,   0)
	DW FRGB(  0,   0,   0)
	DW FRGB(255, 255,   0)
	DW FRGB(  0, 255, 255)

Palettes_LinkUp_End:

Palette_LinkUpFont_Index Equ 9


;*************
;* Variables *
;*************

	UserRAM_DB bPlayer1X
	UserRAM_DB bPlayer1Y
	UserRAM_DB bPlayer2X
	UserRAM_DB bPlayer2Y
	UserRAM_DB bPlayer1Score
	UserRAM_DB bPlayer2Score
	UserRAM_DW wBallX
	UserRAM_DW wBallY
	UserRAM_DW wBallVelocityX
	UserRAM_DW wBallVelocityY
	UserRAM_DB bScrollOffset
	UserRAM_DB bWait
	UserRAM_DB bWait2


;**************
;* Bat angles *
;**************

BatAngles:
	For nOffset, 0, HIT_POINTS - 1

		DW BallVelocityX_Degrees(90 + ((nOffset - ((HIT_POINTS - 1.0) / 2.0)) * EXTREME_ANGLE / ((HIT_POINTS - 1.0) / 2.0)))
		DW BallVelocityY_Degrees(90 + ((nOffset - ((HIT_POINTS - 1.0) / 2.0)) * EXTREME_ANGLE / ((HIT_POINTS - 1.0) / 2.0)))

	EndM


;******************
;* Raster colours *
;******************

RasterColours:
	For nColour, 0, 128, 32
		DW FRGB(    240, nColour,       0)
	EndM
	For nColour, 128, 224, 16
		DW FRGB(    240, nColour,       0)
	EndM
	For nColour, 240, 128, -16
		DW FRGB(nColour,     240,       0)
	EndM
	For nColour, 128, 16, -32
		DW FRGB(nColour,     240,       0)
	EndM
	For nColour, 0, 128, 32
		DW FRGB(      0,     240, nColour)
	EndM
	For nColour, 128, 224, 16
		DW FRGB(      0,     240, nColour)
	EndM
	For nColour, 240, 128, -16
		DW FRGB(      0, nColour,     240)
	EndM
	For nColour, 128, 16, -32
		DW FRGB(      0, nColour,     240)
	EndM
	For nColour, 0, 128, 32
		DW FRGB(nColour,       0,     240)
	EndM
	For nColour, 128, 224, 16
		DW FRGB(nColour,       0,     240)
	EndM
	For nColour, 240, 128, -16
		DW FRGB(    240,       0, nColour)
	EndM
	For nColour, 128, 16, -32
		DW FRGB(    240,       0, nColour)
	EndM


;***********
;* Numbers *
;***********

Tiles_Number_TL_Index Equ Tiles_Numbers_Index
Tiles_Number_T__Index Equ Tiles_Numbers_Index + 1
Tiles_Number__L_Index Equ Tiles_Numbers_Index + 2
Tiles_Number____Index Equ Tiles_Numbers_Index + 3

Palette_Number_Index Equ 0

NumberGlyphs:

NumberGlyph_Blank:
	DW FScroll_RAMWord(                    0, Palette_Number_Index, FALSE, FALSE), FScroll_RAMWord(                    0, Palette_Number_Index, FALSE, FALSE)
	DW FScroll_RAMWord(                    0, Palette_Number_Index, FALSE, FALSE), FScroll_RAMWord(                    0, Palette_Number_Index, FALSE, FALSE)
	DW FScroll_RAMWord(                    0, Palette_Number_Index, FALSE, FALSE), FScroll_RAMWord(                    0, Palette_Number_Index, FALSE, FALSE)

NumberGlyph_0:
	DW FScroll_RAMWord(Tiles_Number_TL_Index, Palette_Number_Index, FALSE, FALSE), FScroll_RAMWord(Tiles_Number__L_Index, Palette_Number_Index, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Number__L_Index, Palette_Number_Index, FALSE, FALSE), FScroll_RAMWord(Tiles_Number__L_Index, Palette_Number_Index, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Number_T__Index, Palette_Number_Index, FALSE, FALSE), FScroll_RAMWord(Tiles_Number____Index, Palette_Number_Index, FALSE, FALSE)

NumberGlyph_1:
	DW FScroll_RAMWord(                    0, Palette_Number_Index, FALSE, FALSE), FScroll_RAMWord(Tiles_Number__L_Index, Palette_Number_Index, FALSE, FALSE)
	DW FScroll_RAMWord(                    0, Palette_Number_Index, FALSE, FALSE), FScroll_RAMWord(Tiles_Number__L_Index, Palette_Number_Index, FALSE, FALSE)
	DW FScroll_RAMWord(                    0, Palette_Number_Index, FALSE, FALSE), FScroll_RAMWord(Tiles_Number____Index, Palette_Number_Index, FALSE, FALSE)

NumberGlyph_2:
	DW FScroll_RAMWord(Tiles_Number_T__Index, Palette_Number_Index, FALSE, FALSE), FScroll_RAMWord(Tiles_Number__L_Index, Palette_Number_Index, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Number_TL_Index, Palette_Number_Index, FALSE, FALSE), FScroll_RAMWord(Tiles_Number____Index, Palette_Number_Index, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Number_T__Index, Palette_Number_Index, FALSE, FALSE), FScroll_RAMWord(Tiles_Number____Index, Palette_Number_Index, FALSE, FALSE)

NumberGlyph_3:
	DW FScroll_RAMWord(Tiles_Number_T__Index, Palette_Number_Index, FALSE, FALSE), FScroll_RAMWord(Tiles_Number__L_Index, Palette_Number_Index, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Number_T__Index, Palette_Number_Index, FALSE, FALSE), FScroll_RAMWord(Tiles_Number__L_Index, Palette_Number_Index, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Number_T__Index, Palette_Number_Index, FALSE, FALSE), FScroll_RAMWord(Tiles_Number____Index, Palette_Number_Index, FALSE, FALSE)

NumberGlyph_4:
	DW FScroll_RAMWord(Tiles_Number__L_Index, Palette_Number_Index, FALSE, FALSE), FScroll_RAMWord(Tiles_Number__L_Index, Palette_Number_Index, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Number_T__Index, Palette_Number_Index, FALSE, FALSE), FScroll_RAMWord(Tiles_Number__L_Index, Palette_Number_Index, FALSE, FALSE)
	DW FScroll_RAMWord(                    0, Palette_Number_Index, FALSE, FALSE), FScroll_RAMWord(Tiles_Number____Index, Palette_Number_Index, FALSE, FALSE)

NumberGlyph_5:
	DW FScroll_RAMWord(Tiles_Number_TL_Index, Palette_Number_Index, FALSE, FALSE), FScroll_RAMWord(Tiles_Number____Index, Palette_Number_Index, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Number_T__Index, Palette_Number_Index, FALSE, FALSE), FScroll_RAMWord(Tiles_Number__L_Index, Palette_Number_Index, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Number_T__Index, Palette_Number_Index, FALSE, FALSE), FScroll_RAMWord(Tiles_Number____Index, Palette_Number_Index, FALSE, FALSE)

NumberGlyph_6:
	DW FScroll_RAMWord(Tiles_Number_TL_Index, Palette_Number_Index, FALSE, FALSE), FScroll_RAMWord(Tiles_Number____Index, Palette_Number_Index, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Number_TL_Index, Palette_Number_Index, FALSE, FALSE), FScroll_RAMWord(Tiles_Number__L_Index, Palette_Number_Index, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Number_T__Index, Palette_Number_Index, FALSE, FALSE), FScroll_RAMWord(Tiles_Number____Index, Palette_Number_Index, FALSE, FALSE)

NumberGlyph_7:
	DW FScroll_RAMWord(Tiles_Number_T__Index, Palette_Number_Index, FALSE, FALSE), FScroll_RAMWord(Tiles_Number__L_Index, Palette_Number_Index, FALSE, FALSE)
	DW FScroll_RAMWord(                    0, Palette_Number_Index, FALSE, FALSE), FScroll_RAMWord(Tiles_Number__L_Index, Palette_Number_Index, FALSE, FALSE)
	DW FScroll_RAMWord(                    0, Palette_Number_Index, FALSE, FALSE), FScroll_RAMWord(Tiles_Number____Index, Palette_Number_Index, FALSE, FALSE)

NumberGlyph_8:
	DW FScroll_RAMWord(Tiles_Number_TL_Index, Palette_Number_Index, FALSE, FALSE), FScroll_RAMWord(Tiles_Number__L_Index, Palette_Number_Index, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Number_TL_Index, Palette_Number_Index, FALSE, FALSE), FScroll_RAMWord(Tiles_Number__L_Index, Palette_Number_Index, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Number_T__Index, Palette_Number_Index, FALSE, FALSE), FScroll_RAMWord(Tiles_Number____Index, Palette_Number_Index, FALSE, FALSE)

NumberGlyph_9:
	DW FScroll_RAMWord(Tiles_Number_TL_Index, Palette_Number_Index, FALSE, FALSE), FScroll_RAMWord(Tiles_Number__L_Index, Palette_Number_Index, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Number_T__Index, Palette_Number_Index, FALSE, FALSE), FScroll_RAMWord(Tiles_Number__L_Index, Palette_Number_Index, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Number_T__Index, Palette_Number_Index, FALSE, FALSE), FScroll_RAMWord(Tiles_Number____Index, Palette_Number_Index, FALSE, FALSE)


;****************
;* NGPC graphic *
;****************

NGPC_Graphic:
	DW FScroll_RAMWord(Tiles_Link_Index +  0, 0, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Link_Index +  1, 1, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Link_Index +  2, 0, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Link_Index +  3, 0, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Link_Index +  4, 1, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Link_Index +  5, 0, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Link_Index +  6, 0, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Link_Index +  7, 0, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Link_Index +  8, 2, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Link_Index +  9, 3, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Link_Index + 10, 3, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Link_Index + 11, 3, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Link_Index + 12, 0, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Link_Index + 13, 0, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Link_Index + 14, 4, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Link_Index + 15, 0, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Link_Index + 16, 3, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Link_Index + 17, 3, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Link_Index + 18, 3, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Link_Index + 19, 0, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Link_Index + 20, 5, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Link_Index + 21, 0, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Link_Index + 22, 4, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Link_Index + 23, 3, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Link_Index + 24, 3, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Link_Index + 25, 3, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Link_Index + 26, 0, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Link_Index + 27, 0, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Link_Index + 28, 6, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Link_Index + 29, 6, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Link_Index + 30, 6, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Link_Index + 31, 7, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Link_Index + 32, 6, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Link_Index + 33, 6, FALSE, FALSE)
	DW FScroll_RAMWord(Tiles_Link_Index + 34, 6, FALSE, FALSE)


;*********
;* Fonts *
;*********

	Font_Define_ZeroSpace font, "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789,.!-?\I", Tiles_Font_Index


;***********
;* Strings *
;***********

	Font_String String_PressAToStart, font, "PRESS \IA\I TO START"
	Font_String String_Waiting      , font, "WAITING..."
	Font_String String_Connection   , font, " CONNECTION  "
	Font_String String_Established  , font, "ESTABLISHED! "


;*************
;* Functions *
;*************

;**************************************************************************************************
;*
;* Function    - GameState_FrontEndInitialise_Update
;* Parameters  - void
;* Returns     - void
;* Description - Update handler for the GameState_FrontEndInitialise state.
;* Author      - Steve Robb
;*
;**************************************************************************************************

GameState_FrontEndInitialise_Update:

	;/* Initialise link-up */
	LinkUp_Initialise

	;/* Set background */
	Background_SetPaletteEntry 0, FRGB(0, 128, 255)
	Background_SetPaletteIndex 0
	Background_Enable

	;/* Set window */
	Window_Set 0, 0, 160, 152

	;/* Install tiles */
	Tile_Install Tiles_Start, Tiles_End, 0

	;/* Install palettes for scrolls */
	Scroll_InstallPalettes 1, Palettes_Start, Palettes_End, 0
	Scroll_InstallPalettes 2, Palettes_Start, Palettes_End, 0

	;/* Clear scroll planes */
	Scroll_Clear 1, FScroll_RAMWord(0, 0, FALSE, FALSE)
	Scroll_Clear 2, FScroll_RAMWord(2, 0, FALSE, FALSE)

	;/* Display logo */
	Scroll_SetRectangle 1, 1, 1, 18, 9, FScroll_RAMWord(29, 1, FALSE, FALSE)

	;/* Display URL */
	Scroll_SetRectangle 1, 1, 17, 18, 1, FScroll_RAMWord(11, 1, FALSE, FALSE), Vert

	;/* Display "PRESS 'A' TO START" */
	Font_WriteScroll 1, String_PressAToStart, 4, 1, 12

	;/* Reset scroll offset */
	ldb (bScrollOffset), 0

	;/* Advance state */
	State_Advance

	;/* Return from function */
	ret

;**********************************************
;* End of GameState_FrontEndInitialise_Update *
;**********************************************


;**************************************************************************************************
;*
;* Function    - GameState_FrontEnd_Update
;* Parameters  - void
;* Returns     - void
;* Description - Update handler for the GameState_Negotiate state.
;* Author      - Steve Robb
;*
;**************************************************************************************************

GameState_FrontEnd_Update:

	;/* Increment offset */
	incb (bScrollOffset)

	;/* Check if A is pressed */
	Input_IsAPressed
	jr z, GameState_FrontEnd_Update_NoA

	;/* Advance state */
	State_Advance

GameState_FrontEnd_Update_NoA:
	;/* Return from function */
	ret

;************************************
;* End of GameState_FrontEnd_Update *
;************************************


;**************************************************************************************************
;*
;* Interrupt   - GameState_FrontEnd_HInt
;* Parameters  - void
;* Returns     - void
;* Description - H-Int handler for the GameState_FrontEnd state.
;* Author      - Steve Robb
;*
;**************************************************************************************************

GameState_FrontEnd_HInt:

	;/* Store register */
	push xhl
	push   e

	;/* Get current raster line */
	ldb e, (ARaster_YPosition)

	;/* Get current raster line */
	ld  xhl, 0

	;/* Apply offset */
	ldb  l, (bScrollOffset)
	addb l,               e
	andb l,              63

	;/* Determine address of colour */
	sllw   1,            hl
	add  xhl, RasterColours

	;/* Get colour from address */
	ldw hl, (xhl)

	;/* Alter palette colour */
	Scroll_SetPaletteEntry 1, 1, 3, hl

	;/* Get current raster line */
	ld  xhl, 0

	;/* Apply offset */
	ldb  l, (bScrollOffset)
	neg                   l
	addb l,               e
	neg                   l
	andb l,              63

	;/* Determine address of colour */
	sllw   1,            hl
	add  xhl, RasterColours

	;/* Get colour from address */
	ldw hl, (xhl)

	;/* Alter palette colour */
	Scroll_SetPaletteEntry 1, 1, 2, hl

	;/* Restore register */
	pop   e
	pop xhl

	;/* Return from interrupt */
	reti

;**********************************
;* End of GameState_FrontEnd_HInt *
;**********************************


;**************************************************************************************************
;*
;* Function    - GameState_FrontEndFadeOutInitialise_Update
;* Parameters  - void
;* Returns     - void
;* Description - Update handler for the GameState_FrontEndFadeOutInitialise state.
;* Author      - Steve Robb
;*
;**************************************************************************************************

GameState_FrontEndFadeOutInitialise_Update:

	;/* Reset wait timer */
	ldb (bWait), 0

	;/* Advance state */
	State_Advance
	
	;/* Return from function */
	ret

;*****************************************************
;* End of GameState_FrontEndFadeOutInitialise_Update *
;*****************************************************


;**************************************************************************************************
;*
;* Function    - GameState_FrontEndFadeOut_Update
;* Parameters  - void
;* Returns     - void
;* Description - Update handler for the GameState_FrontEndFadeOut state.
;* Author      - Steve Robb
;*
;**************************************************************************************************

GameState_FrontEndFadeOut_Update:

	;/* Check if scroll should be faded */
	incb (bWait)
	cpb  (bWait),                              3
	jp        ne, GameState_FrontEndFadeOut_Skip
;	Input_IsAPressed
;	jp z, GameState_FrontEndFadeOut_Skip

	;/* Reset wait timer */
	ldb (bWait), 0

	;/* Fade scroll 1 */
	Fade_ToBlack GameState_FrontEndFadeOut_ScrollFinished

GameState_FrontEndFadeOut_Skip:
	;/* Return from function */
	ret

GameState_FrontEndFadeOut_ScrollFinished:

	;/* Advance state */
	State_Advance
	
	;/* Return from function */
	ret

;*******************************************
;* End of GameState_FrontEndFadeOut_Update *
;*******************************************


;**************************************************************************************************
;*
;* Function    - GameState_NegotiateInitialise_Update
;* Parameters  - void
;* Returns     - void
;* Description - Update handler for the GameState_NegotiateInitialise state.
;* Author      - Steve Robb
;*
;**************************************************************************************************

GameState_NegotiateInitialise_Update:

	;/* Set window */
	Window_Set 0, 0, 160, 152

	;/* Install tiles */
	Tile_Install Tiles_Start, Tiles_End, 0

	;/* Clear scroll planes */
	Scroll_Clear 1, FScroll_RAMWord(0, 0, FALSE, FALSE)
	Scroll_Clear 2, FScroll_RAMWord(2, 0, FALSE, FALSE)

	;/* Display NGPCs */
	Scroll_CopyRectangle 1,  2, 7, 7, 5, NGPC_Graphic
	Scroll_CopyRectangle 1, 11, 7, 7, 5, NGPC_Graphic

	;/* Display link cable ends */
	Scroll_SetTile 1,  7, 4, FScroll_RAMWord(Tiles_LinkCable_Index    , 8, FALSE, FALSE)
	Scroll_SetTile 1,  7, 5, FScroll_RAMWord(Tiles_LinkCable_Index + 1, 8, FALSE, FALSE)
	Scroll_SetTile 1,  7, 6, FScroll_RAMWord(Tiles_LinkCable_Index + 2, 8, FALSE, FALSE)
	Scroll_SetTile 1, 16, 4, FScroll_RAMWord(Tiles_LinkCable_Index    , 8, TRUE , FALSE)
	Scroll_SetTile 1, 16, 5, FScroll_RAMWord(Tiles_LinkCable_Index + 1, 8, FALSE, FALSE)
	Scroll_SetTile 1, 16, 6, FScroll_RAMWord(Tiles_LinkCable_Index + 2, 8, FALSE, FALSE)

	;/* Display waiting text */
	Font_WriteScroll 1, String_Waiting, Palette_LinkUpFont_Index, 5, 13

	;/* Display link cable middle */
	For x, 8, 15

		Scroll_SetTile 1, x, 4, FScroll_RAMWord(Tiles_LinkCable_Index + 3, 8, FALSE, FALSE)

	EndM

	;/* Set background */
	Background_SetPaletteEntry 0, FRGB(0, 128, 255)
	Background_SetPaletteIndex 0
	Background_Enable

	;/* Install palettes for scrolls */
	Scroll_InstallPalettes 1, Palettes_LinkUp_Start, Palettes_LinkUp_End, 0
	Scroll_InstallPalettes 2, Palettes_Start       , Palettes_End       , 0

	;/* Advance state */
	State_Advance

	;/* Return from function */
	ret

;***********************************************
;* End of GameState_NegotiateInitialise_Update *
;***********************************************


;**************************************************************************************************
;*
;* Function    - GameState_Negotiate_Update
;* Parameters  - void
;* Returns     - void
;* Description - Update handler for the GameState_Negotiate state.
;* Author      - Steve Robb
;*
;**************************************************************************************************

GameState_Negotiate_Update:

	;/* Negotiate link-up */
	LinkUp_Negotiate GameState_Negotiate_Update_Success

	;/* Return from function */
	ret

GameState_Negotiate_Update_Success:
	;/* Advance state */
	State_Advance

	;/* Return from function */
	ret

;*************************************
;* End of GameState_Negotiate_Update *
;*************************************


;**************************************************************************************************
;*
;* Function    - GameState_NegotiateSuccessfulInitialise_Update
;* Parameters  - void
;* Returns     - void
;* Description - Update handler for the GameState_NegotiateSuccessfulInitialise state.
;* Author      - Steve Robb
;*
;**************************************************************************************************

GameState_NegotiateSuccessfulInitialise_Update:

	;/* Display success text */
	Font_WriteScroll 1, String_Connection , Palette_LinkUpFont_Index, 4, 13
	Font_WriteScroll 1, String_Established, Palette_LinkUpFont_Index, 4, 14

	;/* Reset wait timer */
	ldb (bWait), 120

	;/* Advance state */
	State_Advance

	;/* Return from function */
	ret

;*********************************************************
;* End of GameState_NegotiateSuccessfulInitialise_Update *
;*********************************************************


;**************************************************************************************************
;*
;* Function    - GameState_NegotiateSuccessful_Update
;* Parameters  - void
;* Returns     - void
;* Description - Update handler for the GameState_NegotiateSuccessful state.
;* Author      - Steve Robb
;*
;**************************************************************************************************

GameState_NegotiateSuccessful_Update:

	;/* Decrement timer and continue to wait if not zero */
	decb                                                (bWait)
	cpb  (bWait),                                             0
	jr        ne, GameState_NegotiateSuccessful_Update_WaitMore

	;/* Advance state */
	State_Advance

GameState_NegotiateSuccessful_Update_WaitMore:
	;/* Return from function */
	ret

;***********************************************
;* End of GameState_NegotiateSuccessful_Update *
;***********************************************


;**************************************************************************************************
;*
;* Function    - GameState_Negotiate_Display
;* Parameters  - void
;* Returns     - void
;* Description - Display handler for the GameState_Negotiate state.
;* Author      - Steve Robb
;*
;* Function    - GameState_NegotiateSuccessfulInitialise_Display
;* Parameters  - void
;* Returns     - void
;* Description - Display handler for the GameState_NegotiateSuccessfulInitialise state.
;* Author      - Steve Robb
;*
;* Function    - GameState_NegotiateSuccessful_Display
;* Parameters  - void
;* Returns     - void
;* Description - Display handler for the GameState_NegotiateSuccessful state.
;* Author      - Steve Robb
;*
;**************************************************************************************************

GameState_Negotiate_Display:
GameState_NegotiateSuccessfulInitialise_Display:
GameState_NegotiateSuccessful_Display:

	;/* Scroll background */
	incb (AScroll2_XOffset)
	incb (AScroll2_YOffset)

	;/* Return from function */
	ret

;**************************************
;* End of GameState_Negotiate_Display *
;**************************************


;**************************************************************************************************
;*
;* Function    - GameState_GameInitialise_Update
;* Parameters  - void
;* Returns     - void
;* Description - Update handler for the GameState_GameInitialise state.
;* Author      - Steve Robb
;*
;**************************************************************************************************

GameState_GameInitialise_Update:

	;/* Set background */
	Background_SetPaletteEntry 0, FRGB(0, 0, 0)
	Background_SetPaletteIndex 0
	Background_Enable

	;/* Set window */
	Window_Set 0, 0, 160, 152

	;/* Reset scroll offsets */
	Scroll_SetOffset 1,  0, 0
	Scroll_SetOffset 2, -2, 0

	;/* Clear scroll planes */
	Scroll_Clear 1+2, FScroll_RAMWord(0, 0, FALSE, FALSE)

	;/* Install tiles */
	Tile_Install Tiles_Start, Tiles_End, 0

	;/* Install palettes */
	Sprite_InstallPalettes    Palettes_Start      , Palettes_End      , 0
;	Scroll_InstallPalettes 2, Palettes_Arrow_Start, Palettes_Arrow_End, 0
	Scroll_InstallPalettes 1, Palettes_Start      , Palettes_End      , 0

	;/* Draw top and bottom of screen */
	For nX, 0, 19

		Scroll_SetTile 1, nX,  0, FScroll_RAMWord(3, 0, FALSE, FALSE)
		Scroll_SetTile 1, nX, 18, FScroll_RAMWord(8, 0, FALSE, FALSE)

	EndM

	;/* Draw "net" */
	For nY, 1, 17

		Scroll_SetTile 1,  9, nY, FScroll_RAMWord(6, 0, FALSE, FALSE)
		Scroll_SetTile 1, 10, nY, FScroll_RAMWord(7, 0, FALSE, FALSE)

	EndM
	Scroll_SetTile 1,  9,  0, FScroll_RAMWord( 4, 0, FALSE, FALSE)
	Scroll_SetTile 1, 10,  0, FScroll_RAMWord( 5, 0, FALSE, FALSE)
	Scroll_SetTile 1,  9, 18, FScroll_RAMWord( 9, 0, FALSE, FALSE)
	Scroll_SetTile 1, 10, 18, FScroll_RAMWord(10, 0, FALSE, FALSE)

	;/* Create player 1 bat */
	Sprite_JoinRectangle 0, 1, BAT_LENGTH / 8
	For nTile, 0, (BAT_LENGTH / 8) - 1
	
		Sprite_SetTileIndex    nTile, 1
		Sprite_SetPaletteIndex nTile, 0
		
	EndM

	;/* Create player 2 bat */
	Sprite_JoinRectangle 4, 1, BAT_LENGTH / 8
	For nTile, 0, (BAT_LENGTH / 8) - 1
	
		Sprite_SetTileIndex    4 + nTile, 1
		Sprite_SetPaletteIndex 4 + nTile, 0
		
	EndM

	;/* Create ball */
	Sprite_SetTileIndex    8, 2
	Sprite_SetPaletteIndex 8, 0

	;/* Enable bats and ball */
	Sprite_Enable 0, 1, 1
	Sprite_Enable 1, 1, 1
	Sprite_Enable 2, 1, 1
	Sprite_Enable 3, 1, 1
	Sprite_Enable 4, 1, 1
	Sprite_Enable 5, 1, 1
	Sprite_Enable 6, 1, 1
	Sprite_Enable 7, 1, 1
	Sprite_Enable 8, 1, 1

	;/* Set positions of bats */
	ldb (bPlayer1X),               8
	ldb (bPlayer1Y),               8
	ldb (bPlayer2X), 152 - BAT_WIDTH
	ldb (bPlayer2Y),             112
	Sprite_SetPosition 0,               8,   8
	Sprite_SetPosition 4, 152 - BAT_WIDTH, 112

	;/* Reset player scores */
	ldb (bPlayer1Score), 0
	ldb (bPlayer2Score), 0

	;/* Paint player scores */
	call Score_Paint

	;/* Set position of ball */
	ldw (wBallX), 80 << 8
	ldw (wBallY), 76 << 8
	Sprite_SetPosition 8, 80, 76

	;/* Set velocity of ball */
	ldw (wBallVelocityX), BallVelocityX_Degrees(30)
	ldw (wBallVelocityY), BallVelocityY_Degrees(30)

	;/* Check if this is player 1 */
	LinkUp_IsPlayer2
	jp nz, GameState_GameInitialise_Update_DisplayPlayer2Arrow

	;/* Set up player 1 information arrow - set position */
	Sprite_SetPosition  9, 16 + BAT_WIDTH, 12
	Sprite_SetPosition 13, 16 + BAT_WIDTH, 12

	;/* Set position of player id */
	Sprite_SetPosition 17, 36 + BAT_WIDTH, 16
	Sprite_SetPosition 18, 44 + BAT_WIDTH, 16

	;/* Unflip sprites */
	Sprite_SetFlip  9, 0, 0
	Sprite_SetFlip 10, 0, 0
	Sprite_SetFlip 11, 0, 0
	Sprite_SetFlip 12, 0, 0
	Sprite_SetFlip 13, 0, 0
	Sprite_SetFlip 14, 0, 0
	Sprite_SetFlip 15, 0, 0
	Sprite_SetFlip 16, 0, 0

	;/* Set sprite tiles */
	Sprite_SetTileIndex  9, Tiles_Arrow_Index
	Sprite_SetTileIndex 10, Tiles_Arrow_Index + 1
	Sprite_SetTileIndex 11, Tiles_Arrow_Index + 2
	Sprite_SetTileIndex 12, Tiles_Arrow_Index + 3
	Sprite_SetTileIndex 13, Tiles_Arrow_Index + 4
	Sprite_SetTileIndex 14, Tiles_Arrow_Index + 5
	Sprite_SetTileIndex 15, Tiles_Arrow_Index + 6
	Sprite_SetTileIndex 16, Tiles_Arrow_Index + 7
	Sprite_SetTileIndex 17, Tiles_Arrow_Index + 7
	Sprite_SetTileIndex 18, Tiles_Arrow_Index + 7

	;/* Set up rest of arrow */
	jp GameState_GameInitialise_Update_SetUpArrow

GameState_GameInitialise_Update_DisplayPlayer2Arrow:
	;/* Set up player 2 information arrow - set position */
	Sprite_SetPosition  9, 128 - BAT_WIDTH, 116
	Sprite_SetPosition 13, 128 - BAT_WIDTH, 116

	;/* Set position of player id */
	Sprite_SetPosition 17, 108 - BAT_WIDTH, 120
	Sprite_SetPosition 18, 116 - BAT_WIDTH, 120

	;/* Flip sprites horizontally */
	Sprite_SetFlip  9, 1, 0
	Sprite_SetFlip 10, 1, 0
	Sprite_SetFlip 11, 1, 0
	Sprite_SetFlip 12, 1, 0
	Sprite_SetFlip 13, 1, 0
	Sprite_SetFlip 14, 1, 0
	Sprite_SetFlip 15, 1, 0
	Sprite_SetFlip 16, 1, 0

	;/* Set sprite tiles */
	Sprite_SetTileIndex  9, Tiles_Arrow_Index + 1
	Sprite_SetTileIndex 10, Tiles_Arrow_Index
	Sprite_SetTileIndex 11, Tiles_Arrow_Index + 3
	Sprite_SetTileIndex 12, Tiles_Arrow_Index + 2
	Sprite_SetTileIndex 13, Tiles_Arrow_Index + 5
	Sprite_SetTileIndex 14, Tiles_Arrow_Index + 4
	Sprite_SetTileIndex 15, Tiles_Arrow_Index + 7
	Sprite_SetTileIndex 16, Tiles_Arrow_Index + 6
	Sprite_SetTileIndex 17, Tiles_Arrow_Index + 7
	Sprite_SetTileIndex 18, Tiles_Arrow_Index + 7

GameState_GameInitialise_Update_SetUpArrow:
	;/* Set up information arrow */
	Sprite_JoinRectangle    9, 2, 2
	Sprite_JoinRectangle   13, 2, 2
	Sprite_SetPaletteIndex  9, 2
	Sprite_SetPaletteIndex 10, 2
	Sprite_SetPaletteIndex 11, 2
	Sprite_SetPaletteIndex 12, 2
	Sprite_SetPaletteIndex 13, 1
	Sprite_SetPaletteIndex 14, 1
	Sprite_SetPaletteIndex 15, 1
	Sprite_SetPaletteIndex 16, 1
	Sprite_SetPaletteIndex 17, 2
	Sprite_SetPaletteIndex 18, 2
	Sprite_Enable           9, 1, 1
	Sprite_Enable          10, 1, 1
	Sprite_Enable          11, 1, 1
	Sprite_Enable          12, 1, 1
	Sprite_Enable          13, 1, 1
	Sprite_Enable          14, 1, 1
	Sprite_Enable          15, 1, 1
	Sprite_Enable          16, 1, 1
	Sprite_Enable          17, 1, 1
	Sprite_Enable          18, 1, 1

	;/* Set up player id */
	Sprite_SetFlip 17, 0, 0
	Sprite_SetFlip 18, 0, 0

	;/* Reset wait timer for player arrow display */
	ldb (bWait ), 40
	ldb (bWait2),  3

	;/* Advance state */
	State_Advance

	;/* Return from function */
	ret

;******************************************
;* End of GameState_GameInitialise_Update *
;******************************************


;**************************************************************************************************
;*
;* Function    - GameState_PlayerArrows_Update
;* Parameters  - void
;* Returns     - void
;* Description - Update handler for the GameState_PlayerArrows state.
;* Author      - Steve Robb
;*
;**************************************************************************************************

GameState_PlayerArrows_Update:

	;/* Decrement wait counter 2 */
	decb (bWait2)
	cpb  (bWait2), 0
	jp         ne, GameState_PlayerArrows_Update_NotFinished

	;/* Reset wait counter 2 */
	ldb (bWait2), 3

	;/* Cycle colours */
	ldw                                     bc, (FSprite_GetPaletteEntryAddress(1, 1))
	ldw                                     wa, (FSprite_GetPaletteEntryAddress(2, 3))
	ldw (FSprite_GetPaletteEntryAddress(1, 1)),                                     wa
	ldw                                     wa, (FSprite_GetPaletteEntryAddress(2, 2))
	ldw (FSprite_GetPaletteEntryAddress(2, 3)),                                     wa
	ldw                                     wa, (FSprite_GetPaletteEntryAddress(2, 1))
	ldw (FSprite_GetPaletteEntryAddress(2, 2)),                                     wa
	ldw                                     wa, (FSprite_GetPaletteEntryAddress(1, 3))
	ldw (FSprite_GetPaletteEntryAddress(2, 1)),                                     wa
	ldw                                     wa, (FSprite_GetPaletteEntryAddress(1, 2))
	ldw (FSprite_GetPaletteEntryAddress(1, 3)),                                     wa
	ldw (FSprite_GetPaletteEntryAddress(1, 2)),                                     bc

	;/* Decrement wait counter 1 */
	decb (bWait)

	;/* Check if wait counter has completed */
	cpb (bWait), 0
	jr       ne, GameState_PlayerArrows_Update_NotFinished

	;/* Disable arrow sprites */
	Sprite_Enable  9, 0, 0
	Sprite_Enable 10, 0, 0
	Sprite_Enable 11, 0, 0
	Sprite_Enable 12, 0, 0
	Sprite_Enable 13, 0, 0
	Sprite_Enable 14, 0, 0
	Sprite_Enable 15, 0, 0
	Sprite_Enable 16, 0, 0
	Sprite_Enable 17, 0, 0
	Sprite_Enable 18, 0, 0

	;/* Advance state */
	State_Advance

GameState_PlayerArrows_Update_NotFinished:
	;/* Return from function */
	ret

;****************************************
;* End of GameState_PlayerArrows_Update *
;****************************************


;**************************************************************************************************
;*
;* Function    - GameState_Game_Update
;* Parameters  - void
;* Returns     - void
;* Description - Update handler for the GameState_Game state.
;* Author      - Steve Robb
;*
;**************************************************************************************************

GameState_Game_Update:

	;/* Swap controls */
	LinkUp_SyncControls

	;/* Check if up is pressed */
	Input_IsUpPressed (FLinkUp_GetPlayer1ControlsAddress)
	jr eq, GameState_Game_Update_NoPlayer1Up

	;/* Check if bat is at top of screen */
	cpb (bPlayer1Y),                       BORDER_SIZE
	jr           eq, GameState_Game_Update_NoPlayer1Up

	;/* Move player 1 up */
	decb 2, (bPlayer1Y)
GameState_Game_Update_NoPlayer1Up:

	;/* Check if down is pressed */
	Input_IsDownPressed (FLinkUp_GetPlayer1ControlsAddress)
	jr eq, GameState_Game_Update_NoPlayer1Down

	;/* Check if bat is at bottom of screen */
	cpb (bPlayer1Y),    (152 - BORDER_SIZE) - BAT_LENGTH
	jr           eq, GameState_Game_Update_NoPlayer1Down

	;/* Move player 1 down */
	incb 2, (bPlayer1Y)
GameState_Game_Update_NoPlayer1Down

	;/* Check if B is pressed */
	Input_IsUpPressed (FLinkUp_GetPlayer2ControlsAddress)
	jr eq, GameState_Game_Update_NoPlayer2Up

	;/* Check if bat is at top of screen */
	cpb (bPlayer2Y),                       BORDER_SIZE
	jr           eq, GameState_Game_Update_NoPlayer2Up

	;/* Move player 2 up */
	decb 2, (bPlayer2Y)
GameState_Game_Update_NoPlayer2Up:

	;/* Check if A is pressed */
	Input_IsDownPressed (FLinkUp_GetPlayer2ControlsAddress)
	jr eq, GameState_Game_Update_NoPlayer2Down

	;/* Check if bat is at bottom of screen */
	cpb (bPlayer2Y),    (152 - BORDER_SIZE) - BAT_LENGTH
	jr           eq, GameState_Game_Update_NoPlayer2Down

	;/* Move player 2 down */
	incb 2, (bPlayer2Y)
GameState_Game_Update_NoPlayer2Down:

	;/* Update ball position */
	ldw        bc, (wBallVelocityX)
	addw (wBallX),               bc
	ldw        bc, (wBallVelocityY)
	addw (wBallY),               bc

	;/* Check if ball X velocity is negative */
	ldw de,                     (wBallVelocityX)
	cpw de,                                    0
	jr  ge, GameState_Game_Update_NoBounceOnLeft

	;/* Get ball X position */
	ldw  hl,           (wBallX)
	addw hl, BALL_DIAMETER << 8

	;/* Check if ball is past left edge of screen */
	cpb  h,                                  -8
	jr  lt, GameState_Game_Update_NoPointOnLeft
	cpb  h,                                   0
	jr  ge, GameState_Game_Update_NoPointOnLeft

	;/* Increase player 2 score */
	ldb                b, (bPlayer2Score)
	addb               b,               1
	daa                b
	ldb  (bPlayer2Score),               b

	;/* Repaint score */
	call Score_Paint

	;/* Set position of ball to right-hand side of screen */
	ldw (wBallX), 160 << 8

GameState_Game_Update_NoPointOnLeft:

	;/* Check if ball X position is in line with bat */
	ldw   bc,                                    0
	ldb    b,                          (bPlayer1X)
	cpb    b,                                    h
	jr   uge, GameState_Game_Update_NoBounceOnLeft
	addb   b,            BALL_DIAMETER + BAT_WIDTH
	cpb    b,                                    h
	jr   ule, GameState_Game_Update_NoBounceOnLeft

	;/* Get ball Y position */
	ldw  hl,           (wBallY)
	addw hl, BALL_DIAMETER << 8

	;/* Check if ball Y position is in line with bat */
	ldw   bc,                                    0
	ldb    b,                          (bPlayer1Y)
	cpb    b,                                    h
	jr   uge, GameState_Game_Update_NoBounceOnLeft
	addb   b,           BALL_DIAMETER + BAT_LENGTH
	cpb    b,                                    h
	jr   ule, GameState_Game_Update_NoBounceOnLeft

	;/* Get position of ball relative to bat */
	ldw  bc,      (wBallY)
	addb  b, BALL_DIAMETER
	subb  b,   (bPlayer1Y)

	;/* Get address of bat angle */
	ld  xde,         0
	ldb   e,         b
	sla   2,       xde
	add xde, BatAngles

	;/* Get new angle of ball */
	ldw               bc, (xde+)
	ldw (wBallVelocityX),     bc
	ldw               bc,  (xde)
	ldw (wBallVelocityY),     bc

GameState_Game_Update_NoBounceOnLeft:

	;/* Check if ball X velocity is positive */
	ldw de,                      (wBallVelocityX)
	cpw de,                                     0
	jr  le, GameState_Game_Update_NoBounceOnRight

	;/* Get ball X position */
	ldw  hl,           (wBallX)
	addw hl, BALL_DIAMETER << 8

	;/* Check if ball is past right edge of screen */
	cpb   h,                  160 + BALL_DIAMETER
	jr  ult, GameState_Game_Update_NoPointOnRight
	cpb   h,                  168 + BALL_DIAMETER
	jr  uge, GameState_Game_Update_NoPointOnRight

	;/* Increase player 1 score */
	ldb                b, (bPlayer1Score)
	addb               b,               1
	daa                b
	ldb  (bPlayer1Score),               b

	;/* Repaint score */
	call Score_Paint

	;/* Set position of ball to left-hand side of screen */
	ldw (wBallX), (-BALL_DIAMETER) << 8

GameState_Game_Update_NoPointOnRight:

	;/* Check if ball X position is in line with bat */
	ldw   bc,                                     0
	ldb    b,                           (bPlayer2X)
	cpb    b,                                     h
	jr   uge, GameState_Game_Update_NoBounceOnRight
	addb   b,             BALL_DIAMETER + BAT_WIDTH
	cpb    b,                                     h
	jr   ule, GameState_Game_Update_NoBounceOnRight

	;/* Get ball Y position */
	ldw  hl,           (wBallY)
	addw hl, BALL_DIAMETER << 8

	;/* Check if ball Y position is in line with bat */
	ldw   bc,                                     0
	ldb    b,                           (bPlayer2Y)
	cpb    b,                                     h
	jr   uge, GameState_Game_Update_NoBounceOnRight
	addb   b,            BALL_DIAMETER + BAT_LENGTH
	cpb    b,                                     h
	jr   ule, GameState_Game_Update_NoBounceOnRight

	;/* Get position of ball relative to bat */
	ldw  bc,      (wBallY)
	addb  b, BALL_DIAMETER
	subb  b,   (bPlayer2Y)

	;/* Get address of bat angle */
	ld  xde,         0
	ldb   e,         b
	sla   2,       xde
	add xde, BatAngles

	;/* Get new angle of ball */
	ldw               bc, (xde+)
	neg                       bc
	ldw (wBallVelocityX),     bc
	ldw               bc,  (xde)
	ldw (wBallVelocityY),     bc
GameState_Game_Update_NoBounceOnRight:

	;/* Check if ball Y velocity is negative */
	ldw de,                    (wBallVelocityY)
	cpw de,                                   0
	jr  ge, GameState_Game_Update_NoBounceOnTop

	;/* Check if ball Y position is negative */
	cpw (wBallY),                            152 << 8
	jr       ule, GameState_Game_Update_NoBounceOnTop
	cpw (wBallY),                    BORDER_SIZE << 8
	jr        ge, GameState_Game_Update_NoBounceOnTop

	;/* Negate ball Y velocity */
	neg                   de
	ldw (wBallVelocityY), de
	jr  GameState_Game_Update_NoBounceOnBottom
GameState_Game_Update_NoBounceOnTop:

	;/* Check if ball Y velocity is positive */
	ldw de,                       (wBallVelocityY)
	cpw de,                                      0
	jr  le, GameState_Game_Update_NoBounceOnBottom

	;/* Check if ball Y position is positive */
	cpw (wBallY), ((152 - BORDER_SIZE) - BALL_DIAMETER) << 8
	jr       ule,     GameState_Game_Update_NoBounceOnBottom

	;/* Negate ball Y velocity */
	neg                   de
	ldw (wBallVelocityY), de
GameState_Game_Update_NoBounceOnBottom:

	;/* Return from function */
	ret

;********************************
;* End of GameState_Game_Update *
;********************************


;**************************************************************************************************
;*
;* Function    - GameState_Game_Display
;* Parameters  - void
;* Returns     - void
;* Description - Display handler for the GameState_Game state.
;* Author      - Steve Robb
;*
;**************************************************************************************************

GameState_Game_Display:

	;/* Store registers */
	push de

	;/* Update position of sprites */
	ldw                               de, (bPlayer1X)
	ldw (FSprite_GetPositionXAddress(0)),          de
	ldw                               de, (bPlayer2X)
	ldw (FSprite_GetPositionXAddress(4)),          de
	ldb                                e,  (wBallX+1)
	ldb (FSprite_GetPositionXAddress(8)),           e
	ldb                                e,  (wBallY+1)
	ldb (FSprite_GetPositionYAddress(8)),           e

	;/* Restore registers */
	pop de

	;/* Return from function */
	ret

;*********************************
;* End of GameState_Game_Display *
;*********************************


;**************************************************************************************************
;*
;* Function    - Score_Paint
;* Parameters  - void
;* Returns     - void
;* Description - Paints the players' scores on the screen.
;* Author      - Steve Robb
;*
;**************************************************************************************************

Score_Paint:

	;/* Store player 1 score in b */
	ldb b, (bPlayer1Score)

	;/* Determine address of glyph of lower digit for score 1 */
	ld   xde,                             0
	ldb    e,                             b
	andb   e,                          00Fh
	mul  xde, NumberGlyph_1 - NumberGlyph_0
	add  xde,                 NumberGlyph_0

	;/* Display lower digit of score 1 */
	push b
	Scroll_CopyRectangle 2, 7, 1, 2, 3, xde
	pop b

	;/* Check if upper digit is non-zero */
	srlb  4, b
	cpb   b, 0
	jr   eq, Score_Paint_NoPlayer1UpperDigit

	;/* Determine address of glyph of upper digit for score 1 */
	ld   xde,                             0
	ldb    e,                             b
	mul  xde, NumberGlyph_1 - NumberGlyph_0
	add  xde,                 NumberGlyph_0

	;/* Display upper digit of score 1 */
	Scroll_CopyRectangle 2, 5, 1, 2, 3, xde

Score_Paint_NoPlayer1UpperDigit:
	;/* Store player 2 score in b */
	ldb b, (bPlayer2Score)

	;/* Determine address of glyph of lower digit for score 2 */
	ld   xde,                             0
	ldb    e,                             b
	andb   e,                          00Fh
	mul  xde, NumberGlyph_1 - NumberGlyph_0
	add  xde,                 NumberGlyph_0

	;/* Display lower digit of score 2 */
	push b
	Scroll_CopyRectangle 2, 12, 1, 2, 3, xde
	pop b

	;/* Check if upper digit is non-zero */
	srlb  4, b
	cpb   b, 0
	jr   eq, Score_Paint_NoPlayer2UpperDigit

	;/* Determine address of glyph of upper digit for score 2 */
	ld   xde,                             0
	ldb    e,                             b
	mul  xde, NumberGlyph_1 - NumberGlyph_0
	add  xde,                 NumberGlyph_0

	;/* Display upper digit of score 2 */
	Scroll_CopyRectangle 2, 10, 1, 2, 3, xde

Score_Paint_NoPlayer2UpperDigit:
	;/* Return from function */
	ret

;**********************
;* End of Score_Paint *
;**********************


;*****************
;* Include files *
;*****************

	Include "NoIntrps.inc"


;********************
;* End of Plink.asm *
;********************
