#include "ngpc.h"		// required
#include "carthdr.h"		// required
#include "library.h"		// NGPC routines
#include <stdlib.h>		// std C routines
#include <stdio.h>

#include "common.h"

// Base index for tiles (common tiles)
#define TILE_BASE 130

// Common GFX
#define TILE_EXP TILE_BASE
#define PAL_EXP 0
#define TILE_BONUS (TILE_BASE+1)
#define PAL_BONUS 1
#define TILE_FADE (TILE_BONUS+3)

//Traps
#define TILE_TRAP_BASE (TILE_FADE+25)
#define TILE_TRAP_STATIC TILE_TRAP_BASE
#define PAL_TRAP_STATIC 1
#define TILE_TRAP_PONG (TILE_TRAP_STATIC+1)
#define PAL_TRAP_PONG (PAL_TRAP_STATIC+1)
#define TILE_TRAP_TETRIS (TILE_TRAP_PONG+1)
#define PAL_TRAP_TETRIS (PAL_TRAP_PONG+1)

#define TRAP_STATIC 0
#define TRAP_PONG 1
#define TRAP_TETRIS 2

#define TILE_TITLE (TILE_TRAP_TETRIS+1)
#define PAL_TITLE 3

//SFX
enum
{
   SND_SHOOT = 1,
   SND_FADE,
   NUM_SOUNDS = SND_FADE
};
const SOUNDEFFECT Sounds[NUM_SOUNDS] = {
   //{ 2, 0x60, 0, 0x60, 8, 1, 2, 0x40, 0x80, 0xf, 0,0,0,0,0 },
   { 1, 0x4, 0, 0x1F0, 16, 1, 2, 0x80, 0x80, 0xf, 0,0,0,0,0 },
   { 2, 0x20, 0, 0xE0, 0x10, 1, 1, 0x44, 0x82, 0xf, 0,0,0,0,0 }
};


typedef struct {
  int x,y;
  int status;
  int hp;
  int score;
  int dx,dy; //pong
  int color; //tetris
} trap_t;

#define MAX_TRAP 60
trap_t trap[MAX_TRAP];

// Base index for level-specific tiles (main character,bullets,enemies,bonus)
#define VAR_TILE_BASE (TILE_BASE+150)
// Base index for level-specific palettes
#define VAR_PAL_BASE 4

#define DIR_UP (1<<1)
#define DIR_DOWN (1<<2)
#define DIR_LEFT (1<<3)
#define DIR_RIGHT (1<<4)

#define ANIM_LSL 2
#define SHOOT_FREQ 10
#define EXPLODE_FREQ 20
#define BONUS_FREQ 50

//Stages
#define STAGE_TROGDOR 0
#define STAGE_VIDEOPAC 1
#define STAGE_PACMAN 2
#define STAGE_INVADERS 3
#define STAGE_ROBOTRON 4
#define STAGE_MARIO 5

typedef struct {
  int active;
  int x,y;
  int dir;
  int ttl;
} bullet_t;

// Enemies and bonus
enemy_t enemy[MAX_ENEMY];
bonus_t bonus[MAX_BONUS];

// Player and bullets
int x,y;
int size;
int dir,profile,dirlock;
bullet_t bullet[MAX_BULLET];

long frame,shootFrame;
long enemyMoveFrame,enemyAnimFrame;
long trapMoveFrame, trapAnimFrame;
long bonusMoveFrame,bonusAnimFrame;

long score;
int lives;

int enemySlow; int enemyShootSlow;
int trapSlow;

int killCount;

int extraShoot;

// Common GFX
extern const unsigned short gfxexp[][8];
extern const unsigned short gfxbonus[][8];
extern const unsigned short gfxfade[][8];
extern const unsigned short gfxtraps[][8];
extern const unsigned short gfxtitle[][8];

///////////// STAGES - BEGIN

// TROGDOR
extern void tgdInit(int,int);
extern void tgdInitLevel(int,int);
// VIDEOPAC
extern void vpcInit(int,int);
extern void vpcInitLevel(int,int);
// PACMAN
extern void pacInit(int,int);
extern void pacInitLevel(int,int);
// INVADERS
extern void invInit(int,int);
extern void invInitLevel(int,int);
// ROBOTRON
extern void robInit(int,int);
extern void robInitLevel(int,int);
// MARIO
extern void marInit(int,int);
extern void marInitLevel(int,int);

///////////// STAGES - END

typedef struct {
  int level;
  int stage;
  
  int trap;
  int trapSlow;
  int trapLevel;
  
  char *msg1;
  char *msg2;
  char *msg3;
} play_t;

///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////

const play_t play[] = {
  { 0, STAGE_ROBOTRON, TRAP_STATIC, 2, 1, "  Back to the roots", "",""},
  { 1, STAGE_ROBOTRON, TRAP_STATIC, 2, 2, " A bit more tricky...", "", ""},

  { 0, STAGE_INVADERS, TRAP_STATIC, 2, 1, " All your base are", " belong to us", ""},
  { 1, STAGE_INVADERS, TRAP_PONG, 10, 2, "   Classic Gaming ", "       Mix #1", ""},

  { 0, STAGE_TROGDOR, TRAP_STATIC, 2, 2, " Burninate the ", "  countryside !!!", ""},
  { 1, STAGE_TROGDOR, TRAP_STATIC, 2, 2, " And a beefy arm ", "  for good measure", ""},

  { 0, STAGE_PACMAN, TRAP_STATIC, 2, 4, "   I ate too much", "   of those pills", ""},
  { 1, STAGE_PACMAN, TRAP_TETRIS, 15, 2, "   Classic Gaming ", "       Mix #2", ""},

  { 1, STAGE_MARIO, TRAP_PONG, 10, 1, " robbies, ph34r my ", " Italian cousin !!!", ""},
  { 1, STAGE_MARIO, TRAP_PONG, 8, 4, " Let's speed it up ", "", ""},
  
  { 0, STAGE_VIDEOPAC, TRAP_TETRIS, 15, 4, " 1978 : a Videogame ", "        Odyssey 2 ", ""},
  { 1, STAGE_VIDEOPAC, TRAP_TETRIS, 15, 4, " 1978 : a Videogame ", "        Odyssey 2 ", ""},
};
#define STAGE_MAX 12

int currentPlay;

// Functions Declarations
void sprite(int SpriteNo, int TileNo, int Chain, int XPos, int YPos, int PaletteNo, int Priority, int Flips);

void clearDisplay();
void clearStage();
int playerMove();
void playerDisplay();
void shoot();
void bulletMove();
int enemyMove();
int enemyDisplay();
void scoreDisplay();
void fade();
void fadebox(int, int, int, int);
void playerDeath();
void setupTraps(int);
int trapMove();
int trapDisplay();
void bonusMove();
void bonusDisplay();
void bham(int);
int enemyBulletMove();

// STAGE PARAMETERS
int currentStage;
int currentLevel;
int currentTrap;
int trapLevel;

int enemyShootFrame;

extern const unsigned char sfxShoot[];
extern const unsigned char sfxExplode[];

void main(void) {

  int exitStatus;
  int firstTime;
  int i,j,k;

  int selectStatus;

  // HW init
  InitNGPC();
  SysSetSystemFont();

  InstallSoundDriver();
  InstallSounds(Sounds, NUM_SOUNDS);

  clearDisplay();

  // ====== Common GFX setup
  // Explosion
  InstallTileSetAt(gfxexp, 8, TILE_EXP );
  SetColors(SPRITE_PLANE, PAL_EXP, RGB(15,15,15), 0, 0 );
  // Bonus
  InstallTileSetAt(gfxbonus, 3*8, TILE_BONUS );
  SetColors(SPRITE_PLANE, PAL_BONUS, RGB(15,0,0), RGB(0,15,0), RGB(0,0,15) );
  // Fade
  InstallTileSetAt(gfxfade, 3*8*8, TILE_FADE );
  SetColors(SCR_1_PLANE, 1, RGB(15,0,0), RGB(0,15,0), RGB(15,0,0) );
  SetColors(SCR_1_PLANE, 2, RGB(0,0,15), RGB(15,0,15), RGB(0,15,0) );
  // Traps
  InstallTileSetAt(gfxtraps, 3*8, TILE_TRAP_BASE ); 
  SetColors(SCR_2_PLANE, 0, RGB(15,15,15), 0, 0 ); // explosion
  // static-trap = set at runtime
  SetColors(SCR_2_PLANE, PAL_TRAP_PONG, RGB(15,15,15), RGB(15,15,15), RGB(15,15,15) ); // pong-trap
  SetColors(SCR_2_PLANE, PAL_TRAP_TETRIS, RGB(8,0,0), RGB(12,0,0), RGB(15,0,0) ); // tetris-trap - col 1
  SetColors(SCR_2_PLANE, PAL_TRAP_TETRIS+1, RGB(0,8,0), RGB(0,12,0), RGB(0,15,0) ); // tetris-trap - col 2
  SetColors(SCR_2_PLANE, PAL_TRAP_TETRIS+2, RGB(0,0,8), RGB(0,0,12), RGB(0,0,15) ); // tetris-trap - col 3

  //Title
  InstallTileSetAt(gfxtitle, 20*4*8, TILE_TITLE );
  SetColors(SCR_1_PLANE, PAL_TITLE, RGB(0,0,15), RGB(15,0,15), RGB(0,15,0) );

  while(1) {

    clearDisplay();
    SetColors(SCR_1_PLANE, PAL_TITLE, RGB(0,0,15), RGB(15,0,15), RGB(0,15,0) );

    PrintString(SCR_1_PLANE, 0, 0, 0,  "  Torlus presents   ");

    PrintString(SCR_1_PLANE, 0, 0, 14, " Press \"A\" to start");

    PrintString(SCR_1_PLANE, 0, 0, 18, "www.consolefever.com");

    k=0;
    for(j=0;j<4;j++) {
      for(i=0; i<20;i++) {
        PutTile(SCR_1_PLANE, PAL_TITLE, i, j+4, TILE_TITLE+k++);
      }
    }

    currentPlay = 0;
    
    exitStatus = 0;
    selectStatus = 0;
    i=0;
    while(1) {
 	    if ((exitStatus == 0) && (SYS_LEVER & J_A)) exitStatus = 1;
 	    if ((exitStatus == 1) && !(SYS_LEVER & J_A)) break;

 	    if ((selectStatus == 0) && (SYS_LEVER & J_OPTION) && (SYS_LEVER & J_B)) selectStatus = 1;
 	    if ((selectStatus == 1) && (SYS_LEVER & J_OPTION) && !(SYS_LEVER & J_B)) {
 	      currentPlay = ( (currentPlay+1)%STAGE_MAX );
 	      PrintString(SCR_1_PLANE, 0, 0, 2, "LV");
 	      PrintNumber(SCR_1_PLANE, 0, 2, 2, currentPlay+1, 2);
        selectStatus = 0;
      }

 	    PrintString(SCR_1_PLANE, ((i+1) % 3), 0, 9, "    A trip into    ");
 	    PrintString(SCR_1_PLANE, ((i+2) % 3), 0, 10, " Videogame history ");
 	    i++;

 	    WaitVsync();    
 	    Sleep(5);
    }
    
// BEGINNING OF GAME

  clearDisplay();
  SetColors(SCR_1_PLANE, 0, 0, 0, RGB(15, 15, 15)); //Text

  score = 0; lives = 3;

  while(1) {

// STAGE INTRO

    for(i=0;i<MAX_ENEMY;i++) { enemy[i].status = 0; enemy[i].type = 0; enemy[i].profile = 0; } 
    for(i=0;i<MAX_BONUS;i++) bonus[i].status = 0;  

    clearDisplay();
    clearStage();

    currentStage = play[currentPlay%STAGE_MAX].stage;
    currentLevel = play[currentPlay%STAGE_MAX].level;
    currentTrap = play[currentPlay%STAGE_MAX].trap;
    trapSlow = play[currentPlay%STAGE_MAX].trapSlow;
    trapLevel = play[currentPlay%STAGE_MAX].trapLevel;

    switch(currentStage) {  
      case STAGE_TROGDOR: // TROGDOR
        tgdInit(VAR_TILE_BASE,VAR_PAL_BASE);
        tgdInitLevel(currentLevel,1);
        break;       
      case STAGE_VIDEOPAC: // VIDEOPAC
        vpcInit(VAR_TILE_BASE,VAR_PAL_BASE);
        vpcInitLevel(currentLevel,1);
        break;       
      case STAGE_PACMAN: // PACMAN
        pacInit(VAR_TILE_BASE,VAR_PAL_BASE);
        pacInitLevel(currentLevel,1);
        break;       
      case STAGE_INVADERS: // INVADERS
        invInit(VAR_TILE_BASE,VAR_PAL_BASE);
        invInitLevel(currentLevel,1);
        break;       
      case STAGE_ROBOTRON: // ROBOTRON
        robInit(VAR_TILE_BASE,VAR_PAL_BASE);
        robInitLevel(currentLevel,1);
        break;       
      case STAGE_MARIO: // MARIO
        marInit(VAR_TILE_BASE,VAR_PAL_BASE);
        marInitLevel(currentLevel,1);
        break;       
    }    
    if (currentPlay>=STAGE_MAX) enemySlow=2;
    if (currentPlay>=(STAGE_MAX*2)) enemySlow=1;

    PrintString(SCR_1_PLANE,0,0,16,play[currentPlay%STAGE_MAX].msg1);
    PrintString(SCR_1_PLANE,0,0,17,play[currentPlay%STAGE_MAX].msg2);
    PrintString(SCR_1_PLANE,0,0,18,play[currentPlay%STAGE_MAX].msg3);
    
    setupTraps(1);    
        
    scoreDisplay();
    playerDisplay();
    enemyDisplay();
    trapDisplay();
    bonusDisplay();

    WaitVsync(); PlaySound(SND_FADE); Sleep(6);    
    WaitVsync(); PlaySound(SND_FADE); Sleep(6);    
    WaitVsync(); PlaySound(SND_FADE); Sleep(6);    

    Sleep(2*60);

    PrintString(SCR_1_PLANE,0,0,16,"                    ");
    PrintString(SCR_1_PLANE,0,0,17,"                    ");
    PrintString(SCR_1_PLANE,0,0,18,"                    ");

// GAME LOOP
game_loop:
    exitStatus = 0;

    while(!exitStatus) {	   

      scoreDisplay();
    
      if (playerMove()) {
        exitStatus = 1; //death
      }
      playerDisplay();

      if (enemyMove()) {
        exitStatus = 1; //death
      }
      if (enemyBulletMove()) {
        exitStatus = 1; //death
      }
      if (enemyDisplay()) {
        exitStatus = 2; //level completed
      }

      bonusMove();
      bonusDisplay();

      trapMove();
      if (trapDisplay()) {
        exitStatus = 1; //death
      }

   	  shoot();
   	  bulletMove();

   	  WaitVsync();
  	  //Sleep(20);
    }

// GAME LOOP END
    if (exitStatus == 1) {
      //death
      playerDeath();
  
      clearDisplay();
      clearStage();

      lives--;
      scoreDisplay();

      if (lives<0) break;      

      switch(currentStage) {  
        case STAGE_TROGDOR:
          tgdInit(VAR_TILE_BASE,VAR_PAL_BASE);
          tgdInitLevel(currentLevel,0);
          break;
        case STAGE_VIDEOPAC:
          vpcInit(VAR_TILE_BASE,VAR_PAL_BASE);
          vpcInitLevel(currentLevel,0);
          break;
        case STAGE_PACMAN:
          pacInit(VAR_TILE_BASE,VAR_PAL_BASE);
          pacInitLevel(currentLevel,0);
          break;
        case STAGE_INVADERS:
          invInit(VAR_TILE_BASE,VAR_PAL_BASE);
          invInitLevel(currentLevel,0);
          break;
        case STAGE_ROBOTRON:
          robInit(VAR_TILE_BASE,VAR_PAL_BASE);
          robInitLevel(currentLevel,0);
          break;
        case STAGE_MARIO:
          marInit(VAR_TILE_BASE,VAR_PAL_BASE);
          marInitLevel(currentLevel,0);
          break;
      }        
      if (currentPlay>=STAGE_MAX) enemySlow=2;
      if (currentPlay>=(STAGE_MAX*2)) enemySlow=1;
      
      setupTraps(0);
    
      scoreDisplay();
      playerDisplay();
      enemyDisplay();
      trapDisplay();
      bonusDisplay();
    
      WaitVsync(); PlaySound(SND_FADE); Sleep(6);    
      WaitVsync(); PlaySound(SND_FADE); Sleep(6);    
      WaitVsync(); PlaySound(SND_FADE); Sleep(6);    

      Sleep(2*60);

      goto game_loop;
       
    } else {
      //level completed
      lives++;
      scoreDisplay();
      fade();
      currentPlay++;
    }  

  }
//GAME OVER
  clearDisplay();
  SetColors(SCR_1_PLANE, 0, 0, 0, RGB(15, 15, 15)); //Text
  scoreDisplay();

  PrintString(SCR_1_PLANE, 0, 0, 9, "     GAME OVER");
  Sleep(3*60);
  PrintString(SCR_1_PLANE, 0, 0, 14, " Press \"A\" to exit");

  exitStatus = 0;
  while(1) {
    if ((exitStatus == 0) && (SYS_LEVER & J_A)) exitStatus = 1;
    if ((exitStatus == 1) && !(SYS_LEVER & J_A)) break;
    Sleep(5);
  }
  


  } // main while(1) 

}

//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////

const u8 trapStaticPos[] = { 
  3,4 , 2,7 , 5,9 , 4,11 , 5,5 , 1,2 , 2,13 , 5,14 , 2,15, 4,16 , 1,17 ,
  6,2 , 8,2 , 10,3 , 9,4 , 7,4 //, 10,5 , 7,6 , //8,7 , 12,8 , 5,9 , 6,10
};

const s8 trapPongData[] = {
  3,10 , -1,1 ,
  5,5 , 1,-1 ,
  12,14 , 1,1 , 
  14,3 , -1,1 ,
};

const s8 trapTetrisData[] = {
  2,1 , 2,2 , 2,3 , 3,3 ,
  12,2 , 13,2 , 14,2 , 15,2 ,    
  
  8,5 , 9,5 , 8,6 , 9,6 ,
  4,12 , 5,12 , 6,12 , 5,13 , 
  
  14,8 , 14,9 , 14,10 , 13,10 ,
  10,15 , 10,16 , 10,17, 10,18 , 
  
  1,16 , 2,16 , 1,17 , 2,17 ,  
  16,14 , 17,14 , 18,14 , 18,15 ,
};

void setupTraps(int firstTime) {
  int i;

  if (firstTime) {
    for(i=0; i<MAX_TRAP;i++) trap[i].status = 0;
  }
      
  switch(currentTrap) {
    case TRAP_STATIC:
      SetColors(SCR_2_PLANE, PAL_TRAP_STATIC, RGB(15,0,0), RGB(0,15,0), 0 );
      //for(i=0; i < sizeof(trapStaticPos); i+=2) {
      for(i=0; i < trapLevel*8; i+=2) {
        if (firstTime) { 
          trap[i].status = 1;
          trap[i+1].status = 1;
        }
        trap[i].x = trapStaticPos[i];
        trap[i].y = trapStaticPos[i+1];
        trap[i+1].x = 20-trapStaticPos[i];
        trap[i+1].y = 19-trapStaticPos[i+1];

        trap[i].score = 0;
        trap[i].hp = 1;
        trap[i+1].score = 0;
        trap[i+1].hp = 1;
        
        if (trap[i].status == 1) PutTile(SCR_2_PLANE, PAL_TRAP_STATIC, trap[i].x, trap[i].y, TILE_TRAP_STATIC );
        if (trap[i+1].status == 1) PutTile(SCR_2_PLANE, PAL_TRAP_STATIC, trap[i+1].x, trap[i+1].y, TILE_TRAP_STATIC );
      }
      break;
      
    case TRAP_PONG:
      for(i=0; i<5;i++) {
        // pad 1
        trap[i].x = 0;
        trap[i].y = 8+i;
        trap[i].dy = 1;
        trap[i].status = 1;
        trap[i].hp = -1;
        // pad 2
        trap[i+5].x = 19;
        trap[i+5].y = 8+i;
        trap[i+5].dy = -1;
        trap[i+5].status = 1;
        trap[i+5].hp = -1;
      }

      //for(i=0; i < (sizeof(trapPongData)>>2); i++) {
      for(i=0; i < trapLevel; i++) {
        //balls
        trap[10+i].status = 1;
        trap[10+i].hp = -1;

        trap[10+i].x = trapPongData[4*i];
        trap[10+i].y = trapPongData[4*i+1];
        trap[10+i].dx = trapPongData[4*i+2];
        trap[10+i].dy = trapPongData[4*i+3];      
      }            
      break; 
      
    case TRAP_TETRIS:
      for(i=0; i < (sizeof(trapTetrisData)>>1); i++) {
        trap[i].x = trapTetrisData[2*i];
        trap[i].y = trapTetrisData[2*i+1];
        trap[i].color = ((i>>2) % 3);
        trap[i].status = 1;
        trap[i].hp = -1;
      }
      break;
              
  }

}

int trapMove() {
  int i;

  trapMoveFrame++;
 	if (trapMoveFrame != trapSlow) return 0;
  trapMoveFrame = 0;

  trapAnimFrame++;
  if (trapAnimFrame == (1<<(ANIM_LSL+1))) trapAnimFrame = 0;
  
  switch(currentTrap) {
    case TRAP_PONG:
    
      if (trap[0].dy > 0) {
        PutTile(SCR_2_PLANE,0,trap[0].x, trap[0].y, ' ');
        for(i=0;i<5;i++) trap[i].y++;
        if (trap[4].y == 18) trap[0].dy = -1;
      } else {
        PutTile(SCR_2_PLANE,0,trap[4].x, trap[4].y, ' ');        
        for(i=0;i<5;i++) trap[i].y--;
        if (trap[0].y == 1) trap[0].dy = 1;
      }
      if (trap[5].dy > 0) {
        PutTile(SCR_2_PLANE,0,trap[5].x, trap[5].y, ' ');
        for(i=0;i<5;i++) trap[i+5].y++;
        if (trap[9].y == 18) trap[5].dy = -1;
      } else {
        PutTile(SCR_2_PLANE,0,trap[9].x, trap[9].y, ' ');        
        for(i=0;i<5;i++) trap[i+5].y--;
        if (trap[5].y == 1) trap[5].dy = 1;
      }

    
      for(i=10;i<10+(sizeof(trapPongData)>>2);i++) {
        PutTile(SCR_2_PLANE,0,trap[i].x, trap[i].y, ' ');
        trap[i].x += trap[i].dx;
        trap[i].y += trap[i].dy;
        //PutTile(SCR_2_PLANE,PAL_TRAP_PONG,trap[i].x, trap[i].y, TILE_TRAP_PONG);        
        if (trap[i].x == 18) trap[i].dx = -1;
        if (trap[i].x == 1) trap[i].dx = 1;        
        if (trap[i].y == 18) trap[i].dy = -1;
        if (trap[i].y == 1) trap[i].dy = 1;
      }
      break;
    case TRAP_TETRIS:
      for(i=0; i<MAX_TRAP;i++) {
        if (trap[i].status == 1) {
          //TODO
          if ( (trap[i].y >= 1) && (trap[i].y < 19) ) PutTile(SCR_2_PLANE,0,trap[i].x, trap[i].y, ' ');
          trap[i].y++;
          if (trap[i].y > 25) trap[i].y = -5;
        }
      }    
    
  }
  
  return 0;
}

int trapDisplay() {

  int i;
  int tx,ty;
  int dx,dy;

  switch(currentTrap) {
    case TRAP_STATIC:
      if (trapAnimFrame>>ANIM_LSL) {
          SetColors(SCR_2_PLANE, PAL_TRAP_STATIC, RGB(0,15,0), RGB(15,0,0), 0 );        
      } else {
          SetColors(SCR_2_PLANE, PAL_TRAP_STATIC, RGB(15,0,0), RGB(0,15,0), 0 );
      }
      break;

    case TRAP_PONG:      
      for(i=0; i<10+trapLevel; i++) PutTile(SCR_2_PLANE,PAL_TRAP_PONG,trap[i].x,trap[i].y,TILE_TRAP_PONG);      
      break;

    case TRAP_TETRIS:      
      for(i=0; i<MAX_TRAP; i++) {
        if (trap[i].status == 1) {
          if ( (trap[i].y >= 1) && (trap[i].y < 19) ) {
            PutTile(SCR_2_PLANE,PAL_TRAP_TETRIS+trap[i].color,trap[i].x,trap[i].y,TILE_TRAP_TETRIS);      
          }
        }
      }
      break;

  }
  
  // trap explosion
  for(i = 0; i < MAX_TRAP; i++) {
    if (trap[i].status < 0) {
      PutTile(SCR_2_PLANE,0,trap[i].x,trap[i].y,TILE_EXP);
      trap[i].status++;
      if (trap[i].status == 0) PutTile(SCR_2_PLANE,0,trap[i].x,trap[i].y,' ');
    }
  }

  // player collision
  if (size == 16) {
    tx=x+4; ty=y+4;
  } else {
    tx=x; ty=y;
  }

  for(i=0; i<MAX_TRAP; i++) {
    if (trap[i].status == 1) {

      dx=(trap[i].x<<3)-tx; if(dx<0) dx=-dx;
      dy=(trap[i].y<<3)-ty; if(dy<0) dy=-dy;

      if ((dx < 8) && (dy < 8)) {
        // player death
        return 1;
      }
    }
  }
  
  return 0;
}

void scoreDisplay() {
  int i;
  int digit;
  int x;
  int val;
  
  val = score;
  x=5;  
#define DIGIT(base) { digit=0; while(val >= base) { val-=base; digit++; } }

  //DIGIT(1000000000); PutTile(SCR_1_PLANE,0,(u8)x++,0,'0'+digit);
  //DIGIT(100000000); PutTile(SCR_1_PLANE,0,(u8)x++,0,'0'+digit);
  //DIGIT(10000000); PutTile(SCR_1_PLANE,0,(u8)x++,0,'0'+digit);
  DIGIT(1000000); PutTile(SCR_1_PLANE,0,(u8)x++,0,'0'+digit);
  DIGIT(100000); PutTile(SCR_1_PLANE,0,(u8)x++,0,'0'+digit);
  DIGIT(10000); PutTile(SCR_1_PLANE,0,(u8)x++,0,'0'+digit);
  DIGIT(1000); PutTile(SCR_1_PLANE,0,(u8)x++,0,'0'+digit);
  DIGIT(100); PutTile(SCR_1_PLANE,0,(u8)x++,0,'0'+digit);
  DIGIT(10); PutTile(SCR_1_PLANE,0,(u8)x++,0,'0'+digit);
  digit = val; PutTile(SCR_1_PLANE,0,(u8)x++,0,'0'+digit);

  PrintString(SCR_1_PLANE,0,0,0,"LV");

  val = currentPlay+1;
  x=2;

  DIGIT(10); PutTile(SCR_1_PLANE,0,(u8)x++,0,'0'+digit);
  digit = val; PutTile(SCR_1_PLANE,0,(u8)x++,0,'0'+digit);  
  
  PrintString(SCR_1_PLANE,0,13,0,"LIVES:");
  PutTile(SCR_1_PLANE,0,19,0,'0'+lives);    
}


void playerDeath() {
  int i;
  
  for(i=0;i<MAX_BULLET;i++) { UnsetSprite(4+i); }

  SysPlayWave(sfxExplode);

  SetBackgroundColour(RGB(7, 0, 0));
             
  SetColors(SPRITE_PLANE,VAR_PAL_BASE, 0,0,0 );
  SetColors(SPRITE_PLANE,VAR_PAL_BASE+1, 0,0,0 );
  SetColors(SPRITE_PLANE,VAR_PAL_BASE+2, 0,0,0 );
  SetColors(SPRITE_PLANE,VAR_PAL_BASE+3, 0,0,0 );
  WaitVsync();
  Sleep(3*60);
  SetBackgroundColour(RGB(0, 0, 0));
  
}


void fade() {

  int d,i;
  int pal=0;
    
  for(d=1; d<=9;d++) {  
    fadebox(1+pal,TILE_FADE,d,0);    
    WaitVsync();
    PlaySound(SND_FADE);
    Sleep(4);    
    fadebox(1+pal,TILE_FADE+1,d,0);    
    WaitVsync();
    Sleep(4);        
    pal = 1-pal;
  }

  // clear the traps
  FillScreen(SCR_2_PLANE, (u16)' ', 0);
  // unset all sprites
  for(i=0; i<63;i++) UnsetSprite(i);

  pal=0;
  for(d=1; d<=9;d++) {  
    fadebox(1+pal,TILE_FADE+2,d,0);    
    WaitVsync();
    PlaySound(SND_FADE);
    Sleep(4);        
    fadebox(1+pal,TILE_FADE+2,d,1);    
    WaitVsync();
    Sleep(4);        
    pal = 1-pal;
  }


}

void fadebox(int pal, int tile, int d,int clear) {
    int i;
    int x1,y1,x2,y2;

    x1=9-d;
    x2=10+d;

    y1=10-d;
    y2=9+d;
    
    PutTile(SCR_1_PLANE, pal, x1, y1, (clear?' ':tile+12) );
    PutTile(SCR_1_PLANE, pal, x2, y1, (clear?' ':tile+15) );
    PutTile(SCR_1_PLANE, pal, x1, y2, (clear?' ':tile+18) );
    PutTile(SCR_1_PLANE, pal, x2, y2, (clear?' ':tile+21) );
    for(i=x1+1;i<x2;i++) { 
      PutTile(SCR_1_PLANE, pal, i, y1, (clear?' ':tile) );
      PutTile(SCR_1_PLANE, pal, i, y2, (clear?' ':tile+3) );
    }
    for(i=y1+1;i<y2;i++) { 
      PutTile(SCR_1_PLANE, pal, x1, i, (clear?' ':tile+6) );
      PutTile(SCR_1_PLANE, pal, x2, i, (clear?' ':tile+9) );
    }
}

int playerMove() {

  int newdir=0;
  int move=0;

  int i;
  int dx,dy;
  int tx,ty;

  if (size == 16) {
    tx=x+4; ty=y+4;
  } else {
    tx=x; ty=y;
  }

 	if (SYS_LEVER & J_UP) {
    if (y>8) { y--; move=1; }
    newdir |= DIR_UP;
  }
 	if (SYS_LEVER & J_DOWN) { 
    if (y<152-size) { y++; move=1; }
    newdir |= DIR_DOWN;
  }
	if (SYS_LEVER & J_LEFT) { 
	  profile = DIR_LEFT;
    if (x>0) { x--; move=1; }
    newdir |= DIR_LEFT;
  }
 	if (SYS_LEVER & J_RIGHT) {
	  profile = DIR_RIGHT;
    if (x<160-size) { x++; move=1; }
    newdir |= DIR_RIGHT;
  }

  if ((newdir & DIR_UP) && (newdir & DIR_DOWN)) newdir=0;
  if ((newdir & DIR_LEFT) && (newdir & DIR_RIGHT)) newdir=0;
  
  if (move) { 
    frame++;
    if (frame == (1<<(ANIM_LSL+1))) frame = 0;
  }

  if (newdir) dir = newdir;

 	if (SYS_LEVER & J_A) {
 	  if (!dirlock) dirlock = dir; 
 	} else {
 	  dirlock = 0;
 	}

 	for(i=0;i<MAX_ENEMY;i++) {
 	  if (enemy[i].status == 1) {
      dx=enemy[i].x-tx; if(dx<0) dx=-dx;
      dy=enemy[i].y-ty; if(dy<0) dy=-dy;

      if ((dx < 8) && (dy < 8)) {
        // player death
        return 1;
      }
    }
  }

  return 0;
}




void playerDisplay() {

  if (size == 16) {
    // Main character == 16x16
    if (profile & DIR_LEFT) {
      sprite(0, VAR_TILE_BASE+1 + ((frame>>ANIM_LSL)<<2), 0, (u8)x, (u8)y, VAR_PAL_BASE+1,SPR_MIDDLE,SPR_HFLIP);      
      sprite(1, VAR_TILE_BASE+0 + ((frame>>ANIM_LSL)<<2), 1, 8, 0, VAR_PAL_BASE+0,SPR_MIDDLE, SPR_HFLIP);      
      sprite(2, VAR_TILE_BASE+3 + ((frame>>ANIM_LSL)<<2), 1, -8, 8, VAR_PAL_BASE+3, SPR_MIDDLE, SPR_HFLIP);
      sprite(3, VAR_TILE_BASE+2 + ((frame>>ANIM_LSL)<<2), 1, 8, 0, VAR_PAL_BASE+2, SPR_MIDDLE, SPR_HFLIP);          
    } else {
      sprite(0, VAR_TILE_BASE+0 + ((frame>>ANIM_LSL)<<2), 0, (u8)x, (u8)y, VAR_PAL_BASE+0,SPR_MIDDLE,0);
      sprite(1, VAR_TILE_BASE+1 + ((frame>>ANIM_LSL)<<2), 1, 8, 0, VAR_PAL_BASE+1,SPR_MIDDLE,0);
      sprite(2, VAR_TILE_BASE+2 + ((frame>>ANIM_LSL)<<2), 1, -8, 8, VAR_PAL_BASE+2,SPR_MIDDLE,0);
      sprite(3, VAR_TILE_BASE+3 + ((frame>>ANIM_LSL)<<2), 1, 8, 0, VAR_PAL_BASE+3,SPR_MIDDLE,0);  
    }
  } else {
    // Main character == 8x8  
    if (profile & DIR_LEFT) {
      sprite(0, VAR_TILE_BASE + ((frame>>ANIM_LSL)), 0, (u8)x, (u8)y, VAR_PAL_BASE+0,SPR_MIDDLE,SPR_HFLIP);      
    } else {
      sprite(0, VAR_TILE_BASE + ((frame>>ANIM_LSL)), 0, (u8)x, (u8)y, VAR_PAL_BASE+0,SPR_MIDDLE,0);      
    }    
  }

}

void bonusMove() {
  int i,s;
  int dx,dy;
  int tx,ty;
#define BONUS_STEP 20
  bonusMoveFrame++;
 	if (bonusMoveFrame != 3) return;
  bonusMoveFrame = 0;

  bonusAnimFrame++;
  if (bonusAnimFrame == (1<<(ANIM_LSL+1))) bonusAnimFrame = 0;

  if (size == 16) {
    tx=x+4; ty=y+4;
  } else {
    tx=x; ty=y;
  }

  for(i=0; i<MAX_BONUS; i++) {
    if (bonus[i].status == 1) {
    
      dx=bonus[i].x-tx; if(dx<0) dx=-dx;
      dy=bonus[i].y-ty; if(dy<0) dy=-dy;

      if ((dx < 8) && (dy < 8)) {
        if (bonus[i].score < 0) {
          bonus[i].status = 0;
          extraShoot = -bonus[i].score;
          UnsetSprite(4+MAX_BULLET+MAX_ENEMY+i);
        } else {
          // caught bonus
          bonus[i].status = -BONUS_FREQ;        
          score += bonus[i].score;
        }
        continue;
      }
    
      bonus[i].step++;
      if (bonus[i].step == 3*BONUS_STEP) bonus[i].step=0;
      s = bonus[i].step;
      
      if (s < BONUS_STEP) {
        bonus[i].x++;
      } else if (s < 2*BONUS_STEP) {
        bonus[i].y++;
      } else {
        bonus[i].x--;
        bonus[i].y--;
      }
    }
  }
}

void bonusDisplay() {
  int i,b;

  for(i=0; i<MAX_BONUS; i++) {
    if (bonus[i].status == 1) {
      sprite(4+MAX_BULLET+MAX_ENEMY+i, VAR_TILE_BASE+8+3+(bonus[i].tile<<1) + (bonusAnimFrame>>ANIM_LSL), 0, 
        (u8)bonus[i].x, (u8)bonus[i].y, VAR_PAL_BASE+bonus[i].tile+5,SPR_MIDDLE,0);
    } else if (bonus[i].status < 0) {
      // caught bonus
      b = bonus[i].score;
      if (b == 100) b=0;
      else if (b == 200) b=1;
      else if (b == 500) b=2;
      if (b >= 0) {
        sprite(4+MAX_BULLET+MAX_ENEMY+i, TILE_BONUS+b, 0, (u8)bonus[i].x, (u8)bonus[i].y, PAL_BONUS,SPR_MIDDLE,0);
        bonus[i].status++;
        if (bonus[i].status == 0) UnsetSprite(4+MAX_BULLET+MAX_ENEMY+i);  
      }
    } 
  }

}

void bham(int i) {
  // part of bresenham algo for enemy shoots
	int xinc1;
	int xinc2;
	int yinc1;
	int yinc2;
	int den;
	int num;
	int numadd;
	int deltax;
	int deltay;

	int x1,y1,x2,y2;
	
	x1 = enemy[i].x; y1 = enemy[i].y;
  if (size == 16) {
    x2=x+4; y2=y+4;
  } else {
    x2=x; y2=y;
  }

  // avoid horz and vert lines
  if (x1 == x2) x2++;
  if (y1 == y2) y2++;

	//There appears to be an absence of an abs() function.
	if (x2>x1)
		deltax = x2 - x1;        // The difference between the x's
	else
		deltax = x1 - x2;
	if (y2>y1)
		deltay = y2 - y1;        // The difference between the y's
	else
		deltay = y1 - y2;

	if (x2 >= x1)                 // The x-values are increasing
	{
		xinc1 = 1;
		xinc2 = 1;
	}
	else                          // The x-values are decreasing
	{
		xinc1 = -1;
		xinc2 = -1;
	}

	if (y2 >= y1)                 // The y-values are increasing
	{
		yinc1 = 1;
		yinc2 = 1;
	}
	else                          // The y-values are decreasing
	{
		yinc1 = -1;
		yinc2 = -1;
	}

	if (deltax >= deltay)         // There is at least one x-value for every y-value
	{
		xinc1 = 0;                  // Don't change the x when numerator >= denominator
		yinc2 = 0;                  // Don't change the y for every iteration
		den = deltax;
		num = deltax / 2;
		numadd = deltay;
	}
	else                          // There is at least one y-value for every x-value
	{
		xinc2 = 0;                  // Don't change the x for every iteration
		yinc1 = 0;                  // Don't change the y when numerator >= denominator
		den = deltay;
		num = deltay / 2;
		numadd = deltax;
	}

  enemy[i].dx = xinc1; enemy[i].dy = yinc1;
  enemy[i].dx2 = xinc2; enemy[i].dy2 = yinc2;
  enemy[i].num = num;
  enemy[i].den = den;
  enemy[i].numadd = numadd;
}

int enemyBulletMove() {
  int i;
  int tx,ty;
  int dx,dy;

  if (size == 16) {
    tx=x+4; ty=y+4;
  } else {
    tx=x; ty=y;
  }

  for(i=0; i<MAX_ENEMY; i++) {
    if ( (enemy[i].type == 2) && (enemy[i].status == 1)) {
      dx=enemy[i].x-tx; if(dx<0) dx=-dx;
      dy=enemy[i].y-ty; if(dy<0) dy=-dy;

      if ((dx < 7) && (dy < 7)) {
        // player death
        return 1;
      }
      
      enemy[i].num += enemy[i].numadd;              // Increase the numerator by the top of the fraction
		  if (enemy[i].num >= enemy[i].den)             // Check if numerator >= denominator
		  {
			  enemy[i].num -= enemy[i].den;               // Calculate the new numerator value
			  enemy[i].x += enemy[i].dx;               // Change the x as appropriate
			  enemy[i].y += enemy[i].dy;               // Change the y as appropriate
  		}
		  enemy[i].x += enemy[i].dx2;                 // Change the x as appropriate
		  enemy[i].y += enemy[i].dy2;                 // Change the y as appropriate

      if ( (enemy[i].x >= 160) || (enemy[i].x <= 0) || (enemy[i].y >= 152) || (enemy[i].y <= 8) ) {
        enemy[i].status = 0;
        UnsetSprite(4+MAX_BULLET+i);
      }
    }
  }

  return 0;

}

int enemyMove() {
  int i;
  int tx,ty;
  int dx,dy;

  enemyMoveFrame++;
 	if (enemyMoveFrame != enemySlow) return 0;
  enemyMoveFrame = 0;

  enemyAnimFrame++;
  if (enemyAnimFrame == (1<<(ANIM_LSL+1))) enemyAnimFrame = 0;

  if (enemyAnimFrame == 0) enemyShootFrame++;

  if (size == 16) {
    tx=x+4; ty=y+4;
  } else {
    tx=x; ty=y;
  }

  for(i=0; i<MAX_ENEMY; i++) {

    if (enemy[i].status == 1) {

      if (enemy[i].type == 1) {
        // shooter
        enemy[i].x += enemy[i].dx;
        enemy[i].y += enemy[i].dy;
        if (enemy[i].x >= 160-8) { enemy[i].dx =-1; }
        if (enemy[i].x <= 0) { enemy[i].dx =1; }
        if (enemy[i].y >= 152-8) { enemy[i].dy =-1; }
        if (enemy[i].y <= 8) { enemy[i].dy =1; }                                              

        // shooters has "profile" management
        if (enemy[i].dx >= 0) {
          enemy[i].profile = 0;
        } else {
          enemy[i].profile = SPR_HFLIP;
        }

        if ((enemyShootFrame % enemyShootSlow) == 0) {          
          // shoot a bullet
          if (enemy[i+1].status == 0) {
            enemyShootFrame++;
            enemy[i+1].status = 1;
            enemy[i+1].hp = 1;
            enemy[i+1].score = 0;
            enemy[i+1].type = 2;
            enemy[i+1].tile = enemy[i].tile+1;
            enemy[i+1].x = enemy[i].x;
            enemy[i+1].y = enemy[i].y;                
            bham(i+1);
          }
        }
      } else if (enemy[i].type == 0) {
        // standard enemy

        dx=enemy[i].x-tx; if(dx<0) dx=-dx;
        dy=enemy[i].y-ty; if(dy<0) dy=-dy;

        if ((dx < 8) && (dy < 8)) {
          // player death
          return 1;
        }

        if ( (i % 4) == 0 ) { 
          // AI n1
          if (dx > dy) {
            if (enemy[i].x < tx) enemy[i].x++;
            if (enemy[i].x > tx) enemy[i].x--;
          } else if (dx < dy) {
            if (enemy[i].y < ty) enemy[i].y++;
            if (enemy[i].y > ty) enemy[i].y--;                    
          } else {
            if (enemy[i].x < tx) enemy[i].x++;
            if (enemy[i].x > tx) enemy[i].x--;
            if (enemy[i].y < ty) enemy[i].y++;
            if (enemy[i].y > ty) enemy[i].y--;            
          }
        }
        if ( (i % 4) == 1 ) { 
          // AI n2
          if (enemy[i].x < tx) enemy[i].x++;
          if (enemy[i].x > tx) enemy[i].x--;
          if (enemy[i].y < ty) enemy[i].y++;
          if (enemy[i].y > ty) enemy[i].y--;            
        }
        if ( (i % 4) == 2 ) { 
          // AI n3
          if (dx > 24) {
            if (enemy[i].x < tx) enemy[i].x++;
            if (enemy[i].x > tx) enemy[i].x--;
          } else if (dy > 24) {
            if (enemy[i].y < ty) enemy[i].y++;
            if (enemy[i].y > ty) enemy[i].y--;                    
          } else {
            if (enemy[i].x < tx) enemy[i].x++;
            if (enemy[i].x > tx) enemy[i].x--;
            if (enemy[i].y < ty) enemy[i].y++;
            if (enemy[i].y > ty) enemy[i].y--;            
          }
        }
        if ( (i % 4) == 3 ) { 
          // AI n4
          if (dy > 24) {
            if (enemy[i].y < ty) enemy[i].y++;
            if (enemy[i].y > ty) enemy[i].y--;                    
          } else if (dx > 24) {
            if (enemy[i].x < tx) enemy[i].x++;
            if (enemy[i].x > tx) enemy[i].x--;
          } else {
            if (enemy[i].x < tx) enemy[i].x++;
            if (enemy[i].x > tx) enemy[i].x--;
            if (enemy[i].y < ty) enemy[i].y++;
            if (enemy[i].y > ty) enemy[i].y--;            
          }
        }
      }
      
    } else if (enemy[i].status < 0) {
      // dying enemy
    } 
  }
  
  return 0;
}

int enemyDisplay() {
  int i;

  for(i=0; i<MAX_ENEMY; i++) {
    if (enemy[i].status == 1) {
      if (enemy[i].type == 1) {
        // shooters has "profile" management
        sprite(4+MAX_BULLET+i, VAR_TILE_BASE+8+3+(enemy[i].tile<<1) + (enemyAnimFrame>>ANIM_LSL), 0, 
          (u8)enemy[i].x, (u8)enemy[i].y, VAR_PAL_BASE+enemy[i].tile+5,SPR_MIDDLE,enemy[i].profile);
      } else {
        sprite(4+MAX_BULLET+i, VAR_TILE_BASE+8+3+(enemy[i].tile<<1) + (enemyAnimFrame>>ANIM_LSL), 0, 
          (u8)enemy[i].x, (u8)enemy[i].y, VAR_PAL_BASE+enemy[i].tile+5,SPR_MIDDLE,0);
      }
    } else if (enemy[i].status < 0) {
      // dying enemy
      sprite(4+MAX_BULLET+i, TILE_EXP, 0, (u8)enemy[i].x, (u8)enemy[i].y, PAL_EXP,SPR_MIDDLE,0);
      enemy[i].status++;
      if (enemy[i].status == 0) UnsetSprite(4+MAX_BULLET+i);  
    } 
  }

  for(i=0; i<MAX_ENEMY; i++) {
    if (enemy[i].status != 0) return 0;
  }

  return 1;

}


const int bulletDirs [] = { DIR_UP, DIR_UP|DIR_RIGHT, DIR_RIGHT, DIR_DOWN|DIR_RIGHT , 
                            DIR_DOWN, DIR_DOWN|DIR_LEFT, DIR_LEFT, DIR_LEFT|DIR_UP };

void shoot() {
  int i,j,k;
  int buldir,bd;

  shootFrame++;
 	if (shootFrame != SHOOT_FREQ) return;
  shootFrame = 0;
  
  for(i=0; i<MAX_BULLET;i++) {
    if (!bullet[i].active) break;
  }
  if (i == MAX_BULLET) return;

  //SysPlayWave(sfxShoot);
  PlaySound(SND_SHOOT);
       
  bullet[i].active = 1;
  if (extraShoot == 1) {
    bullet[i].ttl = 20;
  } else {
    bullet[i].ttl = 200;
  }

  if (dirlock) {
    buldir = dirlock;
  } else {
    buldir = dir;
  }
  bullet[i].dir = buldir;
  
  if (size == 16) {
    bullet[i].x = x+4;
    bullet[i].y = y+4;
  } else {
    bullet[i].x = x;
    bullet[i].y = y;  
  }
  
  if (extraShoot == 1) {
    for(j=0;j<2;j++) {
      for(i=0; i<MAX_BULLET;i++) {
        if (!bullet[i].active) break;
      }
      if (i == MAX_BULLET) return;
      for(k=0;k<8;k++) {
        if (buldir == bulletDirs[k]) {
          bd = bulletDirs[ (k+(j==0?1:7)) % 8 ];
          break;
        }
      }

      if (size == 16) {
        bullet[i].x = x+4;
        bullet[i].y = y+4;
      } else {
        bullet[i].x = x;
        bullet[i].y = y;  
      }
      bullet[i].active = 1;
      bullet[i].ttl = 20;
      bullet[i].dir = bd;
    }
  }
  
}

void bulletMove() {
  int i,j;
  int flags,tile;
  int dx,dy;
  
#define BUL_STEP 2
  for(i=0; i<MAX_BULLET;i++) {
    if (bullet[i].active) {

        bullet[i].ttl--;
        if (bullet[i].ttl < 0) {
          UnsetSprite(4+i);
          bullet[i].active = 0;
          continue;
        }
          
        //if (!bullet[i].active) continue;        

        // Trap collision
        for(j=0; j<MAX_TRAP;j++) {
          if (trap[j].status == 1) {
            dx=bullet[i].x-(trap[j].x<<3); if (dx<0) dx=-dx;
            dy=bullet[i].y-(trap[j].y<<3); if (dy<0) dy=-dy;
            
            if ((dx<6) && (dy<6)) {
              if (extraShoot != 2) {            
                bullet[i].active = 0;
                UnsetSprite(4+i);
              }
              trap[j].hp--;
              if (trap[j].hp == 0) {
                trap[j].status = -EXPLODE_FREQ;
                score += trap[j].score;
                //SysPlayWave(sfxShoot);
                WaitVsync();
                PlaySound(SND_FADE);

                //SetBackgroundColour(RGB(15, 15, 15));
                //WaitVsync();
                //SetBackgroundColour(RGB(0, 0, 0));
              }
              break;
            }
            
          }
        }

        if (!bullet[i].active) continue;        

        // Enemy collision
        for(j=0; j<MAX_ENEMY;j++) {
          if (enemy[j].status == 1) {
            dx=bullet[i].x-enemy[j].x; if (dx<0) dx=-dx;
            dy=bullet[i].y-enemy[j].y; if (dy<0) dy=-dy;
            
            if ((dx<6) && (dy<6)) {
              if (extraShoot != 2) {                        
                bullet[i].active = 0;
                UnsetSprite(4+i);
              }
              enemy[j].hp--;
              if (enemy[j].hp == 0) {
                enemy[j].status = -EXPLODE_FREQ;
                score += enemy[j].score;
                killCount++;
                WaitVsync();
                if ((killCount % 4) == 3 ) {
                  SysPlayWave(sfxShoot);
                }
                PlaySound(SND_FADE);
                SetBackgroundColour(RGB(15, 15, 15));
                WaitVsync();
                SetBackgroundColour(RGB(0, 0, 0));
              }
              break;
            }
            
          }
        }

        if (!bullet[i].active) continue;        
   
        tile=0;
        flags=0;
   
        if (bullet[i].dir & DIR_UP) {
          if (bullet[i].dir & DIR_LEFT) {
            tile=2;
          } else if (bullet[i].dir & DIR_RIGHT) {
            tile=2;
          } else {
            tile=1;
          }
          bullet[i].y -= BUL_STEP;
        }
        if (bullet[i].dir & DIR_DOWN) {
          if (bullet[i].dir & DIR_LEFT) {
            tile=2;
          } else if (bullet[i].dir & DIR_RIGHT) {
            tile=2;
          } else {
            tile=1;
          }
          flags |= SPR_VFLIP;        
          bullet[i].y += BUL_STEP;
        }        
        if (bullet[i].dir & DIR_LEFT) bullet[i].x -= BUL_STEP;
        if (bullet[i].dir & DIR_RIGHT) {
          bullet[i].x += BUL_STEP;
          flags |= SPR_HFLIP;
        }
    
        if ( (bullet[i].x<0) || (bullet[i].x>=160) 
                || (bullet[i].y<8) || (bullet[i].y>=152) ) {
          UnsetSprite(4+i);
          bullet[i].active = 0;
        } else {

          if (extraShoot == 2) {        
            switch(bullet[i].ttl % 3) {
              case 0: SetColors(SPRITE_PLANE,VAR_PAL_BASE+4,0,RGB(15,0,0),RGB(0,15,0),RGB(0,0,15)); break;                    
              case 1: SetColors(SPRITE_PLANE,VAR_PAL_BASE+4,0,RGB(0,0,15),RGB(15,0,0),RGB(0,15,0)); break;                    
              case 2: SetColors(SPRITE_PLANE,VAR_PAL_BASE+4,0,RGB(0,15,0),RGB(0,0,15),RGB(15,0,0)); break;                    
            }
          }
          sprite(4+i,VAR_TILE_BASE+8+tile,0,bullet[i].x,bullet[i].y,VAR_PAL_BASE+4,SPR_MIDDLE,flags);
        }
    }
  }
}

//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////

void sprite(int SpriteNo, int TileNo, int Chain, int XPos, int YPos, int PaletteNo, int Priority, int Flips) {
   u16 SprCtrlReg;
   u8 * theSprite = SPRITE_RAM;
   u8 * theSpriteCol = SPRITE_COLOUR;
   
   theSprite += ((u16)SpriteNo * 4);
   theSpriteCol += (u16)SpriteNo;

   //SprCtrlReg = 24; // topmost priority
   SprCtrlReg = Priority;
   if (Chain)
      SprCtrlReg += 6; // v and h chaining

   *(theSprite)   = TileNo;
   *(theSprite+1) = (SprCtrlReg+(TileNo>>8) &0x27) | Priority | Flips ;
   *(theSprite+2) = (u8)XPos;
   *(theSprite+3) = (u8)YPos;

   *theSpriteCol = (u8)PaletteNo;
}

//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////

void clearDisplay() {
  int i;

  // bg color
  SetBackgroundColour(RGB(0, 0, 0));

  // text color
  SetColors(SCR_1_PLANE, 0, 0, 0, RGB(15, 15, 15));
    
  // print blanks on screen - really clear it
  FillScreen(SCR_1_PLANE, (u16)' ', 0);
  FillScreen(SCR_2_PLANE, (u16)' ', 0);

  // unset all sprites
  for(i=0; i<63;i++) {
      UnsetSprite(i);
  }
    
}

void clearStage() {
  int i;

  x=80; y=80; profile=DIR_LEFT; dir=DIR_LEFT; 

  frame=0;
  shootFrame = 0;
  
  enemyMoveFrame = 0;
  enemyAnimFrame = 0;

  trapMoveFrame = 0;
  trapAnimFrame = 0;

  bonusMoveFrame = 0;
  bonusAnimFrame = 0;

  enemyShootFrame = 0;

  killCount = 0;

  extraShoot = 0;

  for(i=0;i<MAX_BULLET;i++) {
    bullet[i].active = 0;
  }
  for(i=0;i<MAX_ENEMY;i++) {
    if (enemy[i].type == 2) enemy[i].status = 0;  
    if (enemy[i].status<0) enemy[i].status = 0;  
  }
  for(i=0;i<MAX_BONUS;i++) {
    if (bonus[i].status<0) bonus[i].status = 0;  
    bonus[i].step = 0;
  }

  for(i=0;i<MAX_TRAP;i++) {
    if (trap[i].status<0) trap[i].status = 0;  
  }
  
}



