// ORGANIZATION
// 8 tiles = main character
// 3 tiles = bullets
// X*2 tiles = enemies/bonus

#include "ngpc.h"		// required
#include "library.h"		// NGPC routines

#include "common.h"

const unsigned short pac[][8] = {
  // character - 2 8x8 tiles
	{0x00550, 0x01514, 0x05555, 0x05555, 0x05540, 0x05555, 0x01554, 0x00550},
 	{0x00550, 0x01514, 0x05554, 0x05540, 0x05500, 0x05540, 0x01554, 0x00550},
};

const unsigned short pac2[][8] = {
  // ghost1 - 2 8x8 tiles
	{0x01550, 0x05554, 0x05964, 0x05964, 0x05964, 0x05554, 0x05554, 0x04444},
	{0x01550, 0x05554, 0x05964, 0x05964, 0x05964, 0x05554, 0x05554, 0x01110},
  // ghost2 - 2 8x8 tiles
	{0x01550, 0x05554, 0x05964, 0x05964, 0x05964, 0x05554, 0x05554, 0x04444},
	{0x01550, 0x05554, 0x05964, 0x05964, 0x05964, 0x05554, 0x05554, 0x01110},
  // ghost3 (shooter) - 2 8x8 tiles
	{0x01550, 0x05554, 0x05964, 0x05964, 0x05964, 0x05554, 0x05554, 0x04444},
	{0x01550, 0x05554, 0x05964, 0x05964, 0x05964, 0x05554, 0x05554, 0x01110},
  // bullets - 2 8x8 tiles
	{0x00000, 0x00540, 0x01550, 0x01990, 0x01550, 0x01110, 0x00000, 0x00000},
	{0x00000, 0x00000, 0x00150, 0x00554, 0x00664, 0x00554, 0x00110, 0x00000},
  // bonus - 2 8x8 tiles
	{0x00020, 0x00020, 0x00088, 0x00208, 0x00805, 0x01415, 0x05505, 0x01400},
	{0x00020, 0x00020, 0x00088, 0x00208, 0x00805, 0x01415, 0x05505, 0x01400},
};

const unsigned short pacbul[][8] = {
  // bullets - 3 8x8 tiles
	{0x00000, 0x00000, 0x00000, 0x00540, 0x01a90, 0x00540, 0x00000, 0x00000},
	{0x00040, 0x00190, 0x00190, 0x00190, 0x00040, 0x00000, 0x00000, 0x00000},
	{0x01000, 0x06400, 0x01900, 0x00640, 0x00100, 0x00000, 0x00000, 0x00000},
};

void pacInit(int tileBase, int palBase) {

  // Main character size
  size = 8;

  // ==== Install Tile Sets
  // Main character
  InstallTileSetAt( pac, (sizeof(pac)>>1), tileBase); // 2 x 16x16  
  // Bullets
  InstallTileSetAt( pacbul, (sizeof(pacbul)>>1), tileBase + 8); // 3 x 8x8    
  // Enemies, bonus
  InstallTileSetAt( pac2, (sizeof(pac2)>>1), tileBase + 8 + 3); // 4 x 8x8

  // ==== Install palettes
  // Main character
  SetColors( SPRITE_PLANE, palBase, RGB(15,15,7), 0, 0 );  
  // Bullets
  SetColors( SPRITE_PLANE, palBase+4, RGB(0,15,0), RGB(15,0,0), 0 );  
  // Enemies
  SetColors( SPRITE_PLANE, palBase+5, RGB(15,3,3), 0, 0 );  
  SetColors( SPRITE_PLANE, palBase+6, RGB(3,3,15), 0, 0 );  
  //shooter + bullets
  SetColors( SPRITE_PLANE, palBase+7, RGB(3,15,3), 0, 0 );  
  SetColors( SPRITE_PLANE, palBase+8, RGB(3,15,3), 0, 0 );  
  //bonus
  SetColors( SPRITE_PLANE, palBase+9, RGB(15,0,0), RGB(0,10,0), 0 );  
}

void pacInitLevel(int level, int firstTime) {
  int i;
  
  enemySlow = 3;
  enemyShootSlow = 3;
    
  for(i=0;i<10;i++) {
    if (firstTime) enemy[i].status = 1;
    enemy[i].x = 2*8 + (i % 3)*10;
    enemy[i].y = 10+i*12;
    enemy[i].tile = (i%2);
    enemy[i].hp = 1;
    enemy[i].score = 10;
  }

  for(i=0;i<10;i++) {  
    if (firstTime) enemy[i+10].status = 1;
    enemy[i+10].x = 150 - (i % 3)*10;
    enemy[i+10].y = 10+i*12;
    enemy[i+10].tile = (i%2);
    enemy[i+10].hp = 1;
    enemy[i+10].score = 10;
  }

  i=24;
  if (firstTime) enemy[i].status = 1;
  enemy[i].x = 30;
  enemy[i].y = 10;
  enemy[i].tile = 2;
  enemy[i].hp = 5;
  enemy[i].score = 50;
  
  enemy[i].type = 1;
  enemy[i].dx = 1;
  enemy[i].dy = 0;
  
  i+=2;
  if (firstTime) enemy[i].status = 1;
  enemy[i].x = 130;
  enemy[i].y = 140;
  enemy[i].tile = 2;
  enemy[i].hp = 5;
  enemy[i].score = 50;
  
  enemy[i].type = 1;
  enemy[i].dx = -1;
  enemy[i].dy = 0;

  if (firstTime) bonus[0].status = 1;
  bonus[0].x = 10;
  bonus[0].y = 10;
  bonus[0].score = 200;
  bonus[0].tile = 4;
  if (firstTime) bonus[1].status = 1;
  bonus[1].x = 120;
  bonus[1].y = 10;
  bonus[1].score = 200;
  bonus[1].tile = 4;



}
