// ORGANIZATION
// 8 tiles = main character
// 3 tiles = bullets
// X*2 tiles = enemies/bonus

#include "ngpc.h"		// required
#include "library.h"		// NGPC routines

#include "common.h"

const unsigned short rob[][8] = {
  // character - 2 8x8 tiles
	{0x002b0, 0x00ff0, 0x00eb0, 0x00140, 0x00140, 0x00150, 0x00110, 0x00114},
	{0x002b0, 0x00ff0, 0x00eb0, 0x00140, 0x00150, 0x00100, 0x00100, 0x00140},
};

const unsigned short rob2[][8] = {
  // red suit - 2 8x8 tiles
	{0x00140, 0x00280, 0x05ff5, 0x057d5, 0x04551, 0x04551, 0x01410, 0x00014},
	{0x00140, 0x00280, 0x05ff5, 0x057d5, 0x04551, 0x04551, 0x00414, 0x01400},
  // boxed - 2 8x8 tiles
	{0x00280, 0x00140, 0x0eaab, 0x0eaab, 0x02aa8, 0x02aa8, 0x01415, 0x05400},
	{0x00280, 0x00140, 0x0eaab, 0x0eaab, 0x02aa8, 0x02aa8, 0x05414, 0x00015},
	// bullet - 2 8x8 tiles
  {0x00000, 0x00000, 0x00040, 0x00040, 0x00554, 0x00040, 0x00040, 0x00000},
	{0x00000, 0x00000, 0x00404, 0x00110, 0x00040, 0x00110, 0x00404, 0x00000},
	// bonus(human) - 2 8x8 tiles
	{0x00000, 0x00000, 0x00440, 0x00150, 0x00044, 0x00040, 0x00110, 0x00104},
	{0x00000, 0x00000, 0x00044, 0x00150, 0x00440, 0x00040, 0x00110, 0x00410},
};

const unsigned short robbul[][8] = {
  // bullets - 3 8x8 tiles
	{0x00000, 0x00000, 0x00000, 0x05555, 0x05555, 0x00000, 0x00000, 0x00000},
	{0x00140, 0x00140, 0x00140, 0x00140, 0x00140, 0x00140, 0x00140, 0x00140},
	{0x05000, 0x05400, 0x01500, 0x00540, 0x00150, 0x00054, 0x00015, 0x00005},
};

void robInit(int tileBase, int palBase) {

  // Main character size
  size = 8;

  // ==== Install Tile Sets
  // Main character
  InstallTileSetAt( rob, (sizeof(rob)>>1), tileBase); // 2 x 16x16  
  // Bullets
  InstallTileSetAt( robbul, (sizeof(robbul)>>1), tileBase + 8); // 3 x 8x8    
  // Enemies, bonus
  InstallTileSetAt( rob2, (sizeof(rob2)>>1), tileBase + 8 + 3); // 4 x 8x8

  // ==== Install palettes
  // Main character
  SetColors( SPRITE_PLANE, palBase, RGB(15,15,15), RGB(15,0,0), RGB(0,15,0) );  
  // Bullets
  SetColors( SPRITE_PLANE, palBase+4, RGB(15,15,15), 0, 0 );  
  // Enemies
  SetColors( SPRITE_PLANE, palBase+5, RGB(15,0,0), RGB(0,15,0), RGB(15,15,15) );  
  SetColors( SPRITE_PLANE, palBase+6, RGB(15,0,0), RGB(0,15,0), RGB(15,15,15) );  
  SetColors( SPRITE_PLANE, palBase+7, RGB(15,15,15), 0, 0 );  
  // Bonus
  SetColors( SPRITE_PLANE, palBase+8, RGB(15,15,7), 0, 0 );  
}

void robInitLevel(int level, int firstTime) {
  int i;
  
  enemySlow = 3;
  enemyShootSlow = 3;
  //enemyShootSlow = 1;
  
  for(i=0;i<10;i++) {
    if (firstTime) enemy[i].status = 1;
    enemy[i].x = 2*8 + (i % 3)*10;
    enemy[i].y = 10+i*12;
    enemy[i].tile = 0;
    enemy[i].hp = 1;
    enemy[i].score = 10;
  }

  for(i=0;i<10;i++) {  
    if (firstTime) enemy[i+10].status = 1;
    enemy[i+10].x = 150 - (i % 3)*10;
    enemy[i+10].y = 10+i*12;
    enemy[i+10].tile = 0;
    enemy[i+10].hp = 1;
    enemy[i+10].score = 10;
  }

  if (level == 0) return;

  i=20;
  if (firstTime) enemy[i].status = 1;
  enemy[i].x = 10;
  enemy[i].y = 30;
  enemy[i].tile = 1;
  enemy[i].hp = 5;
  enemy[i].score = 50;
  
  enemy[i].type = 1;
  enemy[i].dx = 1;
  enemy[i].dy = 0;

  i+=2;
  if (firstTime) enemy[i].status = 1;
  enemy[i].x = 140;
  enemy[i].y = 140;
  enemy[i].tile = 1;
  enemy[i].hp = 5;
  enemy[i].score = 50;
  
  enemy[i].type = 1;
  enemy[i].dx = -1;
  enemy[i].dy = 0;

  //bonus
  if (firstTime) bonus[0].status = 1;
  bonus[0].x = 10;
  bonus[0].y = 10;
  bonus[0].score = 100;
  bonus[0].tile = 3;

  if (level == 1) return;
  
  i+=2;
  if (firstTime) enemy[i].status = 1;
  enemy[i].x = 50;
  enemy[i].y = 140;
  enemy[i].tile = 1;
  enemy[i].hp = 5;
  enemy[i].score = 50;
  
  enemy[i].type = 1;
  enemy[i].dx = 0;
  enemy[i].dy = -1;

  i+=2;
  if (firstTime) enemy[i].status = 1;
  enemy[i].x = 150;
  enemy[i].y = 50;
  enemy[i].tile = 1;
  enemy[i].hp = 5;
  enemy[i].score = 50;
  
  enemy[i].type = 1;
  enemy[i].dx = 1;
  enemy[i].dy = 1;
  

}
