// ORGANIZATION
// 8 tiles = main character
// 3 tiles = bullets
// X*2 tiles = enemies/bonus

#include "ngpc.h"		// required
#include "library.h"		// NGPC routines

#include "common.h"

const unsigned short vpc[][8] = {
  // character - 2 8x8 tiles
	{0x00150, 0x00150, 0x00140, 0x00154, 0x00140, 0x00140, 0x00140, 0x00150},
	{0x00150, 0x00150, 0x00140, 0x00154, 0x00140, 0x00150, 0x00510, 0x00414},
};

const unsigned short vpc2[][8] = {
  // enemy - 2 8x8 tiles
	{0x04001, 0x01144, 0x00550, 0x01144, 0x05555, 0x04551, 0x01004, 0x00550},
	{0x00000, 0x05145, 0x00550, 0x01144, 0x05555, 0x04551, 0x01004, 0x00550},
  // shooter
  {0x00550, 0x01554, 0x01144, 0x01554, 0x00410, 0x00140, 0x00450, 0x01400},
	{0x00550, 0x01554, 0x01144, 0x01554, 0x00410, 0x00140, 0x00510, 0x00014},
	// shooter bullets
	{0x00000, 0x00140, 0x00410, 0x01104, 0x01044, 0x00410, 0x00140, 0x00000},
	{0x00000, 0x00140, 0x00410, 0x01044, 0x01104, 0x00410, 0x00140, 0x00000},
};

const unsigned short vpcbul[][8] = {
  // bullets - 3 8x8 tiles
	{0x00000, 0x00000, 0x00000, 0x00140, 0x00140, 0x00000, 0x00000, 0x00000},
	{0x00000, 0x00000, 0x00000, 0x00140, 0x00140, 0x00000, 0x00000, 0x00000},
  {0x00000, 0x00000, 0x00000, 0x00140, 0x00140, 0x00000, 0x00000, 0x00000},
};

void vpcInit(int tileBase, int palBase) {

  // Main character size
  size = 8;

  // ==== Install Tile Sets
  // Main character
  InstallTileSetAt( vpc, (sizeof(vpc)>>1), tileBase); // 2 x 16x16  
  // Bullets
  InstallTileSetAt( vpcbul, (sizeof(vpcbul)>>1), tileBase + 8); // 3 x 8x8    
  // Enemies, bonus
  InstallTileSetAt( vpc2, (sizeof(vpc2)>>1), tileBase + 8 + 3); // 4 x 8x8

  // ==== Install palettes
  // Main character
  SetColors( SPRITE_PLANE, palBase, RGB(15,15,15), 0, 0 );  
  // Bullets
  SetColors( SPRITE_PLANE, palBase+4, RGB(0,15,0), 0, 0 );  
  // Enemies, bonus
  SetColors( SPRITE_PLANE, palBase+5, RGB(5,5,15), 0, 0 );  
  SetColors( SPRITE_PLANE, palBase+6, RGB(15,8,8), 0, 0 );  
  SetColors( SPRITE_PLANE, palBase+7, RGB(8,15,15), 0, 0 );  
}

void vpcInitLevel(int level, int firstTime) {
  int i;
  
  enemySlow = 3;
  enemyShootSlow = 3;
  
  for(i=0;i<10;i++) {
    if (firstTime) enemy[i].status = 1;
    enemy[i].x = 2*8 + (i % 3)*10;
    enemy[i].y = 10+i*12;
    enemy[i].tile = 0;
    enemy[i].hp = 1;
    enemy[i].score = 10;
  }

  for(i=0;i<10;i++) {  
    if (firstTime) enemy[i+10].status = 1;
    enemy[i+10].x = 150 - (i % 3)*10;
    enemy[i+10].y = 10+i*12;
    enemy[i+10].tile = 0;
    enemy[i+10].hp = 1;
    enemy[i+10].score = 10;
  }

  if (level == 0) return;

  i=20;
  if (firstTime) enemy[i].status = 1;
  enemy[i].x = 10;
  enemy[i].y = 30;
  enemy[i].tile = 1;
  enemy[i].hp = 5;
  enemy[i].score = 50;
  
  enemy[i].type = 1;
  enemy[i].dx = 1;
  enemy[i].dy = 0;

  i+=2;
  if (firstTime) enemy[i].status = 1;
  enemy[i].x = 140;
  enemy[i].y = 140;
  enemy[i].tile = 1;
  enemy[i].hp = 5;
  enemy[i].score = 50;
  
  enemy[i].type = 1;
  enemy[i].dx = -1;
  enemy[i].dy = 0;

  i+=2;
  if (firstTime) enemy[i].status = 1;
  enemy[i].x = 50;
  enemy[i].y = 140;
  enemy[i].tile = 1;
  enemy[i].hp = 5;
  enemy[i].score = 50;
  
  enemy[i].type = 1;
  enemy[i].dx = 0;
  enemy[i].dy = -1;

  i+=2;
  if (firstTime) enemy[i].status = 1;
  enemy[i].x = 150;
  enemy[i].y = 50;
  enemy[i].tile = 1;
  enemy[i].hp = 5;
  enemy[i].score = 50;
  
  enemy[i].type = 1;
  enemy[i].dx = 1;
  enemy[i].dy = 1;


}
