
; Tell the AS assembler we want to code for NGPC
;
        cpu 96C141
        maxmode on
        include "stddef96.inc"


        include "hardware.inc"  ; Include hardware defines
        include "system.inc"
;
; Define variables
;


START_OF_RAM    EVAL    _MAINRAM
   RAMDW VBCount

HEADER_OFS      EQU     0200000H

;
; Standard cartridge header
;
    org    HEADER_OFS
    db     " LICENSED BY SNK CORPORATION"    ; 28 bytes license string
    dd     Start                             ; Program Counter
    dw     0                                 ; Catalog number
    db     0                                 ; Sub catalog number
    db     10h                               ; colour or b+w (10h = colour)
    db     "TIMER TEST  "                    ; Game name (12 bytes)
    dd     0,0,0,0                           ; padding - reserve for future use

; User Interrupt Vectors

UserIntVect:
        dd      nada            ; Software Interrupt (SWI 3)
        dd      nada            ; Software Interrupt (SWI 4)
        dd      nada            ; Software Interrupt (SWI 5)
        dd      nada            ; Software Interrupt (SWI 6)
        dd      nada            ; RTC Alarm Interrupt
        dd      VBlankInt       ; Vertical Blanking Interrupt
        dd      nada            ; Interrupt from Z80
        dd      Timer0            ; Timer Interrupt (8 bit timer 0)
        dd      nada            ; Timer Interrupt (8 bit timer 1)
        dd      nada            ; Timer Interrupt (8 bit timer 2)
        dd      nada            ; Timer Interrupt (8 bit timer 3)
        dd      nada            ; Serial Transmission Interrupt
        dd      nada            ; Serial Reception Interrupt
        dd      nada            ; (Reserved)
        dd      nada            ; End Micro DMA Int (MicroDMA 0)
        dd      nada            ; End Micro DMA Int (MicroDMA 1)
        dd      nada            ; End Micro DMA Int (MicroDMA 2)
        dd      nada            ; End Micro DMA Int (MicroDMA 3)

nada:   reti

; *** Start of User Code ***

Start:
        calr    OS_VERSION      ; Initialize NGP or NGPC mode

        set     6,(rUSERA)      ; User Answer

; Install User Interrupt Vectors

        lda     xix,(UserIntVect)
        lda     xiy,(rSWI3)
        ld      b,18
UIVloop:
        ld      xwa,(xix+)
        ld      (xiy+),xwa
        djnz    b,UIVloop


    ; set up screen size 160x152
    ld   (08002h),0
    ld   (08003h),0
    ld   (08004h),0a0h
    ld   (08005h),98h

    ; set up timer0
    andb	(TRUN),10001110b	;stop count for timer 0
    ldb	(TMOD),00h		;timer 0,1 operation mode <8 bit timer mode>
                           ;timer 0 input clock<external clock TI0>
    ldb	(TREG0),01h       ;interval tine is 0x01

    ldb rw3, VECT_INTLVSET
    ldb rb3, 3
    ldb rc3, 2
    swi 1

    orb	(TRUN),00000001b	;start count for timer 0


; Enable Interrupts

        ei      0

    ; background colour 
    ; _BGCPAL = 0xBGR
    ldw (_BGCPAL), 0036h
    ld (8118h), 80h

MAIN

; User shutdown? (Power off pressed?)

        cp      (rUSERS),0
        jp      z,Main      ; no

; Power off NGP

        ld      rw3,VECT_SHUTDOWN
        calr    SYSTEM_CALL

done:   jr      done




VBlankInt:

    ; Update Watch Dog Timer to prevent CPU reset
    ld      (rWDCR),WD_CLR
    inc (VBCount)
    reti


Timer0:
    push wa
    ld w, 0
    ld a, (8009h)
    srl 1, a
    add a, (VBCount)
    and a, 31
    bit 3, a   
    jr z, t1
    neg a
    and a, 31
t1:
    sll 4, wa
    ldw (_BGCPAL), wa
    ld (8118h), 80h
    pop wa
   reti


SYSTEM_CALL:
        SystemCallCode
OS_VERSION:
        OsVersionCode

    ; make the cartridge size correct
    org HEADER_OFS + 07FFFh
    db 0FFh

