/*
	display .BMP with 8x8 grid lines
*/


// compile    gcc -O4 -o gridview.exe gridvw.c -lalleg

#include <fcntl.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <allegro.h>


typedef unsigned char   uchar;
typedef unsigned short  ushort;
typedef unsigned long   ulong;
typedef unsigned int    uint;

unsigned char	header[300];
unsigned char	indat[60000];

unsigned	Xsiz,
		Ysiz;

unsigned int	fil_len = 0;

int		in_hand, out_hand;
FILE		*ofil;
int		xg,
		yg;
int		sX,
		sY;

PALLETE pallete;

int	Xoff = 10,	// offsets into grid
	Yoff = 10;

unsigned int	Xlc, Ylc;

long	lookup[49];	// listing of palettes
int	lptr = 0;	// number used

int	sizxx;		// picture X size
int	sizyy;		// picture Y size

int	byt_size,	// bytes in picture
	pix_size;	// pixels in picture

int	tmp,
	cnt;

int	histo[16];	// histogram data
int	hmax = 0,	// size of max color use
	hind;		// index of that color

int	xch,		// X character counter
	ych,		// Y character counter
	xpx,		// X pixel counter
	ypx;		// Y pixel counter

int	spr_used = 0,	// number of sprites to show image
	pl2_used = 0,	// number of times second plane used
	mtc_used = 0,	// number of empty blocks
	blc_used = 0,	// number of one color blocks
	co2_used = 0;	// number of 2 color blocks

uchar	blk_col[380],	// colors used by block
	blk_pal[380],	// palettes defined for block
	M_pal[96];	// master palette

int	pal_cnt;	// index into master palette
int	t_col;		// temp color count

int	rd_cnt = 0;	// palette reduction counter

int pbits;
int flag;

uchar keybuf[20];
ulong showadr;
ulong breakpoint;

void display();
void redraw_tiles();
void showinfo();
void showstats();
void test_block();
int test_palette();
void show_pat();
void convert();
void show_block();
void show_bpal();
void print_palette();
void reduce_pal();
void re_palette();


int main(int argc, char *argv[])
{
    /* initializes allegro */
    allegro_init();
    install_keyboard();

    printf("Gridview\n\n");

    if (argc != 3) {
        printf("Usage: GRIDVIEW image.bmp source.asm\n");
        exit (-1);
    }



// read in BMP header
	in_hand  = open(argv[1], O_RDONLY|O_BINARY);
        fil_len = read(in_hand, &header, 11);
	if (fil_len == 0) {
		printf("Invalid input file\n");
		exit (-1);
	}
	close (in_hand);
	in_hand  = open(argv[1], O_RDONLY|O_BINARY);
	tmp = header[10];
        fil_len = read(in_hand, &header, tmp);
// read in BMP file
	Xsiz = header[18] + (256 * header[19]);
	Ysiz = header[22] + (256 * header[23]);
	byt_size = Xsiz * Ysiz/2;

	fil_len = read(in_hand, &indat, byt_size);
	if (fil_len != byt_size) {
		printf("Invalid data size\n");
		exit(-1);
	}
	close (in_hand);

// set colors
	for (cnt=0;cnt<64;cnt+=4) {
		pallete[cnt/4].b = header[54+cnt] >> 2;
		pallete[cnt/4].g = header[55+cnt] >> 2;
		pallete[cnt/4].r = header[56+cnt] >> 2;
	}
		pallete[16].b = 31;
		pallete[16].g = 31;
		pallete[16].r = 31;

	showinfo();
	printf("\nPress a key to display...\n\n");
	readkey();

	display();

	set_gfx_mode(GFX_TEXT, 320, 200, 0, 0);
	Xoff --; Yoff --;
	printf("\n\nX offset= %2d, Y offset= %2d\n", Xoff-(Xoff/9)*9, Yoff-(Yoff/9)*9);
	showstats();
	show_block();
	printf("\n\n");
	show_bpal();
//	if ((pl2_used == 0) && (lptr < 16))
		ofil = fopen(argv[2], "w");
		convert();
		fclose(ofil);
	return 0;

} /* int main () */


void showinfo() {

	printf("Image size = %d, %d\n\n", Xsiz, Ysiz);
	printf("Colors: 0BGR\n");
	for (tmp=0;tmp<4;tmp++) {
		for (cnt=0;cnt<16;cnt+=4)
			printf("%02d:  0%1x%1x%1x  ", tmp*4+cnt/4,
			 header[54+cnt+(tmp*16)] >> 4,
			 header[55+cnt+(tmp*16)] >> 4,
			 header[56+cnt+(tmp*16)] >> 4);
		printf("\n");
	}
	for (cnt=0;cnt<byt_size;cnt++) {
		histo[indat[cnt] & 0x0F] ++;
		histo[indat[cnt] >> 4  ] ++;
	}
	for (cnt=0;cnt<16;cnt++)
		if (histo[cnt] > hmax) {
			hmax = histo[cnt];
			hind = cnt;
		}

	printf("Background color index = %d\n", hind);

	for (cnt=0;cnt<96;cnt++)
		M_pal[cnt] = hind;

	for (cnt=0;cnt<380;cnt++)
		blk_pal[cnt] = 255;

	for (cnt=0;cnt<8;cnt++)
		printf("%02d:%05d ",cnt,histo[cnt]);
	printf("\n");
	for (cnt=8;cnt<16;cnt++)
		printf("%02d:%05d ",cnt,histo[cnt]);
	printf("\n");
}

void display()
{
    /* set VGA graphics mode */
	set_gfx_mode(GFX_AUTODETECT, 320, 240, 0, 0);
	set_pallete(pallete);

   redraw_tiles();
    while (1) {
        if (key[KEY_ESC]) {
            clear_keybuf();
            return;
        } else if (key[KEY_UP]) {
            if (Yoff < 48)
		Yoff+=1;
	    if ((Yoff-(Xoff/9)*9)==0)
		Yoff+=1;
            redraw_tiles();
            clear_keybuf();
        } else if (key[KEY_DOWN]) {
	    if (Yoff > 10)
            	Yoff-=1;
	    if ((Yoff-(Yoff/9)*9)==0)
		Yoff-=1;
            redraw_tiles();
            clear_keybuf();
        } else if (key[KEY_RIGHT]) {
            if (Xoff < 48)
		Xoff+=1;
	    if ((Xoff-(Xoff/9)*9)==0)
		Xoff+=1;
            redraw_tiles();
            clear_keybuf();
        } else if (key[KEY_LEFT]) {
            if (Xoff > 10)
		Xoff-=1;
	    if ((Xoff-(Xoff/9)*9)==0)
		Xoff-=1;
            redraw_tiles();
            clear_keybuf();
	}
    }

} /* display() */


// new routine
// -----------
// put up 8 pixels (4 bytes) starting at Xoff, Yoff
// put a space
// repeat for a line
// do 8 lines
// do a blank line
// repeat until done

void redraw_tiles() {

	for (Ylc=0; Ylc<240; Ylc++)
		for (Xlc=0; Xlc<240; Xlc++)
			putpixel(screen, Xlc, Ylc, 16);

	for (Ylc=0,sY=Yoff; Ylc<Ysiz; Ylc++,sY++) {
		if ((sY-(sY/9)*9) == 0) sY ++;
		for (Xlc=0, sX=Xoff; Xlc<Xsiz; Xlc++,sX++) {
			if ((sX-(sX/9)*9) == 0) sX ++ ;
			if ((Xlc & 1) == 0)
			putpixel(screen, sX, (220-sY),
				indat[(Ylc*Xsiz/2)+Xlc/2] >>   4);
			else
			putpixel(screen, sX, (220-sY),
				indat[(Ylc*Xsiz/2)+Xlc/2] & 0x0F);
		}
	}
}

void showstats() {

int cnt, tmp;

	for (ych=0;ych<Ysiz/8;ych++)
		for (xch=0;xch<Xsiz/8;xch++)
			test_block();

// repack the palettes
// so 0120 and 0123 use the same one
	reduce_pal();
	re_palette();
	print_palette();

	if (pl2_used != 0)
		printf("Second plane used at least %d times\n", pl2_used);
	if (spr_used != 0)
		printf("Also requires at least %d sprites\n", spr_used);
	if (mtc_used != 0)
		printf("Picture has %d background blocks\n", mtc_used);
	if (blc_used !=0)
		printf("Picture has %d one-color blocks\n", blc_used);
 	if (co2_used !=0)
		printf("Picture has %d two-color blocks\n", co2_used);
	if (rd_cnt != 0)
		printf("Palette reduce by %d palettes\n", rd_cnt);
	if (lptr > 48)
		printf("Too many palettes in use\n");
	else
		printf("Requires at least than %d palettes\n", lptr);

	tmp = Xsiz*Ysiz/64 - mtc_used + pl2_used + spr_used +1 -blc_used;

	printf("%d total blocks required\n", tmp);

}


void test_block()
{
int cnt, tmp, rval;

	for (cnt=0;cnt<16;cnt++)
		histo[cnt] = 0;

	for (ypx=0;ypx<8;ypx++)
		for (xpx=0;xpx<4;xpx++) {
			tmp = indat[((ych*8 + ypx)*(Xsiz/2))+(xch*4 + xpx)] >>   4;
			histo[tmp] ++;
			tmp = indat[((ych*8 + ypx)*(Xsiz/2))+(xch*4 + xpx)] & 0x0F;
			histo[tmp] ++;
		}

	tmp = 0;
	for (cnt=0;cnt<16;cnt++)
		if (histo[cnt] != 0) tmp ++;

	blk_col[ych*Xsiz/8+xch] = tmp;
	if (histo[hind] !=0)
		blk_col[ych*Xsiz/8+xch] += 15;

	if (blk_col[ych*Xsiz/8+xch] == 16)
		blk_pal[ych*Xsiz/8+xch] = 0;
	else {
// assumes first one found has 3 colors or less
		if (pal_cnt == 0) {
			test_palette();
			t_col = 0;
			for (cnt=0;cnt<16;cnt++)
				if ((cnt != hind) && (histo[cnt] != 0)) {
					M_pal[t_col] = cnt;
					t_col ++;
				}
			pal_cnt ++;
			blk_pal[ych*Xsiz/8+xch] = 0;
		}
		else {
			if ((blk_col[ych*Xsiz/8+xch] & 15) < 4) {
				rval = test_palette();
				if (flag == 0) {
					t_col = 0;
					for (cnt=0;cnt<16;cnt++)
						if ((cnt != hind) && (histo[cnt] != 0)) {
							M_pal[pal_cnt*3+t_col] = cnt;
							t_col ++;
						}
					blk_pal[ych*Xsiz/8+xch] = pal_cnt;
					pal_cnt ++;
				}
				else
					blk_pal[ych*Xsiz/8+xch] = rval;
			}
// here we need to work on more than 3 color blocks
		}
	}
	if (tmp == 1) {
		if (histo[hind] != 0)
			mtc_used ++;
		else
			blc_used ++;
	}
	if (tmp == 2) {
		if (histo[hind] !=0)
			blc_used ++;
		else
			co2_used ++;
	}
	if ((tmp == 3) && (histo[hind] != 0))
		co2_used ++;

	if ((tmp > 3) && (histo[hind] != 0))
		pl2_used ++;
	else
		if (tmp > 4)
			pl2_used ++;
	if (((tmp > 9) && (histo[hind] != 0)) || (tmp > 10)) {
		spr_used +=2;
		printf ("2 sprites needed near pixel %d, %d\n", xch*8, Ysiz-(ych*8));
//		show_pat();
		}
	else if (((tmp > 6) && (histo[hind] != 0)) || (tmp > 7)) {
		spr_used ++;
		printf ("1 sprite needed near pixel %d, %d\n", xch*8, Ysiz-(ych*8));
//		show_pat();
		}

}

void print_palette() {

int cnt, tmp;

	printf("Palettes:\n");
	for (cnt=0;cnt<pal_cnt;cnt++)
		printf("%2x  %x|%x|%x   %04x\n", cnt,
			M_pal[cnt*3], M_pal[cnt*3+1], M_pal[cnt*3+2],
			lookup[cnt]);
}

int test_palette() {

int cnt, tmp, rval;

	pbits = 0;
	for (cnt=0;cnt<16;cnt++)
		if (histo[cnt] != 0)
			pbits += (1 << cnt);

	if (lptr == 0) {
		lookup[0] = pbits | (1 << hind);
		lptr ++;
		return;
	}

	flag = 0;
	for (tmp=0;tmp<lptr;tmp++)
		if (((pbits & lookup[tmp])^pbits) == 0) {
			if (flag == 0) rval = tmp;
			flag = 1;
		}
	if (flag == 0) {
		lookup[lptr] = pbits | (1 << hind);
		lptr ++;
	}
	else
		return rval;
}

void re_palette() {

int cnt, tmp, rval;

	cnt = 0;
	tmp = 0;
	while (cnt < lptr) {
		if (lookup[tmp] == 99999) {
			tmp ++;
			printf("Z");
			pal_cnt --;
		}
		else {
			lookup[cnt] = lookup[tmp];
			M_pal[cnt*3]   = M_pal[tmp*3];
			M_pal[cnt*3+1] = M_pal[tmp*3+1];
			M_pal[cnt*3+2] = M_pal[tmp*3+2];
			cnt ++; tmp++;
		}
	}
		lptr = pal_cnt;
		printf("\n");
	
	for(xch=0;xch<Xsiz/8;xch++)
		for (ych=0;ych<Ysiz/8;ych++) {

	for (cnt=0;cnt<16;cnt++)
		histo[cnt] = 0;

	for (ypx=0;ypx<8;ypx++)
		for (xpx=0;xpx<4;xpx++) {
			tmp = indat[((ych*8 + ypx)*(Xsiz/2))+(xch*4 + xpx)] >>   4;
			histo[tmp] ++;
			tmp = indat[((ych*8 + ypx)*(Xsiz/2))+(xch*4 + xpx)] & 0x0F;
			histo[tmp] ++;
		}

	tmp = 0;
	for (cnt=0;cnt<16;cnt++)
		if ((histo[cnt] != 0) && (cnt != hind)) tmp ++;

	if (tmp < 4)
		blk_pal[ych*Xsiz/8+xch] = test_palette();
	}
}
void show_pat() {

int cnt, tmp;

	for (ypx=7;ypx>=0;ypx--) {
		for (xpx=0;xpx<4;xpx++) {
			tmp = indat[((ych*8 + ypx)*(Xsiz/2))+(xch*4 + xpx)] >>   4;
			printf("%1x",tmp);
			tmp = indat[((ych*8 + ypx)*(Xsiz/2))+(xch*4 + xpx)] & 0x0F;
			printf("%1x",tmp);
		}
		printf("\n");
	}
}

void show_block(){

	for (ych=(Ysiz/8)-1;ych>=0;ych--) {
		for (xch=0;xch<Xsiz/8;xch++) {
			if (blk_col[ych*Xsiz/8+xch] == 16)
				printf(".. ");
			if (blk_col[ych*Xsiz/8+xch] < 16)
				printf("%2x ",blk_col[ych*Xsiz/8+xch]);
			if (blk_col[ych*Xsiz/8+xch] > 16)
				printf(".%1x ",blk_col[ych*Xsiz/8+xch]-16);
		}
		printf("\n");
	}
}

void show_bpal(){

	for (ych=(Ysiz/8)-1;ych>=0;ych--) {
		for (xch=0;xch<Xsiz/8;xch++)
			if (blk_pal[ych*Xsiz/8+xch] != 255)
				printf("%2x ",blk_pal[ych*Xsiz/8+xch]);
			else
				printf(".. ");
		printf("\n");
	}
}

void reduce_pal() {

int tmp, cnt, flag, val;

	for (cnt=0;cnt<pal_cnt;cnt++)
		if (M_pal[cnt*3+2] == hind) {
			printf ("X");
			flag = 0;
			for (tmp=cnt+1;tmp<lptr;tmp++)
				if (((lookup[cnt] & lookup[tmp])^lookup[cnt]) == 0) {
					if (flag == 0) val = tmp;
					flag = 1;
				}
			if (flag == 1) {
				printf("Y");
//				M_pal[cnt*3] = val;
				lookup[cnt] = 99999;
				rd_cnt ++;
			}
		}

	printf("\n");
}
// write out the .ASM file 
void convert() {

int	cnt, tmp, tile, lnct, colr;

int	ctr, plt, col1, col2, col3;

int	pixc1, pixc2;
unsigned int	val, tvl;

// init pointers
// get a block
// check colors
// print palettes

	fprintf(ofil, "BACKGROUND\n");
	fprintf(ofil, "\tdw\t0%1x%1x%1xh\n",
		pallete[hind].b>>2,
		pallete[hind].g>>2,
		pallete[hind].r>>2);

	fprintf(ofil,"PALETTE\n");
	for (cnt=0;cnt<lptr;cnt++) {
		fprintf(ofil,"\tdw\t0");
		for (tmp=0;tmp<3;tmp++)
			fprintf(ofil,", 0%1x%1x%1xh",
			pallete[M_pal[cnt*3+tmp]].b>>2,
			pallete[M_pal[cnt*3+tmp]].g>>2,
			pallete[M_pal[cnt*3+tmp]].r>>2);
	fprintf(ofil,"\t; %d\n", cnt);
	}
	fprintf(ofil,"ENDPALETTE\n");

	fprintf(ofil,"\nTEXT\n");
	tmp = 0;
	lnct = 0;
	for (ych=(Ysiz/8)-1;ych>=0;ych--) {
		for (xch=0;xch<Xsiz/8;xch++) {
			cnt = ych*Xsiz/8+xch;
// need block reduction here
			if (blk_col[cnt] == 16)
				tile = 0;
			else
				if ((blk_col[cnt] & 15) < 4) {
					tmp ++;
					tile = tmp;
				}
			if ((blk_col[cnt] & 15) > 3) {
				blk_pal[cnt] = 0;
				tile = 0;
			}
			switch (lnct & 7) {
				case 7:
				fprintf(ofil,", 0%04xh\n", (blk_pal[cnt]<<9) | tile);
				break;
				case 0:
				fprintf(ofil,"\tdw\t0%04xh", (blk_pal[cnt]<<9) | tile);
				break;
				default:
				fprintf(ofil,", 0%04xh", (blk_pal[cnt]<<9) | tile); 
				break;
			}
			lnct ++;
		}
		for (xch=0;xch<(256-Xsiz)/8;xch++) {
			switch (lnct & 7) {
				case 7:
				fprintf(ofil,", 00000h\n");
				break;
				case 0:
				fprintf(ofil,"\tdw\t00000h");
				break;
				default:
				fprintf(ofil,", 00000h"); 
				break;
			}
			lnct ++;
		}
		if ((lnct & 7) != 0)
			fprintf(ofil,"\n");
	}
	fprintf(ofil,"ENDTEXT\n");
	printf("%d tiles allocated\n", tmp);

// get a pixel
// see what color palette we are using
// translate color of pixel(0-15) into palette index(0-3)
// move index into correct position within word
// loop until word done
// write word

	fprintf(ofil,"\nCHARSET\n");
	tmp = 0;
	for (ych=(Ysiz/8)-1;ych>=0;ych--)
		for (xch=0;xch<Xsiz/8;xch++) {
			cnt = ych*Xsiz/8+xch;
			if (blk_col[cnt] == 16)
				continue;
			if ((blk_col[cnt] & 15) > 3)
				continue;
			plt = blk_pal[cnt];
			col1 = M_pal[plt*3];
			col2 = M_pal[plt*3+1];
			col3 = M_pal[plt*3+2];
			fprintf(ofil,"\tdw\t");
			for (ypx=7;ypx>=0;ypx--) {
				val = 0;
				for (xpx=0;xpx<4;xpx++) {
					tvl = 0;
					ctr = ((ych*8 + ypx)*(Xsiz/2))+(xch*4 + xpx);
					pixc1 = indat[ctr] >>   4;
					pixc2 = indat[ctr] & 0x0F;
					if (pixc1==col1) tvl  = 4;
					if (pixc1==col2) tvl  = 8;
					if (pixc1==col3) tvl  = 12;
					if (pixc2==col1) tvl |= 1;
					if (pixc2==col2) tvl |= 2;
					if (pixc2==col3) tvl |= 3;
					val |= (tvl << (4*(3-xpx)));
					}
				if (ypx == 7)
					fprintf(ofil,"0%04xh", val);
				else
					fprintf(ofil,", 0%04xh", val);
				}
			fprintf(ofil,"\n");
			tmp ++ ;
		}
	fprintf(ofil,"ENDCHARSET\n");
	printf("%d tiles created\n", tmp);

// create block
//   get data, reduce colors, write data
// loop
// write palettes
// write character map
// write blocks


/* notes

background color is free - all palettes get it - remove from equation
set the first 15 palettes - 1 background color, 1 different color
that fills in all the single color blocks
add second colors to fill in the 2 color blocks
add third colors to fill in the 3 color blocks
  can use all 32 front/back palettes for this
all palettes must use ascending color indexes

*/

/* how about this

get all block info - # colors & which ones (excluding background on both)
sort them first by least number of colors then by which ones
remove duplicates and count them up - if it exceeds 96 then fail

	sort
	first_val = val[0]
	first_#col = #col[0]
	pal_counter = 1
	for count = 1 to x*y
		if val[count] != first_val or #col[count] != first_#col 
			first_val = val[count]
			first_#col = #col[count]
			pal_counter ++

not right - doesn't fold colors together e.g.
(0,1,2,3) = (0,1,-,-) = (0,2,-,-) = (0,3,-,-) = (0,1,2,-) = (0,1,3,-)
            (-,1,-,-) = (-,2,-,-) = (-,3,-,-) = (-,1,2,-) = (-,1,3,-)

2 planes, 11 colors - all permutations

(09a6)(0a67)(0678)(0a78)(0a68)(0967)(0968)(09a7)(09a8)(0978)

(0123)(0124)(0134)(0234)(0125)(0135)(0145)(0235)(0245)(0345)

good = (((palcol[] & mycol) ^ mycol) == 0) 

*/
}

/* ***
   EOF
   *** */