#include <stdio.h>

// Code by Ivan Mackintosh (& a small part by THOR)

#include <stdio.h>
#include <string.h>

typedef unsigned char BYTE;
typedef int BOOL;
enum { FALSE, TRUE};

int FindBytes(FILE * fp, const BYTE String[], int Len, int Start, int End);
void ConvertAddress2String(int Addr, BYTE * OutBuffe);
int ReadAddress(FILE * fp, int Len);

const BYTE DriverSig[] = { 0xF3, 0x31, 0xc0, 0x00 };

const int MAX_GROUP_COUNT = 1000;

unsigned char res[65536*2];
int count=0;
int Neiro_Tbl;
int Host_Tbl;
int Pattern_Tbl;
int GroupArrayCount;

int getMusicFromROM (char *romName)
{
   FILE * fp;
   int i, Len;
   int NeiroStart;
   int NeiroCount;
   int HostStart;
   int HostCount;
   int PatternStart;
   int GroupList1Start;
   BYTE Buffer[10];
   int c, n, g, t;
   int GroupArray[MAX_GROUP_COUNT];
   int BGMDataArray[MAX_GROUP_COUNT];
   int BGMGroupArray[MAX_GROUP_COUNT];
   int BGMOffset[MAX_GROUP_COUNT];
   int BGMDataCount = 0;
   
   fp = fopen(romName, "rb");
   if (fp == NULL)
   {
      printf("ROM not found\n");
      return 0;
   }

   // search for sound driver
   fseek(fp, 0, SEEK_END);
   Len = ftell(fp);
   fseek(fp, 0, SEEK_SET);

   i = FindBytes(fp, DriverSig, sizeof(DriverSig), 0, Len);
   if (i == -1)
   {
      printf("Couldn't find SNK music driver signature\n");
      fclose(fp);
      return 0;
   }

   fseek(fp, i+0x8f, SEEK_SET);
   c = ReadAddress(fp, 2);
   if (c != 0xab5)
   {
      printf("SNK driver size not supported - 0x%04x\n", c);
      fclose(fp);
      return 0;
   }
   
   // skip the driver code
   printf("%s has SNK music driver located at offset %08x\n", romName, i);

   NeiroStart = i + 0xab5;
   printf("Neiro table @ %08x\n", NeiroStart);

   fseek(fp, NeiroStart, SEEK_SET);

   Buffer[0] = (NeiroStart + 0x200000) / 65536;
   Buffer[1] = 0;

   i = FindBytes(fp, Buffer, 2, NeiroStart, Len);
   if (i == -1)
   {
      printf("Couldn't find a group list table\n");
      fclose(fp);
      return 0;
   }

   PatternStart = i - 2;
   HostStart = PatternStart - 2;
   n = 0;
   do
   {
      fseek(fp, HostStart, SEEK_SET);
      c = ReadAddress(fp, 2);
      if (n != c)
      {
         n += 2;
         HostStart-=2;
      }
   } while (n != c);
   
   NeiroCount = (HostStart - NeiroStart) / 9;
   if ((HostStart - NeiroStart) % 9 != 0)
   {
      printf("error: a problem occurred decoding the host table\n");
      fclose(fp);
      return 0;
   }
   
   HostStart = NeiroStart + (NeiroCount * 9) ;
   HostCount = (PatternStart - HostStart) / 2;

   fseek(fp, PatternStart, SEEK_SET);
   GroupList1Start = ReadAddress(fp, 4) - 0x200000;
   
   printf("Neiro count = %d\n", NeiroCount);
   printf("Host Table @ %08x\n", HostStart);
   printf("Pattern Table @ %08x\n", PatternStart);
   printf("Group1 @ %08x\n", GroupList1Start);

   // now dump out in correct format
   fseek(fp, NeiroStart, SEEK_SET);
   for (c = 0; c < NeiroCount; c ++)
   {
      for (n = 0; n < 9; n++)
      {
		 res[count++] = fgetc(fp);
      }
   }

   for (c = 0; c < HostCount; c++)
   {
	  res[count++] = fgetc(fp);
	  res[count++] = fgetc(fp);
   }
   
   if (ftell(fp) != PatternStart)
   {
      printf("error: a problem occurred decoding the host table\n");
      fclose(fp);
      return 0;
   }

   GroupArrayCount = 0;
   n = 1;
   do
   {
      c = ReadAddress(fp, 4);
      if (c != 0)
      {
		 int addr = c-0x200000-NeiroStart;
         res[count++] = (addr&0xff);
		 res[count++] = ((addr>>8)&0xff);
		 res[count++] = ((addr>>16)&0xff);
		 res[count++] = ((addr>>24)&0xff);
         GroupArray[GroupArrayCount++] = c - 0x200000;
         if (GroupArrayCount == MAX_GROUP_COUNT)
         {
            printf("\nerror: too many groups to process\n");
            fclose(fp);
            return 0;
         }
      }
      else
	  {
		  res[count++] = 0;
		  res[count++] = 0;
		  res[count++] = 0;
		  res[count++] = 0;
	  }
      n++;
   } while (c != 0);

   // decode each group
   for (g = 0; g < GroupArrayCount; g++)
   {
      fseek(fp, GroupArray[g], SEEK_SET);
	  res[count++] = fgetc(fp);
	  res[count++] = fgetc(fp);
	  res[count++] = fgetc(fp);
	  res[count++] = fgetc(fp);
	  res[count++] = fgetc(fp);
	  res[count++] = fgetc(fp);
	  c = fgetc(fp);
	  res[count++] = c;
      for (i = 0; i < c; i ++)
      {
		  res[count++] = fgetc(fp);
      }

	  // fill BGM array
	  do
	  {
		 c = ReadAddress(fp, 4);
		 if (c != 0)
		 {
			// see if song already exists
			BOOL BGMFound = FALSE;
			
			for (i = 0; i < BGMDataCount; i ++)
			{
			   if (BGMDataArray[i] == c) // found
			   {
				  BGMFound = TRUE;
				  break;
			   }
			}
			int cc = c - 0x200000 - NeiroStart;
			if (!BGMFound)
			{
			   BGMGroupArray[BGMDataCount] = g;
			   BGMDataArray[BGMDataCount] = c;
			   BGMOffset[BGMDataCount++] = cc;
			   i = BGMDataCount-1;
			}
		    res[count++] = (cc&0xff);
		    res[count++] = ((cc>>8)&0xff);
		    res[count++] = ((cc>>16)&0xff);
		    res[count++] = ((cc>>24)&0xff);
			n = fgetc(fp);
		    res[count++] = n;
			for (i = 0; i < n; i ++)
			{
			    res[count++] = fgetc(fp);
			}
		 }
	  } while (c != 0);
      res[count++] = 0;
      res[count++] = 0;
      res[count++] = 0;
      res[count++] = 0;
   }

   // header complete - dump out all BGMS
   for (i = 0; i < BGMDataCount; i ++)
   {
	  int j;
	  for (j = 0; j < BGMDataCount; j ++)
	  {
		  if (BGMOffset[j]==count)
			  break;
	  }
	  fseek(fp, BGMDataArray[j] - 0x200000, SEEK_SET);
      res[count++] = fgetc(fp);
      res[count++] = fgetc(fp);
      res[count++] = fgetc(fp);
	  int z = fgetc(fp);
      res[count++] = z;
	  int zz = fgetc(fp);
      res[count++] = zz;
      n = z+(zz<<8);
	  n &= 0xfff; // remove high nibble to leave BGM length
	  n -=2;
	  for (t = 0; t < n; t++)
	  {
	     res[count++] = fgetc(fp);
	  }
   }

   res[count++] = 0;
   int offset = 0;
   Neiro_Tbl=offset;
   offset += NeiroCount*9;
   Host_Tbl=offset;
   offset+=HostCount*2;
   Pattern_Tbl=offset;
   
   fclose(fp);

   return 1;
}

int FindBytes(FILE * fp, const BYTE String[], int Len, int Start, int End)
{
   int FoundCount = 0;
   int i, ch;

   fseek(fp, Start, SEEK_SET);
   
   for (i = Start; i < End; i ++)
   {
      ch = fgetc(fp);
      if (ch == String[FoundCount])
      {
         FoundCount++;
         if (FoundCount == Len)
            return i - Len + 1;
      }
      else
      {
         FoundCount = 0;
      }
   }

   return -1;
}

void ConvertAddress2String(int Addr, BYTE * OutBuffer)
{
   int i;
   int FullAddr = Addr + 0x200000;
   
   for (i = 0; i < 4; i ++)
   {
      OutBuffer[i] = FullAddr % 256;
      FullAddr >>= 8;
   }
}

int ReadAddress(FILE * fp, int Len)
{
   int retval = 0;
   int i;

   for (i = 0; i < Len; i ++)
   {
      retval += (fgetc(fp) << (i * 8));
   }

   return retval;
}

#define SET16p(r,v) { writeMem(r++,v&0xff); writeMem(r++,v>>8); }
#define GET16(r) music->data[r]+(music->data[r+1]<<8)
#define GET32(r) music->data[r]+(music->data[r+1]<<8)+(music->data[r+2]<<16)+(music->data[r+3]<<24)

typedef struct
{
	int HOST_TBL;
	int PATTERN_TBL;
	int NEIRO_TBL;
	unsigned char *data;
} MUSIC;

unsigned char ram[4096];

int driverSize()
{
	return 0xab5;
}

void writeMem(int addr,int value)
{
	ram[addr] = value;
}

int sendGroup(MUSIC *music,int xwa)
{
	int Z80_GROUP_INFO = driverSize();
	int Z80_HOST_TABLE_START = Z80_GROUP_INFO+6;
	int Z80_NEIRO_START;
	int Z80_DATA_START;

	int COPY_DEST_PTR;
	int BGM_NEIRO_CNT;
	int BGM_NEIRO_PTR;
	int BGM_HEADER_LEN;
	int BGM_DATA_LEN;
	int BGM_DATA_START;

    // copy host table if one exists
    int xhl = music->HOST_TBL,xde,xhl2;
    int bc = GET16(xhl); // get number of bytes to copy
	if (bc>0) // if none skip copy
	{
        xhl+=2; // move source copy to first byte
        xde = Z80_HOST_TABLE_START;
		for (;bc>0;bc--)
            writeMem(xde++, music->data[xhl++]);
	}
    // a = input group parameter
    xwa &= 0xff; // clear upper 24 bits of xwa to just leave group number
    xwa<<=2;     // multiply by 4 to get groups offset (each address is 4 bytes long)

    xhl = music->PATTERN_TBL; // get group offset
    xhl += xwa;
    xhl = GET32(xhl); // xhl now points to selected group

    // all groups start with 3 x 16bit offsets - store first 2 for later use
    int de = GET16(xhl); // start of neiro table (z80 offset)
	xhl+=2;
    int wa = driverSize(); // get sound driver size
	de -= 0x0ab5;
	de += wa;
	int xbc = Z80_GROUP_INFO;
	SET16p(xbc,de);
    Z80_NEIRO_START = de;

    de = GET16(xhl); // start of the actual music/sfx data (z80 offset)
	xhl+=2;
	de -= 0x0ab5;
	de += wa;
	SET16p(xbc,de);
    Z80_DATA_START = de;

    de = GET16(xhl);
	xhl+=2;
	de -= 0x0ab5;
	de += wa;
	SET16p(xbc,de);

    // copy all neiros used by this group
    int a = music->data[xhl++]; // a = number of neiros to copy
    if (a==0)
		return 0; // if there are no neiros then there are also no tunes

    xde = Z80_NEIRO_START;
    COPY_DEST_PTR = xde;
	//SG_NeiroCopy
	for (;a>0;a--)
	{
        xde = music->data[xhl]; // get neiro number
        xde *=9; //x9 to get offset in neiro table
        xhl++;
        int xhl2 = music->NEIRO_TBL; // calculate source
        xhl2 += xde;
		xde = COPY_DEST_PTR;
        for (bc=9;bc>0;bc--) // copy count = 9 bytes per neiro
            writeMem(xde++,music->data[xhl2++]); // copy one neiro
        COPY_DEST_PTR = xde;
	} //copy next neiro
        
    // copy music data
	while (true) //SG_Copy_Next:
	{
        xde = GET32(xhl);       // get address of tune information
        if (xde==0)             // is this the last one
           break;
        xhl+=4;                 // skip to number of neiros for this tune
        wa = music->data[xhl];
        BGM_NEIRO_CNT = wa;     // store number of neiros for this BGM
        xhl++;                  // skip to neiros
        BGM_NEIRO_PTR = xhl;    // store pointer to neiro index list
        xhl = xde;              // now points to actual music data
        xde = GET16(xhl);       // get tune number

        xhl+=2;                 // read header length
        de = music->data[xhl];
        BGM_HEADER_LEN = de;
        
        xhl++;                  // read music data length
        bc = GET16(xhl);
        bc &= 0x3fff;           // scrap any control bits
        bc -= de;
        bc -= BGM_NEIRO_CNT;
        BGM_DATA_LEN = bc;

        bc = BGM_HEADER_LEN;    // copy the header
        xde = COPY_DEST_PTR;
		for(;bc>0;bc--)
			writeMem(xde++,music->data[xhl++]);
        COPY_DEST_PTR = xde;

        xwa = BGM_NEIRO_CNT;     // skip to the actual data
        xhl += xwa;
        BGM_DATA_START = xhl;

        xhl = BGM_NEIRO_PTR;     // copy neiro indexes
        bc = BGM_NEIRO_CNT;
        xde = COPY_DEST_PTR;
		for(;bc>0;bc--)
			writeMem(xde++,music->data[xhl++]);
        COPY_DEST_PTR = xde;

        // push xhl                 // points to next entry in group
        xhl2 = BGM_DATA_START;      // copy the BGM data
        bc = BGM_DATA_LEN;
        xde = COPY_DEST_PTR;
		for(;bc>0;bc--)
			writeMem(xde++,music->data[xhl2++]);
        COPY_DEST_PTR = xde;
	}
	return COPY_DEST_PTR;
}


int main(int argc,char**argv)
{
	MUSIC m;

	if (argc<4)
	{
		printf("usage:\nmusicdump <romfile> <outputpefix> <b|h>\n");
		return 1;
	}

	if (!getMusicFromROM(argv[1]))
		return 1;
	m.HOST_TBL = Host_Tbl;
	m.PATTERN_TBL = Pattern_Tbl;
	m.NEIRO_TBL = Neiro_Tbl;
	m.data = res;
	int ds= driverSize();
	char buf[64];
	for (int i=0;i<GroupArrayCount;i++)
	{
		int size = sendGroup(&m,i);
		if (argv[3][0]=='b')
		{
			sprintf(buf,"%s%d.bin",argv[2],i);
			FILE *f = fopen(buf,"wb");
			fwrite(ram+ds,size-ds,1,f);
			fclose(f);
		}
		else
		{
			sprintf(buf,"%s%d.h",argv[2],i);
			FILE *f = fopen(buf,"w");
			sprintf(buf,"%s%d",argv[2],i);
			fprintf(f,"const u8 %s[] = {\n",buf);
			for (int j=ds;j<size;j++)
			{
				fprintf(f,"0x%02x%s",ram[j],(j+1<size?",":"};"));
				if (((j-ds+1)%16)==0)
					fprintf(f,"\n");
			}
			fprintf(f,"\n");
			fclose(f);
		}
	}
	return 0;
}
