//////////////////////////////////////////////////////////////////////////////
// ngpc.c                                                                   //
//////////////////////////////////////////////////////////////////////////////

#define POKELINK "PokeLink v2.50"

// v2.50
//      Changed from WinIO to DLPortIO
// v2.00
//      Flavor modified it to work with PocketSend
//      Added WinIo support
// v1.01j2
//      Jeff modified to work on linux. You must be root
//      to run the linux version.
//      To compile on linux use: cc -o ngpc ngpc.c -O2 -Wl,-s
// v1.01j1
//      Jeff modified to work on win95/98.
//      Compiled with DJGPP.
//      Now displays cart Manufacturer & Device ID's.
//
// v1.01
//		Added size detection.
//		Little programming fix.
//		Added check for valid header in file.
//		Added Windows icon.
//
// v1.00
//		More command line options (/y /b /v)
//		Implemented some algorithm to make it faster.
//
// v0.02
//		Able to flash a file specified on command line (/w)
//		Added GBXPORT environment setting.
//
// v0.01
//		Shows license text of cartridge. (/i)
//
//
// This is my tool to flash NGPC carts with the
// Pocket Linker device. That's why I call it "PokeLink".
// Only EPP mode is supported right now because SPP sucks.
// Tested with: 32MBit Pocket Flash Card.
//
// Please credit me if you use anything from this source. Tnx.
//
// For bug reports or suggestions, just e-mail me... at um..
//	ngpcdev@egroups.com ;-)
//
// Dark Fader / BlackThunder
//


//  Game Name             Cart Name   Manufacturer ID  Device ID
// 32M Pocket Flash Card   ------          0x98          0x2f
// Pacman                 PACMAN           0x98          0xab
// Samurai Showdown 2     SAMURAI2v000     0x98          0x2f


//http://www.cubic.org/source/archive/hardware/parallel/parallel.html#5

//////////////////////////////////////////////////////////////////////////////
// includes                                                                 //
//////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "PocketSend.h"
#include "PocketSendDlg.h"
#include "GradientProgressCtrl.h"

#include "Pokelink.h"

//#include "phys32.h"
#include "DLPortIO.h"

void DeInitGBX();

//////////////////////////////////////////////////////////////////////////////
// defines                                                                  //
//////////////////////////////////////////////////////////////////////////////
// EPP I/O registers
#define EPP_STAT	(data_port+1)
#define EPP_CTRL	(data_port+2)
#define EPP_AI		(data_port+3)
#define EPP_DATA	(data_port+4)

// EPP_CTRL register
#define CTRL_READ	0
#define CTRL_WRITE	1
#define CTRL_RESET	4

// EPP_AI register
#define AI_ADDR0	0	// A0-A7
#define AI_ADDR1	1	// A8-A15
#define AI_GBXCTRL	2	// A16-A20, auto-increment
#define AI_DATA		3	// D0-D7, A21-A23 (via latch)

// flash status bits
#define DATA_POLLING		7
#define TOGGLE_BIT			6	// toggles when busy
#define TIMEOUT				5	// 1 indicated timout error
#define SECTOR_ERASE_TIMER	3	// 0 when accepting sector erase command

// flash unlock addresses with chip select
#define fx0002AA    ((addr) & 0x200000 | 0x0002AA)
#define fx000555    ((addr) & 0x200000 | 0x000555)
#define fx000AAA    ((addr) & 0x200000 | 0x000AAA)
#define fx002AAA    ((addr) & 0x200000 | 0x002AAA)
#define fx005555	((addr) & 0x200000 | 0x005555)

// options
#define OPTION_INFO		1
#define OPTION_BACKUP	2
#define OPTION_WRITE	4
#define OPTION_VERIFY	8
#define OPTION_YES		16

CGradientProgressCtrl *pProgressBar = NULL;

//WinIO
//SetPortVal(WORD wPortAddr, DWORD dwPortVal, BYTE bSize);
bool outportb(WORD addr, BYTE val)  //with WinIO, it's (DWORD val)
{
	/* WinIO stuff
	return SetPortVal(addr, val, 1);*/

	DlPortWritePortUchar(addr, val);
	return true;
}

//bool _stdcall GetPortVal(WORD wPortAddr, PDWORD pdwPortVal, BYTE bSize);
BYTE inportb(WORD addr)
{
	/*  WinIO stuff
	DWORD retVal;
	GetPortVal(addr, &retVal, 1);
	return (BYTE)(retVal & 0xFF);*/

	return DlPortReadPortUchar(addr);
}

#define EPP_PORT_TYPE 0
#define SPP_PORT_TYPE 1
#define BAD_PORT_TYPE -1

unsigned port_8;
unsigned port_9;
unsigned port_a;
unsigned port_b;
unsigned port_c;      
unsigned data_port = 0x378;
char 	 port_type=BAD_PORT_TYPE;       // 0=epp, 1=spp


#define ai              port_b
#define data            port_c

#define set_ai_write    outportb(port_a,5)	// 5 ninit=1, nwrite=0
#define set_data_write  outportb(port_a,1)	// 1 ninit=0, nwrite=0
#define set_data_read   outportb(port_a,0)	// 0 ninit=0, nwrite=1
#define set_normal      outportb(port_a,4)	// 4 ninit=1, nwrite=1

/* WinIO Stuff
extern "C++"
{
	bool _stdcall InitializeWinIo();
	void _stdcall ShutdownWinIo();
	PBYTE _stdcall MapPhysToLin(PBYTE pbPhysAddr, DWORD dwPhysSize, HANDLE *pPhysicalMemoryHandle);
}
extern "C"	bool _stdcall GetPhysLong(PBYTE pbPhysAddr, PDWORD pdwPhysVal);
*/




void spp_set_ai(unsigned char _ai)
{
   set_data_write;
   outportb(port_8,_ai);        // put ai at data bus
   outportb(port_a,9);          // nastb=0,nib_sel=0,ndstb=1,nwrite=0
   outportb(port_a,1);          // nastb=1,nib_sel=0,ndstb=1,nwrite=0
                                // nastb ~~~~|___|~~~~
}

void epp_set_ai(unsigned char _ai)
{
   outportb(EPP_AI, _ai);
}

void spp_write_data(unsigned char _data)
{
   outportb(port_8,_data);      // put data at data bus
   outportb(port_a,3);          // nastb=1,nib_sel=0,ndstb=0,nwrite=0
   outportb(port_a,1);          // nastb=1,nib_sel=0,ndstb=1,nwrite=0
                                // ndstb ~~~~|___|~~~~
}

void epp_write_data(unsigned char _data)
{
   outportb(EPP_DATA, _data);
}

void spp_set_ai_data(unsigned char _ai,unsigned char _data)
{
   spp_set_ai(_ai);
   spp_write_data(_data);
}

void epp_set_ai_data(unsigned char _ai,unsigned char _data)
{
   outportb(EPP_AI, _ai);
   outportb(EPP_DATA, _data);
}

void set_ai(unsigned char _ai)
{
   if (port_type == SPP_PORT_TYPE)
        spp_set_ai(_ai);
   else
        epp_set_ai(_ai);
}

void set_ai_data(unsigned char _ai,unsigned char _data)
{
   if (port_type == SPP_PORT_TYPE)
        spp_set_ai_data(_ai,_data);     // spp mode
   else
        epp_set_ai_data(_ai,_data);     // epp mode
}

char spp_read_data(void)
{
   char low_nibble,high_nibble,temp;
   
   //set_data_read;  //do this before calling
   outportb(port_a,0x2);//2           // nastb=1,nib_sel=0,ndstb=0,nwrite=1
   low_nibble=inportb(port_9);
   outportb(port_a,6);           // nastb=1,nib_sel=1,ndstb=0,nwrite=1
   high_nibble=inportb(port_9);
   outportb(port_a,0);           
                                 // nibble_sel ___|~~~ and ndstb ~~~~|___|~~~~
   temp=(((high_nibble<<1)&0xf0)|((low_nibble>>3)&0x0f));
   return(temp);
}

unsigned char spp_read_low_nibble(void)
{
   unsigned char low_nibble;
   unsigned char temp;
   
   outportb(port_a,2);//2           // nastb=1,nib_sel=0,ndstb=0,nwrite=1
   low_nibble=inportb(port_9);
   outportb(port_a,0);           
                                 // nibble_sel ___|~~~ and ndstb ~~~~|___|~~~~
   temp=((low_nibble>>3)&0x0f);
   return(temp);
}

unsigned char spp_read_high_nibble(void)
{
   unsigned char high_nibble,temp;
   
   outportb(port_a,6);           // nastb=1,nib_sel=1,ndstb=0,nwrite=1
   high_nibble=inportb(port_9);
   outportb(port_a,0);           
                                 // nibble_sel ___|~~~ and ndstb ~~~~|___|~~~~
   temp=((high_nibble<<1)&0xf0);
   return(temp);
}

//////////////////////////////////////////////////////////////////////////////
// SetAddress                                                               //
//////////////////////////////////////////////////////////////////////////////
void SetAddress(unsigned long addr, int inc)
{
	if(port_type == EPP_PORT_TYPE)
	{
		outportb(EPP_CTRL, CTRL_WRITE);        //_outp

		outportb(EPP_AI, AI_ADDR0);
		outportb(EPP_DATA, (addr>>0) & 0xFF);  // A0-A7

		outportb(EPP_AI, AI_ADDR1);
		outportb(EPP_DATA, (addr>>8) & 0xFF);  // A8-A15

		outportb(EPP_AI, AI_GBXCTRL);
		outportb(EPP_DATA, 0x02);              // latch chip select

		outportb(EPP_AI, AI_DATA);
		outportb(EPP_DATA, ~(1<<(addr>>21)));  // A21-A23

		outportb(EPP_AI, AI_GBXCTRL);
		outportb(EPP_DATA,                     // A16-A20, auto-increment
			(((addr>>16) & 0x0F) << 2) |
			(((addr>>20) & 0x01) << 7) |
			(inc ? 0x01 : 0x00)
		);

		outportb(EPP_AI, AI_DATA);
	}
	else if(port_type == SPP_PORT_TYPE)
	{
		set_data_write;

		spp_set_ai(AI_ADDR0);
		spp_write_data((addr>>0) & 0xFF);  // A0-A7

		spp_set_ai(AI_ADDR1);
		spp_write_data((addr>>8) & 0xFF);  // A8-A15

		spp_set_ai(AI_GBXCTRL);
		spp_write_data(0x02);              // latch chip select

		spp_set_ai(AI_DATA);
		spp_write_data(~(1<<(addr>>21)));  // A21-A23

		spp_set_ai(AI_GBXCTRL);
		spp_write_data(                    // A16-A20, auto-increment
			(((addr>>16) & 0x0F) << 2) |
			(((addr>>20) & 0x01) << 7) |
			(inc ? 0x01 : 0x00)
		);

		spp_set_ai(AI_DATA);
	}
}



BYTE check_port(void)
{  
	if (port_type == SPP_PORT_TYPE)
	{ 
		set_ai_data(1,0x12);    //these don't work for EPP
		set_ai_data(0,0x34);
		set_ai(1);
		set_data_read;			// ninit=0, nwrite=1    
		
		if (spp_read_data() != 0x12) 
			return(1); 
		
		set_ai(0);
		set_data_read;			// ninit=0, nwrite=1
		
		if (spp_read_data() != 0x34) 
			return(1); 
	}
	else	if(port_type == EPP_PORT_TYPE)
	{
		//AfxMessageBox("init EPP_PORT_TYPE");
		// Init/test GBX chip
		outportb(EPP_STAT, 1);     // EPP mode
		SetAddress(0x123456, 0);
		outportb(EPP_CTRL, CTRL_READ);
		outportb(EPP_AI, AI_ADDR0);
		if (inportb(EPP_DATA) != 0x56)
			return 1;  //error
		
		outportb(EPP_AI, AI_ADDR1); 
		if (inportb(EPP_DATA) != 0x34) 
			return 1;  //error
		
		outportb(EPP_AI, AI_GBXCTRL); 
		if (inportb(EPP_DATA) != 0x7E && inportb(EPP_DATA) != 0x7F) 
			return 1;  //error
	}
	else
		return 1;
	

   return(0);
}   

int init_port(int port = 1)
{                                                                      
	unsigned long tempVal;
	/*if (GetPhysLong((unsigned char *)0x00400+(6 + (2 * port)), &tempVal))
		data_port = tempVal & 0xFFFF;
	else*
		data_port = 0x378;*/

	// From Fuz
	//char* pByte = reinterpret_cast<char*>((unsigned char *)0x00400+(6 + (2 * port))); // C++ 
	//char* pByte = (char*)(0x12345678); // C 
    
	//data_port = *pByte;

	port_8 = data_port;
	port_9 = data_port + 1;
	port_a = data_port + 2;
	port_b = data_port + 3;
	port_c = data_port + 4;
   	outportb(port_9,1);		// clear EPP time flag
   	set_ai_data(2,0);		// rst=0, wei=0(dis.), rdi=0(dis.)    
   	set_ai_data(2,0x80);	// rst=1, wei=0(dis.), rdi=0(dis.)  

   	port_type=EPP_PORT_TYPE;// try EPP first    0=epp, 1=spp
   	if (check_port()==1)	//EPP didn't work
	{
   		port_type=SPP_PORT_TYPE;  // now try SPP 0=epp, 1=spp
   		if (check_port()==1)//SPP didn't work, either
		{
			//CString tempStr;
			//tempStr.Format("No Pocket Linker present at LPT%d!", port);
			//AfxMessageBox(tempStr); 
			return(1); 
   		}
   	}
    //AfxMessageBox("GB-Xchanger found");

   	outportb(port_9,1);		// clear EPP time flag
   	set_ai_data(2,0);		// rst=0, wei=0(dis.), rdi=0(dis.)    
   	set_ai_data(2,0x80);	// rst=1, wei=0(dis.), rdi=0(dis.)  

	return 0;
}




//////////////////////////////////////////////////////////////////////////////
// variables                                                                //
//////////////////////////////////////////////////////////////////////////////
//unsigned int port = 0x378;
char *filename = 0;
int options = 0;
int err = 0;

//////////////////////////////////////////////////////////////////////////////
// InitGBX                                                                  //
//////////////////////////////////////////////////////////////////////////////
int InitGBX()
{
	//AfxMessageBox("In InitGBX");

	if(init_port(1) != 0)//try LPT1
		if(init_port(2) != 0)//try LPT2
			if(init_port(3) != 0)//try LPT3
				return 1;


	/*if(port_type == EPP_PORT_TYPE)
		AfxMessageBox("Using EPP");
	else if(port_type == SPP_PORT_TYPE)
		AfxMessageBox("Using SPP");
	else
		AfxMessageBox("Not EPP or SPP");*/

	return 0;
}

//////////////////////////////////////////////////////////////////////////////
// DeInitGBX                                                                //
//////////////////////////////////////////////////////////////////////////////
void DeInitGBX()
{
   if(port_type == SPP_PORT_TYPE)
	   return;

   outportb(EPP_CTRL, CTRL_WRITE);
   outportb(EPP_AI, AI_GBXCTRL);
   outportb(EPP_DATA, 0);
   // reset EPP port
   outportb(EPP_CTRL, CTRL_RESET);
}

//////////////////////////////////////////////////////////////////////////////
// WriteByte                                                                //
//////////////////////////////////////////////////////////////////////////////
void WriteByte(unsigned char data)
{
   //outportb(EPP_CTRL, CTRL_WRITE);
   //outportb(EPP_DATA, data);

   set_data_write;
   if (port_type == SPP_PORT_TYPE)
      spp_write_data((BYTE)data);// spp write data
   else
      outportb(EPP_DATA,data);    // epp write data}
}

//////////////////////////////////////////////////////////////////////////////
// ReadByte                                                                 //
//////////////////////////////////////////////////////////////////////////////
unsigned char ReadByte()
{

    if (port_type == SPP_PORT_TYPE)
	{
   		set_ai(3);				// default write mode
   		set_data_read;			// ninit=0, nWrite=1
        return(spp_read_data());        // spp read data
    }
    else 
	{
		outportb(EPP_CTRL, CTRL_READ);
        return inportb(EPP_DATA);           // epp read data
    }
}

//////////////////////////////////////////////////////////////////////////////
// SectorErase                                                              //
//////////////////////////////////////////////////////////////////////////////
int SectorErase(unsigned long addr)
{
	// 0x10000 per block
	// last 0x10000 is divided into 4 blocks
	// with sizes of: 0x8000, 0x2000, 0x2000, 0x4000
	while (ReadByte() & SECTOR_ERASE_TIMER);
	SetAddress(fx005555,0); WriteByte(0xAA);
	SetAddress(fx002AAA,0); WriteByte(0x55);
	SetAddress(fx005555,0); WriteByte(0x80);
	SetAddress(fx005555,0); WriteByte(0xAA);
	SetAddress(fx002AAA,0); WriteByte(0x55);
	SetAddress(addr,0);
	//while (ReadByte() & SECTOR_ERASE_TIMER);
	WriteByte(0x30);
	return 0;
}

int inBypass = 0;

void UnlockBypass(unsigned long addr)
{
	SetAddress(fx005555,0); 
	WriteByte(0xAA);
	SetAddress(fx002AAA,0); 
	WriteByte(0x55);
	SetAddress(fx005555,0); 
	WriteByte(0x20);
	inBypass = 1;
}

void ResetBypass()
{
	SetAddress(0,0); WriteByte(0x90);
	SetAddress(0,0); WriteByte(0x00);
	inBypass = 0;
}


//////////////////////////////////////////////////////////////////////////////
// Device ID                                                                //
//////////////////////////////////////////////////////////////////////////////
unsigned char ManufID()
{
	unsigned char man;
	unsigned long addr=0;
	UnlockBypass(addr);

	SetAddress(fx005555,0); WriteByte(0xAA);
	SetAddress(fx002AAA,0); WriteByte(0x55);
	SetAddress(fx005555,0); WriteByte(0x90);
    SetAddress(0x000000,0);   // manufacturer ID (0x98)
	man = ReadByte();

	ResetBypass();
	SetAddress(0,0); WriteByte(0xF0); //reset

	return man;
}


//////////////////////////////////////////////////////////////////////////////
// Device ID                                                                //
//////////////////////////////////////////////////////////////////////////////
unsigned char DeviceID(int chipNum)
{
	unsigned char dev;
	unsigned long addr;

	if(chipNum==1)
		addr = 0x200000;  //want the ID of the 2nd chip
	else 
		addr = 0x000000;

	UnlockBypass(addr);
	SetAddress(fx005555,0); WriteByte(0xAA);
	SetAddress(fx002AAA,0); WriteByte(0x55);
	SetAddress(fx005555,0); WriteByte(0x90);
    SetAddress(0x000001,0); // ID (0x2F)
	dev = ReadByte();

	ResetBypass();
	SetAddress(0,0); WriteByte(0xF0); //reset

	return dev;
}

//////////////////////////////////////////////////////////////////////////////
// Sector Protection Verify                                                                //
//////////////////////////////////////////////////////////////////////////////
unsigned char SectorProtVerify(unsigned long sectorAddr)
{
	unsigned char prot;

	unsigned long addr=0;
	UnlockBypass(addr);

	SetAddress(fx005555,0); WriteByte(0xAA);
	SetAddress(fx002AAA,0); WriteByte(0x55);
	SetAddress(fx005555,0); WriteByte(0x90);
    SetAddress(((sectorAddr & 0xFFFFFFFC) | 0x2),0);
	prot = ReadByte();

	ResetBypass();
	SetAddress(0,0); WriteByte(0xF0); //reset

	return prot;
}

unsigned char CheckCFI()
{
	unsigned long addr=0;
	SetAddress(fx000555,0); WriteByte(0x98);
    SetAddress(0x10, 0);
	if(ReadByte() != 0x51)
		return 0;
    SetAddress(0x11, 0);
	if(ReadByte() != 0x52)
		return 0;
    SetAddress(0x12, 0);
	if(ReadByte() != 0x59)
		return 0;

	return 1;
}


//////////////////////////////////////////////////////////////////////////////
// Program                                                                  //
//////////////////////////////////////////////////////////////////////////////
int Program(unsigned long addr, unsigned char data, int reties)
{
    unsigned long to=10000;

    // program byte
	if(inBypass)
	{
		SetAddress(fx005555,0); WriteByte(0xA0);
		SetAddress(addr,0);	WriteByte(data);
		return 0;
	}
	else
	{
		SetAddress(fx005555,0); WriteByte(0xAA);
		SetAddress(fx002AAA,0); WriteByte(0x55);
		SetAddress(fx005555,0); WriteByte(0xA0);
		SetAddress(addr,0);	WriteByte(data);
	}

    while (to--)
	{
		unsigned char s = ReadByte();
		if ((s & 128) == 0) return 0;	// ok
		if (s & 32)
		{
			int s = ReadByte();
			if ((s & 128) == 0) return 0;	// ok
			//SetAddress(addr,0);
			if (data == ReadByte()) return 0;	// ok
			//return err=1;	// fail
		}
		//if (data == ReadByte()) return 0;	// ok
	}

	SetAddress(addr,0);
	if (data == ReadByte()) return 0;	// ok
	if (reties == 0) return err=1;
	return Program(addr, data, reties-1);
}

//////////////////////////////////////////////////////////////////////////////
// ResetRead                                                                //
//////////////////////////////////////////////////////////////////////////////
void ResetRead()
{
    unsigned long addr=0;

    for (addr=0; addr<0x400000; addr+=0x200000)
	{
		SetAddress(fx005555,0); WriteByte(0xAA);
		SetAddress(fx002AAA,0); WriteByte(0x55);
		SetAddress(fx005555,0); WriteByte(0xF0);
	}
}

//////////////////////////////////////////////////////////////////////////////
// ChipErase                                                                //
//////////////////////////////////////////////////////////////////////////////
int ChipErase()
{
    unsigned long addr=0;

    for (addr=0; addr<0x400000; addr+=0x200000)
    {
        SetAddress(fx005555,0); WriteByte(0xAA);
        SetAddress(fx002AAA,0); WriteByte(0x55);
        SetAddress(fx005555,0); WriteByte(0x80);
        SetAddress(fx005555,0); WriteByte(0xAA);
        SetAddress(fx002AAA,0); WriteByte(0x55);
        SetAddress(fx005555,0); WriteByte(0x10);

		// wait for completion
		while (~ReadByte() & 0x80);

	    // data polling to tell when done
		/*unsigned char data = ReadByte();
		while (!(data & 0x80) && !(data & 0x20))
        {
			data = ReadByte();
		}
		if ((data & 0xA0) == 0xA0)
		{
			printf("Error erasing chip %c !\n", 'A'+(addr>>21));
			return err=1;
		}*/
    }
	return 0;
}

//////////////////////////////////////////////////////////////////////////////
// FileSize                                                                 //
//////////////////////////////////////////////////////////////////////////////
unsigned long FileSize(FILE *f)
{
    unsigned long size = 0;
	fseek(f, 0, SEEK_END);
    size = ftell(f);
	fseek(f, 0, SEEK_SET);
	return size;
}

//////////////////////////////////////////////////////////////////////////////
// DetectSize2                                                              //
//////////////////////////////////////////////////////////////////////////////
int DetectSize2(unsigned char *header, unsigned long addr)
{
   int i=0;

   SetAddress(addr,1);
   for (i=0; i<28; i++)
   {
      if(port_type == SPP_PORT_TYPE)
	     SetAddress(addr+i,0);
      if (header[i] != ReadByte()) 
		  return 1;
   }
   return 0;
}

//////////////////////////////////////////////////////////////////////////////
// DetectSize                                                               //
//////////////////////////////////////////////////////////////////////////////
unsigned long DetectSize()
{
   unsigned long size;

   //first, chec, DeviceID()
   BYTE devID;
   devID = DeviceID();
   if(devID == 0 || devID == 0x3F || devID == 0xFF)
      return 0;  //no cart present

   switch(devID)
   {
	   case 0xAB:  //4Mbit
		   return 0x080000;
	   case 0x2C:  //8Mbit
		   return 0x100000;
	   case 0x2F:  //16Mbit (or 32Mbit)
		   if(DeviceID(1) == 0x2F)
			   return 0x400000;
		   else
			   return 0x200000;
		   //break;  //go on for more testing
	   default:
		   break;  //go on for more testing
   }


   unsigned char prot;
   for(size = 0; size <= 0xF00000; size += 0x10000)
   {
	   prot = SectorProtVerify(size + 0x10000);

		if((prot & 0xFE) != 0)
		   break;
   }

   if(size >= 0x400000)
	   size = 0x400000;

   return size;
}

//////////////////////////////////////////////////////////////////////////////
// ActionInfo                                                               //
//////////////////////////////////////////////////////////////////////////////
int ActionInfo()
{
	unsigned long i;

	ResetRead();
	SetAddress(0x000000,1);

	// show recognition code
	printf("License string  : \"");
	for (i=0; i<28; i++) printf("%c", ReadByte());
	printf("\"\n");

	// show startup address
	i = ReadByte();
	i |= ReadByte()<<8;
	i |= ReadByte()<<16;
	i |= ReadByte()<<24;
    printf("Startup address : 0x%06lX\n", i);

	// show ID code's & color compatability
	i = ReadByte();
	i |= ReadByte()<<8;
    printf("ID code         : 0x%04lX\n", i);
    printf("ID sub-code     : 0x%02X\n", ReadByte());
	printf("Color compatable: %s\n", ReadByte() & 0x10 ? "yes" : "no");

	// show title
	printf("Title           : \"");
	for (i=0; i<16; i++) printf("%c", ReadByte());
	printf("\"\n");

	return 0;
}

//////////////////////////////////////////////////////////////////////////////
// ActionVerify                                                             //
//////////////////////////////////////////////////////////////////////////////
int ActionVerify()
{
	FILE *f = fopen(filename, "rb");
    unsigned long percent = (FileSize(f)-0x100)/100;
    unsigned long addr=0;

    if (!f) { /*printf("Can't open file for verify!\n");*/ return err=1; }

	ResetRead();

    for (addr=0; ; addr++)
	{
        int data = 0;
		if ((addr & 0xFF) == 0x00)
		{
            //printf("\rVerifying... 0x%06lX (%ld%%)", addr, addr/percent);
			SetAddress(addr,1);
		}
        data = fgetc(f);
		if (data == EOF) break;
		if (ReadByte() != data)
		{
			SetAddress(addr,0);
            //printf("\rVerify error at 0x%06lX (w:0x%02X, r:0x%02X)!\n", addr, data, ReadByte());
			return err=1;
		}
	}
	//printf("\n");

	fclose(f);
	return 0;
}

//////////////////////////////////////////////////////////////////////////////
// ActionBackup                                                             //
//////////////////////////////////////////////////////////////////////////////
int ActionBackup(char *filename)
{
	FILE *f = fopen(filename, "wb");
    unsigned long size = DetectSize();
	unsigned long percent = (size-0x100)/100;
    unsigned long addr=0;
	int i;

	f = fopen(filename, "wb");
	if (!f)
	{ 
		CString tempStr;
		tempStr.Format("Can't write to %s!", filename);
		AfxMessageBox(tempStr, MB_ICONEXCLAMATION);
		return err=1; 
	}

	ResetRead();

	// detect size

	// backup

	if(port_type == EPP_PORT_TYPE)
	{
		BYTE byteBuffer[0x100];
		for (addr=0; addr<size; addr+=sizeof(byteBuffer))
		{
			if(pProgressBar != NULL)
				pProgressBar->SetPos((int)(10.0*(double)addr/(double)percent));

			SetAddress(addr,1);//has to be done every 0x100 bytes
			for(i=0; i < sizeof(byteBuffer); i++)
				byteBuffer[i] = ReadByte();

			fwrite(byteBuffer, 1, sizeof(byteBuffer), f);
		}
	}
	else //SPP
	{
		BYTE byteBuffer[0x100];
		for (addr=0; addr<size; addr+=sizeof(byteBuffer))
		{
			//make this read 256 low nibbles and then 256 high nibbles and combine 'em
			if(pProgressBar != NULL)
				pProgressBar->SetPos((int)(10.0*(double)addr/(double)percent));

			SetAddress(addr,1);
			for(i=0; i < sizeof(byteBuffer); i++)
				byteBuffer[i] = spp_read_low_nibble();

			SetAddress(addr,1);
			for(i=0; i < sizeof(byteBuffer); i++)
				byteBuffer[i] |= spp_read_high_nibble();

			fwrite(byteBuffer, 1, sizeof(byteBuffer), f);
		}
	}

	fclose(f);
	return 0;
}

//////////////////////////////////////////////////////////////////////////////
// ActionWrite                                                              //
//////////////////////////////////////////////////////////////////////////////
int ActionWrite(char *filename)
{
	int bypassMode = 0;
	//  Game Name             Cart Name   Manufacturer ID  Device ID
    // 32M Pocket Flash Card   ------          0x98          0x2f
	/*if(ManufID() != 0x98 || DeviceID() != 0x2f)
	{
		AfxMessageBox("32M cart not found!");
		//return 1;
	}*/

	/* Bung carts report not writeable
	if(SectorProtVerify(0))  //if the first sector is protected
	{
		AfxMessageBox("This cart is not writeable.");
		return 1;
	}*/

	if(ManufID() == 0x01)
	{
		AfxMessageBox("AMD Chip Detected!\nUsing bypass mode.");
		bypassMode = 1;
	}


	FILE *f = fopen(filename, "rb");
    unsigned long percent = (FileSize(f)-0x100)/100;
    int erase = 1;
    int fast=0;
    unsigned long addr=0;

    if (!f) { AfxMessageBox("Can't open file to flash!"); 	fclose(f); return err=1; }

	ResetRead();

	// verify header
	if ((options & OPTION_YES) == 0)
	{
        int i=0;
        unsigned long check=0;

		fseek(f, 0x0, SEEK_SET);
        for (i=0; i<28; i++)
		{
			check = (check<<1) ^ fgetc(f);
		}

		if (check != 0xDC4446C4)	//" LICENSED BY SNK CORPORATION"
		if (check != 0xE8D446C4)	//"COPYRIGHT BY SNK CORPORATION"
		{
			AfxMessageBox("Invalid header found!");
			//if (tolower(getch()) != 'y') return err=1;
			fclose(f);
			return err=1;
		}
	}

	// check if it's the same cartridge

	// verify if same ID detected
	/*if ((options & OPTION_YES) == 0)
	{
		fseek(f, 0x20, SEEK_SET);
		SetAddress(0x20, 1);
		if (fgetc(f) == ReadByte())
		if (fgetc(f) == ReadByte())
		if (fgetc(f) == ReadByte())
		{
			//printf("Cartridge ID matches with file, erase first?\n");
            //if (tolower(getch()) != 'y') erase = 0;
		}
	}*/

	// erase chip first
	if (erase)
	{
        //printf("Erasing...\n");
		if (ChipErase())
		{
			AfxMessageBox("Chip Erase Error!");
			fclose(f);
			return err=1;
		}
	}


	ResetRead();

    fseek(f, 0, SEEK_SET);
    for (addr=0; ; addr++)
	{
        int data = 0;

        if ((addr & 255) == 0)
		{
			if(bypassMode)
				ResetBypass();
			SetAddress(addr,0);
			if(bypassMode)
				UnlockBypass(addr);

			//progress bar is from 0 to 1000
			if(pProgressBar != NULL)
				pProgressBar->SetPos((int)(10.0*(double)addr/(double)percent));
		}
        data = fgetc(f);
		//if (fast && (data == ReadByte())) continue;	// check same data in increment mode
		if (data == 0xFF) continue;			// nothing to write
		if (data == EOF) break;				// end of file
		fast=0;								// disable fast checking
		if (Program(addr, data, 3))
		{
			SetAddress(addr,0);
			AfxMessageBox("Programming error!");
			fclose(f);
			return err=1;
		}
	}

	fclose(f);
	return 0;
}

void ActionClearProgress()
{
	if(pProgressBar != NULL)
		pProgressBar->SetPos(0);
}


int ActionInit(CGradientProgressCtrl *pPBar)
{
	int err;
	pProgressBar = pPBar;

	/* WinIO
	err = InitializeWinIo();
	if(err == false)//returns false on error
	{
		//AfxMessageBox("InitializeWinIo returned ERROR");
		return 1;
	}*/

	//AfxMessageBox("About to call InitGBX");
	err = InitGBX();
	return err;
}

char *GetPortTypeString()
{
	switch (port_type)
	{
	case EPP_PORT_TYPE:
		return "EPP";
	case SPP_PORT_TYPE:
		return "SPP";
	case BAD_PORT_TYPE:
	default:
		return "Error";
	}
}

char *GetPortValString()
{
	static char portValStr[10];
	wsprintf(portValStr, "0x%03x", data_port);
	return portValStr;
}

void SetPortValString(char *portValStr)
{
	
	data_port = strtol(portValStr, NULL, 16);  //doesn't like the 0xHEX string
}

char *GetCartSizeString()
{
	static char cartSizeStr[15];
 
	unsigned long size;

	size = DetectSize();

	if(size >= 0x400000) //(ManufID() == 0x98 && DeviceID() == 0x2f)
	{
		wsprintf(cartSizeStr, "32Mb");
	}
	else if(size >= 0x200000) 
	{
		wsprintf(cartSizeStr, "16Mb");
	}
	else if(size >= 0x100000) 
	{
		wsprintf(cartSizeStr, " 8Mb");
	}
	else if(size >= 0x080000) 
	{
		wsprintf(cartSizeStr, " 4Mb");
	}
	else
	{
		wsprintf(cartSizeStr, "");
	}
	return cartSizeStr;
}

char *GetCartNameString()
{
	static char cartNameString[13];
	char checkString[18];
	//get the cart name (read in the first 0x100 bytes and grab the name)
	char byteBuffer[0x100];

	ResetRead();
	//first, check at offset 0x100000, because that's where the menu lives
	if(port_type == SPP_PORT_TYPE)
	{
		SetAddress(0x3FC000,1);
		for(int i=0; i < sizeof(byteBuffer); i++)
		{
			set_data_read;
			byteBuffer[i] = spp_read_low_nibble();
		}

		SetAddress(0x3FC000,1);
		for(i=0; i < sizeof(byteBuffer); i++)
		{
			set_data_read;
			byteBuffer[i] |= spp_read_high_nibble();
		}

		strncpy(checkString, &byteBuffer[0x0A], 18);
		strncpy(cartNameString, &byteBuffer[0x24], 12);
	}
	else
	{
		SetAddress(0x10000A,1);
		for(int i=0; i < 18; i++)
			checkString[i] = ReadByte();

		SetAddress(0x100024,1);
		for(i=0; i < 12; i++)
			cartNameString[i] = ReadByte();
	}

	if(strncmp(checkString,"BY SNK CORPORATION", 18) == 0)
	{
		cartNameString[12] = 0;
		return cartNameString;
	}

	if(port_type == SPP_PORT_TYPE)
	{
		SetAddress(0,1);
		for(int i=0; i < sizeof(byteBuffer); i++)
		{
			set_data_read;
			byteBuffer[i] = spp_read_low_nibble();
		}

		SetAddress(0,1);
		for(i=0; i < sizeof(byteBuffer); i++)
		{
			set_data_read;
			byteBuffer[i] |= spp_read_high_nibble();
		}

		strncpy(checkString, &byteBuffer[0x0A], 18);
		strncpy(cartNameString, &byteBuffer[0x24], 12);
	}
	else
	{
		SetAddress(0x0A,1);
		for(int i=0; i < 18; i++)
			checkString[i] = ReadByte();

		SetAddress(0x24,1);
		for(i=0; i < 12; i++)
			cartNameString[i] = ReadByte();
	}

	if(strncmp(checkString,"BY SNK CORPORATION", 18) != 0)
	{
		sprintf(cartNameString, "");
	}

	cartNameString[12] = 0;
	return cartNameString;
}

void ActionShutdown()
{
	DeInitGBX();
	// WinIO ShutdownWinIo();
}

//////////////////////////////////////////////////////////////////////////////
// main                                                                     //
//////////////////////////////////////////////////////////////////////////////
/*int old_main(int argc, char **argv)  //*argv[])
{
    int a=0;

	// title
    printf("POKELINK by Dark Fader / BlackThunder\n");
    printf("  Modified for win95/98/linux by Jeff Frohwein.\n");
    printf("  This version might not work on Win2000 or NT.\n\n");

	// parse command line
    for (a=1; a<argc; a++)
	{
		if ((argv[a][0] == '/') || (argv[a][0] == '-'))
		{
			// option
			switch (tolower(argv[a][1]))
			{
				case '?':
                case 'h':
				{
					printf("Syntax: PokeLink [<option(s)>] [<filename>]\n");
					printf("\n");
					printf("  /?           Shows this help screen.\n");
                    printf("  /h           Shows this help screen.\n");
                    printf("  /p<port>     Specified the port to use. Default = 0x378.\n");
					printf("                 (overridden by GBXPORT environment setting)\n");
					printf("  /w           Write & verify the specified file.\n");
					printf("  /v           Verify the specified file.\n");
					printf("  /i           Displays cartridge info.\n");
					printf("  /b           Backs up cartridge.\n");
					printf("  /y           Use YES on all questions.\n");
//                    printf("  /e           Pause on errors.\n");
					printf("  <filename>   File to write, backup or/and verify.\n");
					printf("\n");
					printf("For example: \"PokeLink /b /i /v /w mygame.ngp\" would\n");
					printf("  flash mygame.ngp, display info, back-up and then verify.\n");
					printf("  Quite useless example, but it shows the sequence.\n");
					printf("\n");
					return 0;
				}

				case 'p': { port = strtoul(&argv[a][2], NULL, 10); break; }
				case 'i': { options |= OPTION_INFO; break; }
				case 'b': { options |= OPTION_BACKUP; break; }
				case 'w': { options |= OPTION_WRITE; break; }
				case 'v': { options |= OPTION_VERIFY; break; }
				case 'y': { options |= OPTION_YES; break; }

				default:
				{
					printf("Warning: Invalid option '%c'!\n", argv[a][1]);
					break;
				}
			}
		}
		else
		{
			// filename
			filename = argv[a];
			printf("Filename: \"%s\".\n", filename);
		}
	}

#ifdef __linux__
    atexit(clear_sig);
    old = signal(SIGINT, control_c);
    if (getuid() && geteuid()) {
        printf("ERROR: No privileges to use I/O\n");
        exit(1);
    }
    ioperm(port, 5, 1);
    if (getgid() && !getegid()) {
        setegid(getgid());
    }
    if (getuid() && !geteuid()) {
        seteuid(getuid());
    }
#else
    // environment settings
	if (getenv("GBXPORT"))
	{
		port = strtoul(getenv("GBXPORT"), NULL, 10);
	}

	// Init GiveIO
//    if (InitGiveIO()) { printf("GiveIO driver could not be opened!\n"); return err=1; }

#endif

    // Check for Flash Linker at {GBXPORT} or 0x378
	if (InitGBX()) { printf("Flash Linker could not be found at port 0x%3X!\nPlease check the connection and that it's turned on.\n", port); return err=1; }
//    atexit(DeInitGBX);

    printf("Manufacturer ID = 0x%x, Device ID = 0x%x\n\n", ManufID(), DeviceID());

	// Check for cartridge
	if (DeviceID() == 0x00) { printf("No cartridge detected!\n"); return err=1; }

	// Options to perform
    if (options & OPTION_WRITE)
       if (ActionWrite()) return err=1;
    if (options & OPTION_INFO)
       if (ActionInfo()) return err=1;
    if (options & OPTION_BACKUP)
       if (ActionBackup()) return err=1;
    if (options & OPTION_VERIFY)
       if (ActionVerify()) return err=1;

printf("[4]\n");

    DeInitGBX();

printf("[5]\n");

	return 0;
}*/